; #AutoIt3Wrapper_Au3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w- 7

; = Info ===========================================================================================
; Title           : CallTipViewer.au3
; --------------------------------------------------------------------------------------------------
;
; Version (vnots) : 0.0.330
; Date            : 2021-04-06
; Author(s)       : Professor Bernd
;
; Requirements    :
; Language        :
; Description     : (siehe unten)
;
; Contributors    : alpines
;
; Credits         : Many thanks to alpines, for developing the routine
;                   "GetFuncInfoFromReceivedText()", advice and support and
;                   tireless testing of the CallTipViewer.
;
;                   Ein herzliches Dankeschön geht an alpines, für das
;                   Entwickeln der Routine "GetFuncInfoFromReceivedText()",
;                   Rat und Tat und unermüdliches Testen des CallTipViewers.
; ==================================================================================================


#cs ----------------------------------------------------------------------------

  AutoIt Version: 3.3.14.5

  CallTipViewer: Ein CallTip-Fenster anzeigen.

  Mit CallTip ist ein schwebendes, Tooltip-ähnliches Fenster gemeint, ohne
  Rahmen und Titelleiste. Wenn der Cursor (Caret) im Editor hinter der
  öffnenden Klammer einer Funktion steht, kann der CallTip aufgerufen werden,
  indem der Shortcut Shift+Ctrl+Space gedrückt wird. Der CallTip zeigt dann
  die Syntax und die Beschreibung zu der jeweiligen Funktion. Der aktuelle
  Parameter, in dem das Caret steht, wird rot hervorgehoben.

  Das CallTip-Fenster bekommt jedoch nicht den Fokus. Deshalb kann man im
  Editor weiterschreiben, während der CallTip angezeigt wird.

  Diese kurze Beschreibung eingefügt am 2020-03-17, 2020-04-23.


  Bekannte Grenzen:

  - Verzögerung von ein paar Sekunden beim ersten Start von PSPad, bis der
      CallTip-Shortcut reagiert.

      Auf manchen PCs vergehen nach dem Start von CallTipViewer 2 - 3 Sekunden,
      bis der CallTip-Shortcut reagiert und den CallTip zeigt. Vermutlich hat
      es mit der PC-Hardware, dem Windows, und vorallem mit dem Viren-Scanner
      zu tun. Deshalb lasse ich den CallTipViewer laufen, damit die Wartezeit
      nur 1x (beim ersten Start von PSPad) auftritt. 2020-04-24.

  - Die Schrift und -Größe können nur unzuverlässig aus der PSPad.ini ausgelesen
      werden. Mir ist nicht wirklich klar, wann genau PSPad die Schrift-Daten
      speichert, z. B. wenn mehrere PSPad-Instanzen, und/oder mehrere Tabs
      geöffnet sind. 2020-04-23.

  - CallTip wird nicht angezeigt, wenn in PSPad der automatische Zeilenumbruch
      eingeschaltet ist. 2020-04-23.
#ce ;---------------------------------------------------------------------------


#NoTrayIcon
#include <Array.au3> ; Für "CallTip_OUDF_Routines"
#include <AutoItConstants.au3> ; Für z. B. Number().
#include <ColorConstants.au3>
#include <EditConstants.au3>
#include <File.au3>
#include <FileConstants.au3>
#include <FontConstants.au3>
#include <GUIConstantsEx.au3>
#include ".\misc\IROT.au3" ; ROT (Running Object Table)
#include <Misc.au3> ; Für "_Singleton()."
#include <MsgBoxConstants.au3>
#include <StaticConstants.au3>
#include <String.au3>
#include <StringConstants.au3>
#include ".\misc\StringSize.au3"
#include <WinAPIEx.au3>
#include <WinAPILocale.au3>
#include <WinAPISys.au3> ; Für _WinAPI_GetAsyncKeyState(), _WinAPI_GetWorkArea().
#include <WinAPISysInternals.au3> ; <= wirklich benötigt?
#include <WinAPISysWin.au3>
#include <WindowsConstants.au3>

#include <Date.au3>

_Singleton("PSPad4AutoIt3_CallTipViewer")

Opt("MustDeclareVars", 1)

Opt("GUIResizeMode", $GUI_DOCKALL)

OnAutoItExitRegister('OnAutoItExit')

Global $g_hPSPad      = Null
Global $g_hCallTipWin = Null
Global $g_hChildGui   = Null
Global $g_hDLL = DllOpen("user32.dll")

Global $g_LblSyntax, $g_Lbl_Red, $g_LblDescr
Global $g_LblUp, $g_LblDown, $g_LblLeft, $g_LblRight

Global $g_sFontName
Global $g_fFontSize
Global $g_sFontName_Old
Global $g_fFontSize_Old
Global $g_bFontHasChanged = False

Global $g_hexDescriptionColor_Au3Api
Global $g_hexDescriptionColor_OUDF

Global Const $g_iOffsetX = 2
Global Const $g_iOffsetY = 2
Global $g_iLineHeight

Global $g_oDict_Au3Api_SynSets  ; Syntax-Sets aus au3.api (Syntax und Description).
Global $g_oDict_OUDF_SynSets    ; Syntax-Sets für OUDFs (Syntax und Datei-Pfad).
Global $g_oDict_OUDF_Au3Paths   ; Pfade zu den au3 Dateien, aus denen OUDFs ausgelesen wurden.
Global $g_oDict_OUDF_Au3Dirs    ; Ordner-Pfade, in denen die ausgelesenen au3 Dateien liegen (Monitoring).

Global $g_COM_ROT_Handle        ; Handle mit dem das COM Objekt im ROT (Running Object Table) registriert ist.
Global $g_oDict_COM_CallTip     ; COM ROT Dictionary Object zur Kommunikation mit AutoIt3_CallTip.vbs.

Global $g_iID = 0
Global $g_iWinMsg
Global $g_bOUDF_DictsWereEmptied = False ; Flag zum Erkennen, ob OUDF Dictionaries geleert wurden.

Global $bDebugMode = False      ; Wenn True, werden zusätzliche Controls als Debug-Helfer gezeigt.

Global $g_sCaretPosCoordinates  ; Temporär: Während des Umzugs von Funktionen vom VBScript nach hier

Main_Function()


; ==============================================================================
; Main_Function
;
;
; ------------------------------------------------------------------------------
Func Main_Function()

  Local $sTitel_CallTipWin                          ; Der Titel für CallTipViewer" wird im VBScript
  Local $sCOM_ROT_Name                              ; Name mit dem das COM Objekt im ROT
  Local $sChkShowHideTxt, $sChkShowHideTxt_Old      ; Text aus Label der CheckBox.
  Local $sFunctionName, $sFunctionName_Old = ""     ; Name der Funktion, für die der CallTip angezeigt wird.
  Local $sReceivedTxt = "", $sReceivedTxt_Old = ""  ; Vom VBScript gesendeter Text im ReceiverEdit.
  Local $sInpCurrentAu3Path, $sInpCurrentAu3Path_Old; Vom VBScript gesenderer Text im CurrentAu3Path-Input.
  Local $sCurrentSyntaxSet                          ; Aktuelles Syntax-Set (Syntax + Description) als String.

  Local $iFuncNameFirstCharPosX_Intern              ; PosX intern des ersten Zeichens von FuncName.
  Local $iFuncNameFirstCharPosX_Intern_Old
  Local $iFuncNameFirstCharPosX_Extern              ; PosX extern des ersten Zeichens von FuncName.
  Local $iFuncNameFirstCharPosX_Extern_Old
  Local $iFuncNameFirstCharPosY_Extern              ; PosY extern des ersten Zeichens von FuncName.
  Local $iParamNum                                  ; Nummer des Params, in dem das Caret steht.
  Local $iSynSetsCount                              ; Anzahl der Syntax-Sets zu jeweils einem FuncName.
  Local $iSynSetIdx = 1, $iSynSetIdx_Old            ; Index des aktuellen Syntax-Sets zu FuncName.
  Local $iScrollPosX = 0                            ; Zum Scrollen bei zu langen Texten der Syntax-Labels.
  Local $iChildGuiWidth, $iChildGuiHeight

  Local $iArrowLen                                  ; Die Arrow-Labels werden als Buttons benutzt, zum vertikalen
  Local $iAdjustX_Arrw                              ; Hilfsvariable zum Anpassen der PosX von CallTipWin und

  Local $tRECT
  Local $iScreenWA_Left
  Local $iScreenWA_Top
  Local $iScreenWA_Width
  Local $iScreenWA_Height

  Local $aThreadInfo                                ; Für Rückgabe von "GetGUIThreadInfo()()".
  Local $aCallTipRect[4]                            ; Position und Maße des CallTipWin.
  Local $aAllSyntaxSetsOfAFuncName                  ; Array mit allen Syntax-Sets zu FuncName.
  Local $aCurrentSyntaxSet_Parts                    ; Array mit aktuellem Syntax-Set, aufgeteilt in SyntaxPart-L, -M, -R und Descr.

  Local $bShowCallTip = False                       ; Variable zum Vermindern von ROT Dict Zugriffen. 2021-03-19.
  Local $bCallTipHasChanged = False                 ; Für Rückgabe von "SetTextAndPosFor_SyntaxLabels()".
  Local $bFuncNameHasChanged = False                ; Benötigt für SetTextAndPosFor_SyntaxLabels().
  Local $bScrollingOfSyntaxSetsAllowed = False      ; Für GetFuncInfoFromReceivedText().
  Local $bCallTipWasCalledBySendCommand = False     ; Wird gesetzt wenn der Shortcut zum Anzeigen des CallTips an PSPad gesendet wird.
  Local $bKeysForLeftParenthesisWerePressed = False ; Wird gesetzt beim automatischen Erkennen wenn "(" geschieben wird.

  CheckAbortConditionsAtProgrammStart($sTitel_CallTipWin, $sCOM_ROT_Name)

  $g_sFontName = ReadFromPSPad_INI("Font", "Name")
  $g_fFontSize = Number(ReadFromPSPad_INI("Font", "Size"))
  If $g_sFontName = "" Then $g_sFontName = "Courier New"
  If $g_fFontSize = 0  Then $g_fFontSize = 10

  $g_sFontName_Old = $g_sFontName
  $g_fFontSize_Old = $g_fFontSize

  $iArrowLen = $g_fFontSize * 1.4

  $g_iLineHeight = _StringSize(" ", $g_fFontSize, Default, Default, $g_sFontName)[1] ; Zeilenhöhe

  CreateGuisAndControls($iArrowLen, $sTitel_CallTipWin)

  ReadAu3Api_ToDictionary() ; $g_oDict_Au3Api_SynSets

  OUDF_InitializeDictionaries()

  $g_iWinMsg = _WinAPI_RegisterWindowMessage('SHELLCHANGENOTIFY')
  GUIRegisterMsg($g_iWinMsg, 'OUDF_WM_SHELLCHANGENOTIFY')

  $g_COM_ROT_Handle = ROT_RegisterObject(Default, $sCOM_ROT_Name) ; Default => Object = Dictionary object
  $g_oDict_COM_CallTip = ObjGet($sCOM_ROT_Name) ; Dictionary object
  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "CallTipViewer", Function "Main_Function":' _
      & @CRLF & @CRLF & 'Error while retrieving the COM ROT Dictionary Object.')
    Exit(10) ; 10 = Fehler beim Abrufen des Dictionary Objekts.
  Else
    $g_oDict_COM_CallTip.CompareMode = 1 ; (0 = BinärMode (default), 1 = Textmode)
  EndIf ; If @error

  $g_oDict_COM_CallTip.Add("CaretPosCoordinates", "")     ; Interne und externe Koordinaten des Carets in PSPad.
  $g_oDict_COM_CallTip.Add("CaretPosExtern", "")          ; Externe Koordinaten des Carets in PSPad.
  $g_oDict_COM_CallTip.Add("CheckColorScheme", True)      ; Zum Erkennen, ob das CallTip Color Scheme überprüft werden soll.

  $g_oDict_COM_CallTip.Add("LinesLeftAndAbove", "")       ; Text in den Zeilen links und über dem Caret in PSPad.
  $g_oDict_COM_CallTip.Add("PSPadEditorPath", "")         ; Pfad zur Datei im aktiven PSPad-Editor.
  $g_oDict_COM_CallTip.Add("ShowCallTip", False)          ; Zum Erkennen, ob das CallTipWin gezeigt/versteckt werden soll.

  While 1 ; Haupt-Schleife

    If _IsPressed("10", $g_hDLL) And _IsPressed("11", $g_hDLL) _
      And _IsPressed("12", $g_hDLL) And _IsPressed("59", $g_hDLL) Then ExitLoop

    If _IsPressed("1B", $g_hDLL) Then ToolTip("")

    $aThreadInfo = GetGUIThreadInfo()

    $sReceivedTxt = $g_oDict_COM_CallTip.Item("LinesLeftAndAbove")

    $g_sCaretPosCoordinates = $g_oDict_COM_CallTip.Item("CaretPosCoordinates")

    $bShowCallTip = $g_oDict_COM_CallTip.Item("ShowCallTip")

    If $bShowCallTip = False Then

      If Not OwnPSPadWinExists() Then ExitLoop

      If BitAND(WinGetState($g_hCallTipWin), $WIN_STATE_VISIBLE) _
        Or $sReceivedTxt_Old <> $sReceivedTxt Then

        $sReceivedTxt_Old = $sReceivedTxt

        GUISetState(@SW_HIDE, $g_hCallTipWin)

        $g_hPSPad = Null

        $bScrollingOfSyntaxSetsAllowed = False ; Neu 2020-05-25.

        ResetOldVariables($sChkShowHideTxt_Old, $sInpCurrentAu3Path_Old, _
          $iFuncNameFirstCharPosX_Intern_Old)

      EndIf ; If BitAND(WinGetState($g_hCallTipWin), $WIN_STATE_VISIBLE) Or ...

      If KFLP_KeysForLeftParenthesisWerePressed() Then
        If _WinAPI_GetClassName($aThreadInfo[1]) = 'TfPSPad' Then ; $aThreadInfo[1] = Active Win.
          If Not $bCallTipWasCalledBySendCommand Then
            Send("+^{SPACE}") ; Shift+Ctrl+Space => "CallTip_Show()" im VBScript.
            $bCallTipWasCalledBySendCommand = True
          EndIf
        EndIf
      EndIf
    Else ; ---- CheckBox "ShowHide" ist angehakt. ----

      $bShowCallTip = ClosePSPadErrMsg_IfPSPadIsClosedWhileCallTipIsRunning()

      If $bShowCallTip = True Then $bShowCallTip = CheckHidingConditions($aThreadInfo)

      $sChkShowHideTxt = $g_sCaretPosCoordinates

      $sInpCurrentAu3Path = $g_oDict_COM_CallTip.Item("PSPadEditorPath")

      ScrollSynSetsByArrowLabels($iAdjustX_Arrw, $iChildGuiWidth, $aCallTipRect, _
        $iSynSetsCount, $iSynSetIdx, $iScrollPosX)

      ScrollSynSetsByKeyboard($iSynSetsCount, $iSynSetIdx, $iScrollPosX, _  ; Neu 2020-05-25.
        $bScrollingOfSyntaxSetsAllowed)

      If $sInpCurrentAu3Path_Old <> $sInpCurrentAu3Path Then OUDF_EmptyAllDictionaries()

      If ($sChkShowHideTxt_Old <> $sChkShowHideTxt) Or ($iSynSetIdx_Old <> $iSynSetIdx) _
        Or ($sInpCurrentAu3Path_Old <> $sInpCurrentAu3Path) Or $g_bOUDF_DictsWereEmptied Then

        $sChkShowHideTxt_Old = $sChkShowHideTxt
        $iSynSetIdx_Old = $iSynSetIdx
        $sInpCurrentAu3Path_Old = $sInpCurrentAu3Path
        $g_bOUDF_DictsWereEmptied = False

        OUDF_ReadFromFileAndIncludes($sInpCurrentAu3Path)

        GetFuncInfoFromReceivedText($sReceivedTxt, $sFunctionName, $iParamNum, _
          $iFuncNameFirstCharPosX_Intern, $aAllSyntaxSetsOfAFuncName)

        If $sFunctionName = "" Then

          $g_oDict_COM_CallTip.Item("ShowCallTip") = False

          If $bCallTipWasCalledBySendCommand Then

            $bCallTipWasCalledBySendCommand = False

            $sChkShowHideTxt_Old = ""
          Else
            OUDF_ShowToolTipForSavingTheFile($sReceivedTxt, $sChkShowHideTxt_Old)
          EndIf
        Else ; ---- Funktionsname IST gültig. ----

            $bCallTipWasCalledBySendCommand = False

          $iSynSetsCount = $aAllSyntaxSetsOfAFuncName[0]

          If $sFunctionName_Old <> $sFunctionName Then

            $sFunctionName_Old = $sFunctionName
            $bFuncNameHasChanged = True ; Wird benötigt für SetTextAndPosFor_SyntaxLabels().

            $iSynSetIdx = 1
            $iScrollPosX = 0

            $bScrollingOfSyntaxSetsAllowed = False
          EndIf

          $sCurrentSyntaxSet = $aAllSyntaxSetsOfAFuncName[$iSynSetIdx]

          $tRECT = _WinAPI_GetWorkArea()
          $iScreenWA_Left   = DllStructGetData($tRECT, 'Left')
          $iScreenWA_Top    = DllStructGetData($tRECT, 'Top')
          $iScreenWA_Width  = DllStructGetData($tRECT, 'Right') - $iScreenWA_Left
          $iScreenWA_Height = DllStructGetData($tRECT, 'Bottom') - $iScreenWA_Top

          CheckParamNum($sCurrentSyntaxSet, $iParamNum)

          $aCurrentSyntaxSet_Parts = SplitSyntaxAndDescription($sCurrentSyntaxSet, $iParamNum)

          $bCallTipHasChanged = SetTextAndPosFor_SyntaxLabels($aCurrentSyntaxSet_Parts, $iParamNum, _
            $bFuncNameHasChanged, $iChildGuiWidth, $iChildGuiHeight)        ; => 2x raus.

          $bFuncNameHasChanged = False ; Zurücksetzen.

          GetFuncNameFirstCharPos_Extern($iFuncNameFirstCharPosX_Intern, _  ; <= 1x rein.
            $iFuncNameFirstCharPosX_Extern, $iFuncNameFirstCharPosY_Extern) ; => 2x raus.

          If $bCallTipHasChanged _
            Or ($iFuncNameFirstCharPosX_Intern_Old <> $iFuncNameFirstCharPosX_Intern) _
            Or ($iFuncNameFirstCharPosX_Extern_Old <> $iFuncNameFirstCharPosX_Extern) Then

            $iFuncNameFirstCharPosX_Intern_Old = $iFuncNameFirstCharPosX_Intern
            $iFuncNameFirstCharPosX_Extern_Old = $iFuncNameFirstCharPosX_Extern

            $iAdjustX_Arrw = SetStateAndPosFor_ArrowLabels($iSynSetsCount, $iArrowLen, _
              $iChildGuiWidth, $iScreenWA_Width)

            WinMove($g_hChildGui, "", _
              $iAdjustX_Arrw + $iScrollPosX, 0, $iChildGuiWidth, $iChildGuiHeight)

            $aCallTipRect = CalculateCallTipRect($iAdjustX_Arrw, $iChildGuiWidth, $iChildGuiHeight, _
              $iScreenWA_Left, $iScreenWA_Top, $iScreenWA_Width, $iScreenWA_Height, _
              $iFuncNameFirstCharPosX_Intern, $iFuncNameFirstCharPosX_Extern, $iFuncNameFirstCharPosY_Extern)

            _WinAPI_SetWindowPos($g_hCallTipWin, $HWND_TOPMOST, $aCallTipRect[0], $aCallTipRect[1], _
              $aCallTipRect[2], $aCallTipRect[3], $SWP_NOACTIVATE)

            If $bDebugMode Then
              WinMove($g_hCallTipWin, "", $aCallTipRect[0], $aCallTipRect[1], 1216, 240)
            EndIf

            GUISetState(@SW_SHOWNOACTIVATE, $g_hCallTipWin)

          EndIf ; If $bCallTipHasChanged Or ($iFuncNameFirstCharPosX_Intern_Old <> $iFuncNameFirstCharPosX_Intern) Or ...
        EndIf ; If $sFunctionName = ""
      Else
        If Not BitAND(WinGetState($g_hCallTipWin), $WIN_STATE_VISIBLE) Then
          $g_oDict_COM_CallTip.Item("ShowCallTip") = False
        EndIf
      EndIf ; If ($sChkShowHideTxt_Old <> $sChkShowHideTxt) Or ($iSynSetIdx_Old <> $iSynSetIdx) _ ...
    EndIf ; If $g_oDict_COM_CallTip.Item("ShowCallTip") = False

    Switch GUIGetMsg()

      Case $GUI_EVENT_CLOSE
        ExitLoop ; ("ExitLoop" ermöglicht die Ausführung von OnAutoItExit()!)

    EndSwitch ; Windows-Events auswerten.
  WEnd ; ==> Haupt-Schleife - Ende

  DllClose($g_hDLL)

  GUIDelete($g_hChildGui)
  GUIDelete($g_hCallTipWin)
EndFunc ; ==> Main_Function


; ==============================================================================
; CalculateCallTipRect
;
; Routine zum Berechnen der Position und Maße für das CallTipWin.
;
; Rückgabe: Array mit den 4 Rect-Werten
;   [0]: Left
;   [1]: Top
;   [2]: Width
;   [3]: Height
; ------------------------------------------------------------------------------
Func CalculateCallTipRect($_iAdjustX_Arr, $_iChildGuiWidth, $_iChildGuiHeight, _
  $_iScreenWA_Left, $_iScreenWA_Top, $_iScreenWA_Width, $_iScreenWA_Height, _
  $_iFuncNameFirstCharPosX_Intern, _
  $_iFuncNameFirstCharPosX_Extern, $_iFuncNameFirstCharPosY_Extern)

  Local $aCallTipRect[4] ; Array für Postition und Maße des CallTipWins.
  Local $iLinesCnt = 1   ; Anzahl der Zeilen von Oberkannte FuncName bis zur Zeile,

  $aCallTipRect[0] = $_iFuncNameFirstCharPosX_Extern - $_iAdjustX_Arr              ; Left

  $aCallTipRect[1] = _
    $_iFuncNameFirstCharPosY_Extern + ($g_iLineHeight * $iLinesCnt) + $g_iOffsetY  ; Top

  $aCallTipRect[2] = $_iChildGuiWidth + $_iAdjustX_Arr                            ; Width

  $aCallTipRect[3] = $_iChildGuiHeight                                            ; Height

  If $aCallTipRect[2] > $_iScreenWA_Width Then $aCallTipRect[2] = $_iScreenWA_Width - 4

  If $aCallTipRect[0] < $_iScreenWA_Left Then $aCallTipRect[0] = $_iScreenWA_Left + 2

  If ($aCallTipRect[0] + $aCallTipRect[2]) > $_iScreenWA_Width Then
    $aCallTipRect[0] = $_iScreenWA_Width - $aCallTipRect[2] - 2
  EndIf

  If ($aCallTipRect[1] + $aCallTipRect[3]) > ($_iScreenWA_Top + $_iScreenWA_Height) Then
    $aCallTipRect[1] = _
      $aCallTipRect[1] - $aCallTipRect[3] - $g_iLineHeight - (2 * $g_iOffsetY)
  EndIf

  Return $aCallTipRect
EndFunc ; ==> CalculateCallTipRect


; ==============================================================================
; CheckAbortConditionsAtProgrammStart
;
; Routine zum Prüfen von Bedingungen, die den Start des CallTipViewers
; verhindern.
;
; Rückgabe in $_sTitel_CallTipWin und $_sCOM_ROT_Name (ByRef).
; ------------------------------------------------------------------------------
Func CheckAbortConditionsAtProgrammStart(ByRef $_sTitel_CallTipWin, ByRef $_sCOM_ROT_Name)

  If $CmdLine[0] > 0 Then
    $_sTitel_CallTipWin = $CmdLine[1]
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "' & @ScriptName & '"'  & @CRLF _
      & 'Function "CheckAbortConditionsAtProgrammStart"' & @CRLF & @CRLF _
      & 'No title was transmitted.')
    Exit(2) ; 2 = Es wurde kein Titel für das CallTipWin übermittelt.
  EndIf

  If $CmdLine[0] > 1 Then
    $_sCOM_ROT_Name = $CmdLine[2]
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "' & @ScriptName & '"'  & @CRLF _
      & 'Function "CheckAbortConditionsAtProgrammStart"' & @CRLF & @CRLF _
      & 'No COM ROT name was transmitted.')
    Exit(3) ; 3 = Es wurde kein Name für das COM ROT Objekt (Dictionary) übermittelt.
  EndIf

EndFunc ; ==> CheckAbortConditionsAtProgrammStart


; ==============================================================================
; CheckHidingConditions
;
; Routine zum Prüfen, ob das CallTipWin versteckt werden soll.
;
;   Parameter "$_aThreadInfo[]":
;
;     [0] - The thread state. It can be one or more of the $GUI_* values.
;     [1] - A handle to the active window within the thread.
;     [2] - A handle to the window that has the keyboard focus.
;     [3] - A handle to the window that has captured the mouse.
;     [4] - A handle to the window that owns any active menus.
;     [5] - A handle to the window in a move or size loop.
;     [6] - A handle to the window that is displaying the caret.
;     [7] - The x-coordinate of the upper-left corner of the caret's bounding rectangle, in screen coordinates!
;     [8] - The y-coordinate of the upper-left corner of the caret's bounding rectangle, in screen coordinates!
;     [9] - The width of the caret's bounding rectangle.
;     [10] - The height of the caret's bounding rectangle.
; ------------------------------------------------------------------------------
Func CheckHidingConditions($_aThreadInfo)

  Local $bShowCallTip = True
  Local $bHideCallTip = False
  Local Const $cnHWND_ActiveWin = 1, $cnHWND_ActiveCtrl = 2, $cnHWND_Menu = 4, $cnHWND_MoveSize = 5

  If _WinAPI_GetClassName($_aThreadInfo[$cnHWND_ActiveWin]) <> 'TfPSPad' Then
    $bHideCallTip = True
    $g_hPSPad = Null
  Else ; Aktives Fenster ist ein PSPad-Fenster.

    If ($g_hPSPad <> Null) And ($g_hPSPad <> $_aThreadInfo[$cnHWND_ActiveWin]) Then
      $bHideCallTip = True
      $g_hPSPad = Null
    Else
      $g_hPSPad = $_aThreadInfo[$cnHWND_ActiveWin]
    EndIf
  EndIf

  If _WinAPI_GetClassName($_aThreadInfo[$cnHWND_ActiveCtrl]) <> "TPSSynEdit" Then $bHideCallTip = True

  If $_aThreadInfo[$cnHWND_Menu] <> 0 Then $bHideCallTip = True

  If $_aThreadInfo[$cnHWND_MoveSize] <> 0 Then $bHideCallTip = True

  If _IsPressed("1B", $g_hDLL) Then $bHideCallTip = True

  If $bHideCallTip = True Then
    $g_oDict_COM_CallTip.Item("ShowCallTip") = False
    $bShowCallTip = False
  EndIf

  Return $bShowCallTip
EndFunc ; ==> CheckHidingConditions


; ==============================================================================
; CheckParamNum     (vormals "MoreCommasInReceivedTextThanInSyntaxText()")
;
; Routine zum Zählen der Kommas im Syntax-Text. Wenn mehr Kommas im ReceivedTxt
; sind als im Syntax-Text, kann der CallTip entsprechend angepasst werden. Die
; Kommas im ReceivedTxt wurden in "GetFuncInfoFromReceivedText()" ermittelt,
; und "Func in Func", Quotes, Kommentare, usw. schon ausgefiltert. 2020-05-25.
;
; Rückgabe in $_iParamNum (ByRef):
;
;   Wenn im ReceivedTxt NICHT mehr Kommas sind, als im Syntax-Text, wird
;   $_iParamNum nicht verändert.
;
;   Wenn zwar mehr Kommas im ReceivedTxt sind, als im Syntax-Text, aber der
;   Syntax-Text als letzten Parameter einen "Param n" hat, wird die ParamNum
;   auf die (Gesamt-)Anzahl der Params im Syntax-Text gesetzt. 2020-05-24.
;
;   Wenn mehr Kommas im ReceivedTxt sind, als im Syntax-Text und der Syntax-Text
;   als letzten Parameter KEINEN "Param n" hat, wird die ParamNum auf 0 gesetzt.
;   Dadurch kann in "SetTextAndPosFor_SyntaxLabels()" dem CallTip eine Meldung
;   "Too many Params!" statt der Syntax zugewiesen werden. 2020-05-25.
;
; Als "Param n" bisher gefunden:
;
;  "[, param_N]", "[, type n, param n]" und "[, value n]".
; ------------------------------------------------------------------------------
Func CheckParamNum($_sSyntaxAndDescription, ByRef $_iParamNum)

  Local $bSingleQuoteFound = False
  Local $bDoubleQuoteFound = False

  Local $asTmpText
  Local $iCommaCntIn_ReceivedTxt = 0
  Local $iCommaCntIn_SyntaxTxt = 0

  Local $iPos_CB = 0 ; closing bracket - Zum Prüfen, ob die Syntax "Param n" enthält.

  $asTmpText = StringSplit($_sSyntaxAndDescription, "") ; 1-basierendes Array, [0]: Anzahl Elemente (Zeichen).

  For $i = 1 To $asTmpText[0]

    If ($asTmpText[$i] = "'" ) Then     ; Zeichen ist ein "öffnendes" SingleQuote.
      If Not $bDoubleQuoteFound Then    ; SingleQuotes nur berücksichtigen, falls nicht schon ein "öffnendes" DoubleQuote gefunden.
        If Not $bSingleQuoteFound Then  ; Falls noch kein "öffnendes" SingleQuote gefunden war, ...
          $bSingleQuoteFound = True     ; ... dann Variable setzen, ...
        Else
          $bSingleQuoteFound = False    ; ... ansonsten Variable zurücksetzen.
        EndIf
      EndIf
    ElseIf ($asTmpText[$i] = '"' ) Then ; Zeichen ist ein "öffnendes" DoubleQuote.
      If Not $bSingleQuoteFound Then    ; DoubleQuote nur berücksichtigen, falls nicht schon ein "öffnendes" SingleQuote gefunden.
        If Not $bDoubleQuoteFound Then  ; Falls noch kein "öffnendes" DoubleQuote gefunden war, ...
          $bDoubleQuoteFound = True     ; ... dann Variable setzen, ...
        Else
          $bDoubleQuoteFound = False    ; ... ansonsten Variable zurücksetzen.
        EndIf
      EndIf
    EndIf

    If Not $bSingleQuoteFound And Not $bDoubleQuoteFound Then
      If $asTmpText[$i] = ")" Then
        $iPos_CB = $i ; Zum Prüfen weiter unten, ob die Syntax "Param n" enthält.
        ExitLoop
      EndIf
      If $asTmpText[$i] = "," Then $iCommaCntIn_SyntaxTxt += 1
    EndIf
  Next

  $iCommaCntIn_ReceivedTxt = $_iParamNum - 1

  If $iCommaCntIn_ReceivedTxt > $iCommaCntIn_SyntaxTxt Then

    Local $sPnStr = " n]" ; Kennzeichen-String.                                 2020-06-19.
    Local $iPnPos = StringInStr($_sSyntaxAndDescription, $sPnStr)
    If $iPnPos = 0 Then
      $sPnStr = "_n]"
      $iPnPos = StringInStr($_sSyntaxAndDescription, $sPnStr)
    EndIf
    If $iPnPos = 0 Then
      $sPnStr = "var32]"
      $iPnPos = StringInStr($_sSyntaxAndDescription, $sPnStr)
    EndIf

    If ($iPnPos > 0) And ($iPnPos < $iPos_CB) Then

      Local $sSyntaxPartPn = StringMid($_sSyntaxAndDescription, _
        $iPnPos + StringLen($sPnStr), $iPos_CB - $iPnPos - StringLen($sPnStr) - 1)

      If StringStripWS(StringReplace($sSyntaxPartPn, "]", ""), $STR_STRIPALL) = "" Then

        $_iParamNum = $iCommaCntIn_SyntaxTxt + 1 ; $_iParamNum (ByRef) als Rückgabe in dieser Funktion.
        Return False
      EndIf
    Else ; Kein "Param n" im Syntax-Text, aber zuviele Kommas.
      $_iParamNum = 0 ; Auswertung in "SplitSyntaxAndDescription()", "SetTextAndPosFor_SyntaxLabels()".

    EndIf ; If ($iPnPos > 0) And ($iPnPos < $iPos_CB)
  EndIf ; If $iCommaCntIn_ReceivedTxt > $iCommaCntIn_SyntaxTxt

EndFunc ; ==> CheckParamNum


; ==============================================================================
; CheckPSPadIsForegroundWindow
;
; Routine zum Ermitteln, ob PSPad das Vordergrund-Fenster ist.
;
; Rückgabe: Handle zum PSPad-Fenster, oder Null.
;
; Hinweis:  Die Rückgabe wird in der Hauptschleife an $g_hPSPad zugewiesen!
;                                                     ---------
;
; Neue Erkenntnis 2020-04-23:
;
; PSPad scheint die Ausführung von VBScript(-Schleifen) zu einzufrieren, wenn
; man z. B. ins PSPad-Mainmenü oder die -Toolbar oder -Statusbar klickt.
; Dann können z. B. Einstell-Dialoge geöffnet werden, ohne dass eine meiner
; Prüfungen das CallTipWin verstecken kann. Workaround: Prüfen ob PSPad
; (oder z. B. ein Dialog-Fenster) das ForegroundWin ist.
;
; ------------------------------------------------------------------------------
Func CheckPSPadIsForegroundWindow()
  Local $hTmp = Null

  $hTmp = _WinAPI_GetForegroundWindow()

  If ($hTmp <> Null) Then
    If _WinAPI_GetClassName($hTmp) <> 'TfPSPad' Then $hTmp = Null
  EndIf

  Return $hTmp
EndFunc ; ==> CheckPSPadIsForegroundWindow


; ==============================================================================
; ClosePSPadErrMsg_IfPSPadIsClosedWhileCallTipIsRunning
;
; Wenn PSPad geschlossen wird, während das CallTipWin angezeigt wird, verursacht
; die Schleife im VBScript eine Exception in PSPad. Eine Error-MsgBox erscheint.
;
; Mit dieser Routine hier kann die Error-MsgBox automatisch geschlossen werden
; und PSPad kann sich beenden. Dann bekommt der User davon nichts zu sehen. ...
; Zu überlegen ist, ob es besser ist, wenn der User die Info erhält und die
; Error-MsgBox selbst wegklickt, oder wenn er nichts davon sieht.
;
; ------------------------------------------------------------------------------
Func ClosePSPadErrMsg_IfPSPadIsClosedWhileCallTipIsRunning()

  Local $bShowCallTip = True

  Local $hTmp = WinGetHandle("[active]")

  If @error = 0 Then

    Local $sClassNameDlg = _WinAPI_GetClassName($hTmp)
    Local $sClassNameParent = _WinAPI_GetClassName(_WinAPI_GetParent($hTmp))

    Local $sMsgText

    If ($sClassNameDlg = "#32770") And ($sClassNameParent = 'TfPSPad') Then

      Local $sTitel = WinGetTitle($hTmp)
      Local $_sLanguageName = ReadFromPSPad_INI("Config", "Localization")
      Local $sErrorIdentifier = _
        ReadFromPSPadLanguage_INI($_sLanguageName, "General Strings", "rs_Error")

      If $sTitel = $sErrorIdentifier Then ; z. B. für Englisch: "Error".

        Send('^c')                ; Kopierbefehl an ErrorWin schicken (copy to clipboard).
        $sMsgText = ClipGet()     ; Neuen Inhalt aus der Zwischenablage lesen.

        If StringInStr($sMsgText, "[Content]") _
          And StringInStr($sMsgText, "TlvkActiveScript") _
          And StringInStr($sMsgText, "Module: AutoIt3_") Then

          WinMove($g_hCallTipWin, "", -10000, -10000)

          $g_oDict_COM_CallTip.Item("ShowCallTip") = False
          $bShowCallTip = False

          WinClose($hTmp)

        EndIf ;If StringInStr($sMsgText, "[Content]") And StringInStr($sMsgText, "TlvkActiveScript") And StringInStr($sMsgText, "Module: AutoIt3_")
      EndIf ; If $sTitel = $sErrorIdentifier
    EndIf ; If ($sClassNameDlg = "#32770") And ($sClassNameParent = 'TfPSPad')
  EndIf ; If @error = 0

  Return $bShowCallTip

EndFunc ; ==> ClosePSPadErrMsg_IfPSPadIsClosedWhileCallTipIsRunning


; ==============================================================================
; ColorScheme_ReadFromIniAndSetColorsForControls
;
; Routine zum Auslesen des Color Scheme aus der Datei
;   "...\PSPad4AutoIt3\PSPad4Au3\PSPad4AutoIt3.ini"
; und zuweisen der entsprechenden Farben zu den Controls.
;
; Bisher gibt es "auto" und zwei Color Schemes, wobei die Namen der Schemes
; die Farben für den CallTip in folgender Reihenfolge beschreiben:
;
;   Background_FontNormal_FontCurrentParam
;
;   Color Schemes  (2020-06-07, 2020-07-19.)
;   -------------
;   White_Black_Red: Das ursprüngliche helle Scheme.
;   Gray_White_Blue: Ein dunkles Scheme.
;   auto:            Automaische Auswahl des hellen oder dunklen Schemas, je
;                    nachdem welcher IDEStyle in PSPad gewählt ist. 2020-07-19.
;
; Derzeit können die Color Schemes nur manuell in der PSPad4AutoIt3.ini
; eingestellt werden, wobei es nun auch "auto" gibt. Bei "auto" wird entweder
; das helle oder das dunkle Color Scheme automatisch gewählt, je nachdem
; welcher IDEStyle in PSPad eingestellt ist (siehe "Switch $sColorScheme" unten).
; Nach einem Neustart von PSPad wird das geänderte Color Scheme übernommen.
; ------------------------------------------------------------------------------
Func ColorScheme_ReadFromIniAndSetColorsForControls()

  Local $sColorScheme = ReadFromPSPad4AutoIt3_INI("Config", "CallTipColorScheme")

  If ($sColorScheme = "") Or ($sColorScheme = "auto") Then
    $sColorScheme = ReadFromPSPad_INI("Config", "IDEStyle")
  EndIf

  Switch $sColorScheme

    Case "White_Black_Red", "Windows"
      GUISetBkColor($COLOR_WHITE, $g_hCallTipWin) ; CallTipWin Hintergrund weiß färben.
      GUISetBkColor($COLOR_WHITE, $g_hChildGui)   ; Child-GUI Hintergrund weiß färben.

      GUICtrlSetColor($g_LblSyntax, $COLOR_BLACK)
      GUICtrlSetColor($g_Lbl_Red, $COLOR_RED)
      $g_hexDescriptionColor_Au3Api = $COLOR_BLACK
      $g_hexDescriptionColor_OUDF = 0x808000 ; 0x808000, 0x666600, 0x505050, 0x556b2f, 0x107f90, 0xD4AF37

    Case "Gray_White_Blue", "Carbon", "Windows10 SlateGray"
      GUISetBkColor($COLOR_GRAY, $g_hCallTipWin)  ; CallTipWin Hintergrund grau färben.
      GUISetBkColor($COLOR_GRAY, $g_hChildGui)    ; Child-GUI Hintergrund grau färben.

      GUICtrlSetColor($g_LblSyntax, $COLOR_WHITE)
      GUICtrlSetColor($g_Lbl_Red, $COLOR_BLUE) ; $COLOR_BLUE, $COLOR_AQUA, $COLOR_LIME, $COLOR_MEDBLUE,  ----  $COLOR_MAROON, $COLOR_FUCHSIA, $COLOR_SKYBLUE, $COLOR_YELLOW
      $g_hexDescriptionColor_Au3Api = 0xE0E0E0 ; 0xE0E0E0 = helles grau
      $g_hexDescriptionColor_OUDF = 0xFFFF99   ; 0xFFCC00 = helles gelb.

  EndSwitch

EndFunc ; ==> ColorScheme_ReadFromIniAndSetColorsForControls


; ==============================================================================
; CreateGuisAndControls
;
; Routine zum Erstellen des CallTipWin und aller Controls.
; ------------------------------------------------------------------------------
Func CreateGuisAndControls($_iArrowLen, $_sTitel_CallTipWin)

  Local Const $sArrowSign_Up    = "5" ; (ASCII 53)
  Local Const $sArrowSign_Down  = "6" ; (ASCII 54)
  Local Const $sArrowSign_Left  = "3" ; (ASCII 51)
  Local Const $sArrowSign_Right = "4" ; (ASCII 52)

  $g_hCallTipWin = GUICreate($_sTitel_CallTipWin, 1200, 140, 100, 100, _
    BitOR($WS_POPUP, $WS_BORDER), BitOR($WS_EX_TOOLWINDOW, $WS_EX_TOPMOST, $WS_EX_NOACTIVATE))

  $g_hChildGui = GUICreate("", 1100, 120, 0, 0, $WS_CHILD, $WS_EX_TRANSPARENT, $g_hCallTipWin)
  GUISetState(@SW_SHOWNOACTIVATE, $g_hChildGui) ; Child-GUI sichtbar machen, nichtfokusierbar.

  $g_LblSyntax = GUICtrlCreateLabel("Label Syntax", 10, 40, 600, 80, $SS_NOPREFIX)      ; $SS_NOPREFIX: Ampersand NICHT als
  GUICtrlSetFont($g_LblSyntax, $g_fFontSize, $FW_NORMAL, $GUI_FONTNORMAL, $g_sFontName) ; unterstrichenes Zeichen zeigen.

  $g_Lbl_Red = GUICtrlCreateLabel("Label Red", 10, 40, 600, 80, $SS_NOPREFIX, _         ; $SS_NOPREFIX: Ampersand NICHT als
    $WS_EX_TRANSPARENT)                                                                 ; unterstrichenes Zeichen zeigen.
  GUICtrlSetFont($g_Lbl_Red, $g_fFontSize, $FW_NORMAL, $GUI_FONTNORMAL, $g_sFontName)

  $g_LblDescr = GUICtrlCreateLabel("Label Descr", 20, 30, 600, 80, $SS_NOPREFIX)        ; $SS_NOPREFIX: Ampersand NICHT als
  GUICtrlSetFont($g_LblDescr, $g_fFontSize, $FW_NORMAL, $GUI_FONTNORMAL, $g_sFontName)  ; unterstrichenes Zeichen zeigen.

  GUISwitch($g_hCallTipWin)

  $g_LblUp = GUICtrlCreateLabel($sArrowSign_Up, 0, 0, $_iArrowLen, $_iArrowLen)
  GUICtrlSetFont($g_LblUp, $g_fFontSize * 1.3, 0, 0, "Marlett")
  GUICtrlSetState($g_LblUp, $GUI_HIDE)

  $g_LblDown = GUICtrlCreateLabel($sArrowSign_Down, _
    0, $_iArrowLen, $_iArrowLen, $_iArrowLen * 2)
  GUICtrlSetFont($g_LblDown, $g_fFontSize * 1.3, 0, 0, "Marlett")
  GUICtrlSetState($g_LblDown, $GUI_HIDE)

  $g_LblLeft = GUICtrlCreateLabel($sArrowSign_Left, _
    $_iArrowLen, 0, $_iArrowLen, $_iArrowLen)
  GUICtrlSetFont($g_LblLeft, $g_fFontSize * 1.3, 0, 0, "Marlett")
  GUICtrlSetState($g_LblLeft, $GUI_HIDE)
  GUICtrlSetColor($g_LblLeft, 0x9ACD32)  ; grün

  $g_LblRight = GUICtrlCreateLabel($sArrowSign_Right, _
    $_iArrowLen, $_iArrowLen, $_iArrowLen, $_iArrowLen * 2)
  GUICtrlSetFont($g_LblRight, $g_fFontSize * 1.3, 0, 0, "Marlett")
  GUICtrlSetState($g_LblRight, $GUI_HIDE)
  GUICtrlSetColor($g_LblRight, 0x9ACD32) ; grün

EndFunc ; ==> CreateGuisAndControls


; #FUNCTION#============================================================================================================
; Name...........: _FileReadFast
; Description ...: Read text or binary files into a variable. Faster than FileRead at big file sizes
; Syntax.........: _FileReadFast(Const $s_FilePath, [[Const $flag_Encoding = Default], [Const $flag_FileScan = 2]])
; Parameters ....: $s_FilePath - the file path as a string
;                  $flag_Encoding: Default = the file encoding is determined by the function
;                                  0 = file gets handled as a binary file -> return a binary-variable
;                                  other = possible values are the same es the flag parameter of BinaryToString (faster)
;                  $flag_FileScan: The mode-parameter of FileGetEncoding $flag_Encoding = Default
; Return values .: Success - the file content as a string or a binary-variable
;                  Failure - a Null-String and error gets set to:
;                      @error = 1: File doesn't exist
;                             = 2: Couldn't open file for reading
;                             = 3: Couldn't determine file size
;                             = 4: error while reading file
;                      @extended = WinAPI error code
; Author ........: AspirinJunkie
; https://autoit.de/thread/84134-csv-datei-zeilen-z%C3%A4hlen-extrem-schnell-gnuwin32/?postID=673171#post673171
; ----------------------------------------------------------------------------------------------------------------------
Func _FileReadFast(Const $s_FilePath, Const $flag_Encoding = Default, Const $flag_FileScan = 2)

  If Not FileExists($s_FilePath) Then Return SetError(1, 0, "")
  Local $h_DLL_KERNEL32 = DllOpen("kernel32.dll")
  Local $a_Ret
  $a_Ret = DllCall($h_DLL_KERNEL32, "handle", "CreateFileW", "wstr", $s_FilePath, "dword", 0x80000000, "dword", 1, "struct*", Null, "dword", 3, "dword", 0, "ptr", Null)
  If DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0] Then Return SetError(2, DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0], "")
  Local $h_File = $a_Ret[0]
  $a_Ret = DllCall($h_DLL_KERNEL32, "bool", "GetFileSizeEx", "handle", $h_File, "int64*", 0)
  If DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0] Then Return SetError(3, DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0], "")
  Local $d_Bytes = $a_Ret[2]
  Local $t_Buffer = DllStructCreate("byte[" & $d_Bytes & "]")
  DllCall($h_DLL_KERNEL32, 'bool', 'ReadFile', 'handle', $h_File, 'struct*', $t_Buffer, 'dword', $d_Bytes, 'dword*', 0, 'ptr', 0)
  If DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0] Then Return SetError(4, DllCall($h_DLL_KERNEL32, "dword", "GetLastError")[0], "")
  DllCall($h_DLL_KERNEL32, "bool", "CloseHandle", "handle", $h_File)
  DllClose($h_DLL_KERNEL32)

  Switch $flag_Encoding
    Case Default; determine file encoding
      Switch FileGetEncoding($s_FilePath, $flag_FileScan)
        Case 32, 1024 ; UTF-16 LE
          Return BinaryToString(DllStructGetData($t_Buffer, 1), 2)
        Case 64, 2048 ; UTF-16 BE
          Return BinaryToString(DllStructGetData($t_Buffer, 1), 3)
        Case 128, 256 ; UTF-8
          Return BinaryToString(DllStructGetData($t_Buffer, 1), 4)
        Case 512 ; ANSI
          Return BinaryToString(DllStructGetData($t_Buffer, 1), 1)
        Case Else ; handle as binary
          Return DllStructGetData($t_Buffer, 1)
      EndSwitch
    Case 0 ; binary file
      Return DllStructGetData($t_Buffer, 1)
    Case Else ; encoding set by user
      Return BinaryToString(DllStructGetData($t_Buffer, 1), $flag_Encoding)
  EndSwitch
EndFunc   ;==>_FileReadFast


; ==============================================================================
; GetFont
;
; Dummy-Funktion, mit der mein abgebrochener Versuch und die dazugehörigen Infos
; an einer Stelle gespeichert werden können, ohne dass sie woanders stören. 2021-03-25.
; ------------------------------------------------------------------------------
Func GetFont()

EndFunc ; ==> GetFont


; ==============================================================================
; GetFuncInfoFromReceivedText
;
;   Autor: alpines, 2020-05-12.
;
; Routine zum Ermitteln von FunktionsInfos aus dem Text, der im ReceiverEdit
; übergeben wird. Es werden folgende Informationen daraus gewonnen:
;
; Rückgabe in den ByRef Parametern:
;
;   $_sFuncName: Gültiger Name der Funktion - oder "".
;
;   $_iParamNum: Nummer des Funktions-Parameters, in dem das Caret steht.
;                1 = Erster Parameter (aber auch für: kein Parameter).
;                2 = Parameter nach erstem Komma!
;                3 = Parameter nach zweitem Komma.
;                ...
;
;   $_iFuncNameFirstCharPosX_Intern:
;                Interne PosX, an der das erste Zeichen von FuncName steht.
;                Auswertung in GetFuncNameFirstCharPos_Extern().
;
;   $_aAllSyntaxSetsOfAFuncName: Array mit allen SyntaxSets zum FuncName.
; ------------------------------------------------------------------------------
Func GetFuncInfoFromReceivedText($_sReceivedTxt, ByRef $_sFuncName, ByRef $_iParamNum, _
  ByRef $_iFuncNameFirstCharPosX_Intern, ByRef $_aAllSyntaxSetsOfAFuncName)

  Local Static $aValidOperators = StringSplit("+-*/^?:,;([<>=&", "", $STR_ENTIRESPLIT + $STR_NOCOUNT)

  $_sFuncName = ""
  $_iParamNum = 1
  $_iFuncNameFirstCharPosX_Intern = 0

  Local $sCleanedText = ""

  Local $aSplit = StringSplit($_sReceivedTxt, @CRLF, $STR_ENTIRESPLIT + $STR_NOCOUNT)

  For $iCurrentLine = 0 To UBound($aSplit) - 1

    Local $sCurrentLine = $aSplit[$iCurrentLine]

    Local $iLastLCCPosition = 0

    Local $sNewLine = ""

    For $iCharPos = 1 To StringLen($sCurrentLine)

      Local $sCurrentChar = StringMid($sCurrentLine, $iCharPos, 1)

      Switch $sCurrentChar

        Case """", "'";, "[" ; Eckige Klammern rausgenommen am 2020-07-31.

          Local $iStringLength

          Local $bStringTerminates = False

          For $i = $iCharPos + 1 To StringLen($sCurrentLine)
            If StringMid($sCurrentLine, $i, 1) = $sCurrentChar Then
              $bStringTerminates = True
              ExitLoop
            EndIf
          Next

          If $bStringTerminates Then
            $iStringLength = 1 + $i - $iCharPos
          Else
            $iStringLength = 1 + StringLen($sCurrentLine) - $iCharPos
          EndIf

          For $i = 1 To $iStringLength
            $sNewLine &= "!"
          Next

          $iCharPos += $iStringLength - 1

        Case "_"
          $iLastLCCPosition = $iCharPos

          $sNewLine &= "_"

        Case ";"
          If $iCurrentLine < UBound($aSplit) - 1 Then
            Local $iCommentLength = 1 + StringLen($sCurrentLine) - $iCharPos

            For $i = 1 To $iCommentLength
              $sNewLine &= " "
            Next

            ExitLoop
          Else
            $sNewLine &= ";"
          EndIf

        Case Else
          $sNewLine &= $sCurrentChar

      EndSwitch
    Next

    If StringStripWS(StringTrimLeft($sNewLine, $iLastLCCPosition), $STR_STRIPALL) = "" Then
      $sNewLine = StringReplace($sNewLine, "_", " ", -1)
    EndIf

    $sCleanedText &= $sNewLine & (($iCurrentLine < UBound($aSplit) - 1) ? "  " : "")
  Next

  Local $iFuncBracketsCount = 0

  For $iCharPos = StringLen($sCleanedText) To 1 Step -1
    $sCurrentChar = StringMid($sCleanedText, $iCharPos, 1) ; $sCurrentChar wurde schon weiter oben deklariert. Professor Bernd, 2021-03-12.

    Switch $sCurrentChar

      Case ";"
        $_sFuncName = ""
        $iFuncBracketsCount = 0
        $_iParamNum = 1

      Case ","
        If $iFuncBracketsCount = 0 Then $_iParamNum += 1

      Case ")"
        $iFuncBracketsCount -= 1

      Case "("
        $iFuncBracketsCount += 1

        If $iFuncBracketsCount = 1 Then

          Local $bPassedWhitespaceBeforeParanthesis = False

          Local $iIdentifierEndPos = -1, $iIdentifierStartPos = -1

          For $i = $iCharPos - 1 To 1 Step - 1
            Local $sCurrentIdentifierChar = StringMid($sCleanedText, $i, 1)

            If $sCurrentIdentifierChar = " " or $sCurrentIdentifierChar = @TAB Then
              If Not $bPassedWhitespaceBeforeParanthesis Then
                ContinueLoop
              Else

                ExitLoop
              EndIf

            Else

              If Not $bPassedWhitespaceBeforeParanthesis Then
                $bPassedWhitespaceBeforeParanthesis = True

                If StringIsAlNum($sCurrentIdentifierChar) or $sCurrentIdentifierChar = "_" Then
                  $iIdentifierStartPos = $i
                  $iIdentifierEndPos = $i

                Else
                  ExitLoop
                EndIf
              Else

                If StringIsAlNum($sCurrentIdentifierChar) or $sCurrentIdentifierChar = "_" Then
                  $iIdentifierStartPos = $i
                Else

                  For $j = 0 To UBound($aValidOperators) - 1
                    If $sCurrentIdentifierChar = $aValidOperators[$j] Then
                      ExitLoop 2 ; Achtung! ExitLoop 2
                    EndIf
                  Next

                  $iIdentifierStartPos = -1
                  $iIdentifierEndPos = -1

                  ExitLoop
                EndIf
              EndIf
            EndIf
          Next

          If $iIdentifierStartPos <> -1 and $iIdentifierEndPos <> -1 Then
            $_sFuncName = StringMid($sCleanedText, $iIdentifierStartPos, $iIdentifierEndPos - $iIdentifierStartPos + 1)

            Local $sItem = "" ; $sItem ist das Syntax-Set.

            If $g_oDict_Au3Api_SynSets.Exists($_sFuncName) Then
              $sItem = $g_oDict_Au3Api_SynSets.Item($_sFuncName)
            ElseIf $g_oDict_OUDF_SynSets.Exists($_sFuncName) Then
              $sItem = $g_oDict_OUDF_SynSets.Item($_sFuncName)
            ElseIf $g_oDict_OUDF_SynSets.Exists("Volatile " & $_sFuncName) Then
              $sItem = $g_oDict_OUDF_SynSets.Item("Volatile " & $_sFuncName)
            EndIf

            $_aAllSyntaxSetsOfAFuncName = StringSplit($sItem, @CRLF, $STR_ENTIRESPLIT)

            If $sItem <> "" Then

              Local $sPartFuncNameToCaret = StringTrimLeft($_sReceivedTxt, $iIdentifierStartPos - 1)

              If Not StringInStr($sPartFuncNameToCaret, @CRLF) Then
                Local $iDistanceFromLeft = StringInStr($_sReceivedTxt, @CRLF, 1, -1, $iIdentifierStartPos)

                If $iDistanceFromLeft = 0 Then
                  $_iFuncNameFirstCharPosX_Intern = $iIdentifierStartPos
                Else
                  $_iFuncNameFirstCharPosX_Intern = $iIdentifierStartPos - $iDistanceFromLeft - 1
                EndIf
              Else

                $_iFuncNameFirstCharPosX_Intern = - StringInStr($sItem, "(") + 1
              EndIf

              ExitLoop
            EndIf
          EndIf

          $_sFuncName = ""
          $iFuncBracketsCount = 0
          $_iParamNum = 1
        EndIf
    EndSwitch
  Next

EndFunc ; ==> GetFuncInfoFromReceivedText


; ==============================================================================
; GetFuncNameFirstCharPos_Extern
;
; Routine zum Ermitteln der externen Koordinaten, an denen sich die linke,
; obere Ecke des ersten Buchstabens des Funktionsnamens im SynEdit befindet,
; für den der CallTip angezeigt werden soll.
;
; Rückgabe: Koordinaten als X und Y in den "_Extern"-Parametern (ByRef).
; ------------------------------------------------------------------------------
Func GetFuncNameFirstCharPos_Extern($_iFuncNameFirstCharPosX_Intern, _
  ByRef $_iFuncNameFirstCharPosX_Extern, ByRef $_iFuncNameFirstCharPosY_Extern)

  Local $sSpaceText     ; Platzhalter-Text.

  Local $iPosXCnt_FromFuncNameStartToCaret

  Local $aSynEdCaretPos ; Array für die internen und externen Caret-Koordinaten.
  Local $aStringSize    ; Array für Text-Breite, Höhe und Zusatzinfos (siehe _StringSize()).

  $_iFuncNameFirstCharPosX_Extern = 0
  $_iFuncNameFirstCharPosY_Extern = 0

  $aSynEdCaretPos = SplitSynEditCaretPos()
  If $aSynEdCaretPos[0] = 4 Then ; [0]: enthält die Anzahl der Elemente.

    $iPosXCnt_FromFuncNameStartToCaret = $aSynEdCaretPos[1] - $_iFuncNameFirstCharPosX_Intern

    $sSpaceText = _StringRepeat(" ", $iPosXCnt_FromFuncNameStartToCaret)

    $aStringSize = _StringSize($sSpaceText, $g_fFontSize, Default, Default, $g_sFontName)

    If UBound($aStringSize) = 4 Then ; 0-basiertes Array!.

      $_iFuncNameFirstCharPosX_Extern = $aSynEdCaretPos[3] - $aStringSize[2]

      $_iFuncNameFirstCharPosY_Extern = $aSynEdCaretPos[4]

    EndIf ; If UBound($aStringSize) = 4
  EndIf ; If $aSynEdCaretPos[0] = 4

EndFunc ; ==> GetFuncNameFirstCharPos_Extern


; ==============================================================================
; GetGUIThreadInfo
;
; Retrieves information about the active window or a specified GUI thread.
;
; Rückgabe:
;
;   Success: The array that contains the following information:
;
;     [0] - The thread state. It can be one or more of the $GUI_* values.
;     [1] - A handle to the active window within the thread.
;     [2] - A handle to the window that has the keyboard focus.
;     [3] - A handle to the window that has captured the mouse.
;     [4] - A handle to the window that owns any active menus.
;     [5] - A handle to the window in a move or size loop.
;     [6] - A handle to the window that is displaying the caret.
;     [7] - The x-coordinate of the upper-left corner of the caret's bounding rectangle, in screen coordinates!
;     [8] - The y-coordinate of the upper-left corner of the caret's bounding rectangle, in screen coordinates!
;     [9] - The width of the caret's bounding rectangle.
;     [10] - The height of the caret's bounding rectangle.
;
;   Failure: Sets the @error flag to non-zero, call _WinAPI_GetLastError() to get extended error information.
;
;   $GUI_* values: Die AutoIt Hilfe drückt sich sehr ungenau aus (2021-01-03). Die folgenden
;                  Infos stammen von der Microsoft Seite zu GUITHREADINFO structure (winuser.h)
;                  https://docs.microsoft.com/en-us/windows/win32/api/winuser/ns-winuser-guithreadinfo
;
;     Value                           Meaning
;     ------------------------------  ------------------------------------------
;     GUI_CARETBLINKING   0x00000001  The caret's blink state. This bit is set if the caret is visible.
;
;     GUI_INMOVESIZE      0x00000002  The thread's move state. This bit is set if the thread is in a move or size loop.
;
;     GUI_INMENUMODE      0x00000004  The thread's menu state. This bit is set if the thread is in menu mode.
;
;     GUI_SYSTEMMENUMODE  0x00000008  The thread's system menu state. ... if the thread is in a system menu mode.
;
;     GUI_POPUPMENUMODE   0x00000010  The thread's pop-up menu state. ... if the thread has an active pop-up menu.
; ------------------------------------------------------------------------------
Func GetGUIThreadInfo()

  Local $hWnd, $aInfo, $iPID, $aActiveCtrlPos

  $hWnd = WinGetHandle('[ACTIVE]')
  $aInfo = _WinAPI_GetGUIThreadInfo(_WinAPI_GetWindowThreadProcessId($hWnd, $iPID))
  If @error Then
    Dim $aInfo[11]
    For $i = 0 To 10
      $aInfo[$i] = ''
    Next
  Else
    $aInfo[0] = '0x' & Hex($aInfo[0], 8)

    $aActiveCtrlPos = WinGetPos($aInfo[2]) ; [2]: A handle to the window that has the keyboard focus.
    If IsArray($aActiveCtrlPos) Then
      $aInfo[7] = $aActiveCtrlPos[0] + $aInfo[7]
      $aInfo[8] = $aActiveCtrlPos[1] + $aInfo[8]

      $g_oDict_COM_CallTip.Item("CaretPosExtern") = $aInfo[7] & "|" & $aInfo[8]
    EndIf

  EndIf

  Return $aInfo
EndFunc ; ==> GetGUIThreadInfo


; ==============================================================================
; KFLP_GetControlKeys
;
; Die Routine ermittelt aus dem Flag (Gesamtwert der Shift-Buttons) die
; enthaltenen einzelnen Buttons.
;
; Infos zu den möglichen Einzelwerten siehe MS-Docs zu "VkKeyScanExW" im
; Abschnitt "Return value".
; https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-vkkeyscanexw
; ------------------------------------------------------------------------------
Func KFLP_GetControlKeys($_iFlag)

  Local $sRet = ""

  If $_iFlag = 0 Then Return '0'
  If BitAND($_iFlag,  1) Then $sRet &=  "1|" ; 'Either SHIFT key is pressed.' "
  If BitAND($_iFlag,  2) Then $sRet &=  "2|" ; 'Either CTRL key is pressed.' "
  If BitAND($_iFlag,  4) Then $sRet &=  "4|" ; 'Either ALT key is pressed.' "
  If BitAND($_iFlag,  8) Then $sRet &=  "8|" ; 'The Hankaku key is pressed' "
  If BitAND($_iFlag, 16) Then $sRet &= "16|" ; 'Reserved (defined by the keyboard layout driver).' "
  If BitAND($_iFlag, 32) Then $sRet &= "32|" ; 'Reserved (defined by the keyboard layout driver).' "

  Return StringTrimRight($sRet, 1) ; Letztes Pipe-Zeichen entfernen.
EndFunc ; ==> KFLP_GetControlKeys


; ==============================================================================
; KFLP_GetKeyCodesForLeftParenthesis
;
; Um "(" zu schreiben, wird mindestens eine Zeichentaste benötigt und je nach
; Keyboard-Layout eine oder mehrere Steuertasten (Shift, Ctrl, Alt, AltGr, ...).
;
; Die Routine kombiniert eigentlich 2 andere Routinen: "KFLP_VkKeyScanEx()"
; und "KFLP_GetControlKeys()" um die virtuellen KeyCodes der Zeichentaste und der
; Steuertasten zu ermitteln, die benötigt werden, um ein "(" zu schreiben,
; je nach angegebenem Keyboard-Layout. Bei deutschem Keyboard-Layout benötigt
; man z.B. für "(" die Shift-Taste + die 8-Taste (also Shift+8).
;
; Rückgabe ist ein Array mit 1 bis 4 Items:
;
;   [0]:   Enthält den virtuellen KeyCode der Zeichentaste als Hex-Wert, den
;          man z.B. in _IsPressed() benutzen kann.
;
; [1]-[3]: Die Items 1 - 3 enthalten die virtuellen KeyCodes der Steuertasten
;          "Shift", "Ctrl" und "Alt" als Hex-Wert, den man z.B. in _IsPressed()
;          benutzen kann. - Diese Items sind optional, das heißt, es können
;          0 - 3 Steuertasten angegeben sein, jenachdem welche Tasten beim
;          jeweiligen Keyboard-Layout zum Schreiben von "(" benötigt werden.
;
;   Ist KEINE Steuertaste angegeben, enthält das Array nur das Item[0],
;   ist 1 Steuertaste angegeben, enthält das Array Item[0] und Item[1], usw.
; ------------------------------------------------------------------------------
Func KFLP_GetKeyCodesForLeftParenthesis()

  Local $dwHKL
  Local $aKeyScan, $aRet[1]
  Local $sChar, $sCtrlKeys

  $sChar = "("

  $dwHKL = _WinAPI_GetKeyboardLayout(WinGetHandle(""))
  $aKeyScan = KFLP_VkKeyScanEx($sChar, $dwHKL) ; [0]: KeyCode (hex) for character key; [1]: ShiftState

  $aRet[0] = $aKeyScan[0]                         ; KeyCode (hex) for character key.

  $sCtrlKeys = KFLP_GetControlKeys($aKeyScan[1])       ; KeyCodes (hex) for control keys.
  If $sCtrlKeys <> "0" Then
    If StringInStr($sCtrlKeys, "1") Then _ArrayAdd($aRet, "10") ; Shift key - $VK_SHIFT
    If StringInStr($sCtrlKeys, "2") Then _ArrayAdd($aRet, "11") ; Ctrl  key - $VK_CONTROL
    If StringInStr($sCtrlKeys, "4") Then _ArrayAdd($aRet, "12") ; Alt   key - $VK_MENU
  EndIf

  Return $aRet
EndFunc ; ==> KFLP_GetKeyCodesForLeftParenthesis


; ==============================================================================
; KFLP_KeysForLeftParenthesisWerePressed
;
;   "KFLP" steht für "KeysForLeftParenthesis"
;
; Diese Routine prüft, ob die Tasten (1 oder mehrere) gedrückt wurden, die zum
; Schreiben einer öffnenden runden Klammer "(" benötigt werden.
;
; Hint: Die Routine pausiert die weitere Ausführung des Scripts, bis die
;       o.g. Tasten NICHT mehr gedrückt sind!
;
;       Das heißt, wenn alle benötigte Tasten gedrückt sind, wird eine
;       Schleife gestartet, die solange läuft, bis min. 1 der Tasten NICHT
;       mehr gedrückt ist. Erst danach wird das Script weiter ausgeführt.
;
; Wichtig: Dies ist nur ein Workaround, der lediglich einige Fälle prüft, aber
;          bei weitem NICHT alles abfängt. So wird z.B. NICHT geprüft, ob eine
;          weitere Zeichentaste gedrückt wird, z.B. Shift+7+8. 2021-02-23.
;
; Rückgabe: True  = Tasten für linke öffnende Klammer wurden gedrückt.
;           False = Tasten für linke öffnende Klammer wurden NICHT gedrückt.
; ------------------------------------------------------------------------------
Func KFLP_KeysForLeftParenthesisWerePressed()

  Local $aKeyCodes, $aTmpCodes[0]
  Local $bCapsLockOn, $bLeftParenthesisWasPressed = False

  $aKeyCodes = KFLP_GetKeyCodesForLeftParenthesis()

  $bCapsLockOn = (BitAND(_WinAPI_GetKeyState(0x14), 1) = 1)

  Switch UBound($aKeyCodes)

    Case 1 ; 0 Steuertasten werden benutzt = 3 Steuertasten sind UNBENUTZT = Temp-Array bekäme 3 Items.

      If _IsPressed($aKeyCodes[0], $g_hDLL) And $bCapsLockOn And _IsPressed("10", $g_hDLL) _
          And Not _IsPressed("11", $g_hDLL) And Not _IsPressed("12", $g_hDLL) Then
        $bLeftParenthesisWasPressed = True
      ElseIf _IsPressed($aKeyCodes[0], $g_hDLL) And Not $bCapsLockOn And Not _IsPressed("10", $g_hDLL) _
          And Not _IsPressed("11", $g_hDLL) And Not _IsPressed("12", $g_hDLL) Then
        $bLeftParenthesisWasPressed = True
      EndIf

    Case 2 ; 1 Steuertaste wird benutzt = 2 Steuertasten sind UNBENUTZT = Temp-Array bekommt 2 Items.

      If $aKeyCodes[1] <> "10" Then _ArrayAdd($aTmpCodes, "10")
      If $aKeyCodes[1] <> "11" Then _ArrayAdd($aTmpCodes, "11")
      If $aKeyCodes[1] <> "12" Then _ArrayAdd($aTmpCodes, "12")

      If $aKeyCodes[1] = "10" And $bCapsLockOn Then
        If _IsPressed($aKeyCodes[0], $g_hDLL) _
            And Not _IsPressed($aKeyCodes[1], $g_hDLL) And Not _IsPressed($aTmpCodes[0], $g_hDLL) And Not _IsPressed($aTmpCodes[1], $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      Else
        If _IsPressed($aKeyCodes[0], $g_hDLL) And _IsPressed($aKeyCodes[1], $g_hDLL) _
            And Not _IsPressed($aTmpCodes[0], $g_hDLL) And Not _IsPressed($aTmpCodes[1], $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      EndIf

    Case 3 ; 2 Steuertasten werden benutzt = 1 Steuertaste ist UNBENUTZT = Temp-Array bekommt 1 Item.

      If ($aKeyCodes[1] <> "10") And ($aKeyCodes[2] <> "10") Then _ArrayAdd($aTmpCodes, "10")
      If ($aKeyCodes[1] <> "11") And ($aKeyCodes[2] <> "11") Then _ArrayAdd($aTmpCodes, "11")
      If ($aKeyCodes[1] <> "12") And ($aKeyCodes[2] <> "12") Then _ArrayAdd($aTmpCodes, "12")

      If $aKeyCodes[1] = "10" And $bCapsLockOn Then
        If _IsPressed($aKeyCodes[0], $g_hDLL) And _IsPressed($aKeyCodes[2], $g_hDLL) _
            And Not _IsPressed($aKeyCodes[1], $g_hDLL) And Not _IsPressed($aTmpCodes[0], $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      ElseIf $aKeyCodes[2] = "10" And $bCapsLockOn Then
        If _IsPressed($aKeyCodes[0], $g_hDLL) And _IsPressed($aKeyCodes[1], $g_hDLL) _
            And Not _IsPressed($aKeyCodes[2], $g_hDLL) And Not _IsPressed($aTmpCodes[0], $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      Else
        If _IsPressed($aKeyCodes[0], $g_hDLL) And _IsPressed($aKeyCodes[1], $g_hDLL) And _IsPressed($aKeyCodes[2], $g_hDLL) _
            And Not _IsPressed($aTmpCodes[0], $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      EndIf

    Case 4 ; 3 Steuertasten werden benutzt = 0 Steuertasten sind UNBENUTZT = Temp-Array bekommt 0 Items.

      If ($aKeyCodes[1] = "10" Or  $aKeyCodes[2] = "10" Or $aKeyCodes[3] = "10") And $bCapsLockOn Then
        If _IsPressed($aKeyCodes[0], $g_hDLL) And Not _IsPressed("10", $g_hDLL) And _IsPressed("11", $g_hDLL) And _IsPressed("12", $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      Else
        If _IsPressed($aKeyCodes[0], $g_hDLL) And _IsPressed("10", $g_hDLL) And _IsPressed("11", $g_hDLL) And _IsPressed("12", $g_hDLL) Then
          $bLeftParenthesisWasPressed = True
        EndIf
      EndIf
  EndSwitch

  Return $bLeftParenthesisWasPressed
EndFunc ; ==> KFLP_KeysForLeftParenthesisWerePressed


; ==============================================================================
; KFLP_VkKeyScanEx
;
; Um ein Zeichen zu schreiben, wird mindestens eine Zeichentaste benötigt und
; je nach Zeichen eine oder mehrere Steuertasten (Shift, Ctrl, Alt, AltGr, ...).
;
; Die Routine ermittelt den virtuellen KeyCode der eigentlichen Taste und die
; Steuertasten, die benötigt werden, um ein angegebenes Zeichen zu schreiben,
; je nach angegebenem Keyboard-Layout. Bei deutschem Keyboard-Layout benötigt
; man z.B. für "(" die Shift-Taste + die 8-Taste (also Shift+8).
;
; Rückgabe ist ein Array mit 2 Items:
;
;   [0] enthält den virtuellen KeyCode der Zeichentaste als
;       Hex-Wert, den man z.B. in _IsPressed() benutzen kann.
;
;   [1] enthält den Status der Shift-Keys (Steuertasten),
;       der eine Kombination aus fogenden Werten sein kann.
;
;       ShiftState   0 No control key pressed.
;                    1 Either SHIFT key is pressed.
;                    2 Either CTRL key is pressed.
;                    4 Either ALT key is pressed.
;                    8 The Hankaku key is pressed.
;                   16 Reserved (defined by the keyboard layout driver).
;                   32 Reserved (defined by the keyboard layout driver).
;
; Diese Routine ist eine Nachbildung der Win 32 API Funktion "VkKeyScanExW".
; Siehe: https://docs.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-vkkeyscanexw
; Infos zu den möglichen Einzelwerten zu ShiftState siehe o.g. MS-Docs-Seite zu
; "VkKeyScanExW" im Abschnitt "Return value".
; Autor: Professor Bernd, 2021-02-13.
; Mein Code basiert auf:
; Autor: j0kky, 2014-09-14.
; https://www.autoitscript.com/forum/topic/164468-how-can-i-find-non-english-characters-key-code/?do=findComment&comment=1199953
; Autor: ProgAndy, 2012-03-17.
; https://www.autoitscript.com/forum/topic/138681-user32dllvkkeyscan-not-doing-it-right/?do=findComment&comment=971876
; ------------------------------------------------------------------------------
Func KFLP_VkKeyScanEx($s_Char, $dw_HKL)

  Local $vRetVal = DllCall($g_hDLL, "short", "VkKeyScanExW", "short", AscW($s_Char), "hwnd", $dw_HKL)

  Local $aRet[2] ; [0]: KeyCode (hex); [1]: ShiftState
  $aRet[0] = Hex(BitAnd($vRetVal[0], 0xFF), 2)
  $aRet[1] = BitShift($vRetVal[0], 8)

  Return $aRet
EndFunc ; ==> KFLP_VkKeyScanEx


; ==============================================================================
; LabelWasClicked
;
; Routine zum Ermitteln, ob in ein bestimmtes Label geklickt wurde.
;
; Rückgabe: 0 = Label wurde NICHT geklickt, 1 = Label wurde geklickt.
;
; $_sHexKey: '01' = linke Maustaste, '02' = rechte Maustaste, '04' = mittlere Maustaste.
;            (Siehe AutoIt-Hilfe "_IsPressed()", oder WinAPIvkeysConstants.au3.)
; ------------------------------------------------------------------------------
; Func LabelWasClicked($sg_GUI, $sl_Label, $s_hexKey = '01', $st_Text = '', $v_dll = 'user32.dll')
;   Local $a_R = DllCall($v_dll, "int", "GetAsyncKeyState", "int", '0x' & $s_hexKey)
;   If Not @Error And BitAND($a_R[0], 0x8000) = 0x8000 Then
Func LabelWasClicked($_hGui, $_idLabel, $_iCallTipX, $_iCallTipY, $_iSleepVal = 10, $_sHexKey = '01', $_sText = '')

  Local $aLabelRect
  Local $aMousePos
  Local $iRetVal = 0

  Local Static $idCurrentLabel = 0

  Local Const $iOnePix = 1 ; Um den Erfassungsbereich rundherum um 1 Pixel zu verkleinern.
  Local Const $iBitMask = 0x8000 ; a bit mask to strip the high word bits from the return of the function.

  If BitAND(_WinAPI_GetAsyncKeyState($_sHexKey), $iBitMask) <> 0 Then

    If ($idCurrentLabel = 0) Or ($idCurrentLabel = $_idLabel) Then

      $aLabelRect = ControlGetPos($_hGui, $_sText, $_idLabel)
      $aLabelRect[0] += $_iCallTipX
      $aLabelRect[1] += $_iCallTipY

      $aMousePos = MouseGetPos()

      If IsArray($aMousePos) And IsArray($aLabelRect) Then

        If    $aMousePos[0] - $iOnePix >= $aLabelRect[0] _
          And $aMousePos[0] + $iOnePix <= $aLabelRect[0] + $aLabelRect[2] _
          And $aMousePos[1] - $iOnePix >= $aLabelRect[1] _
          And $aMousePos[1] + $iOnePix <= $aLabelRect[1] + $aLabelRect[3] Then

          Sleep($_iSleepVal) ; Professor Bernd, 2020-04-11, 2020-05-24.

          If $idCurrentLabel = 0 Then $idCurrentLabel = $_idLabel
          $iRetVal = 1 ; Auf Label geklickt.
        EndIf ; If $aMousePos[0] - $iOnePix >= $aLabelRect[0] ...
      EndIf ; If IsArray($aMousePos) And IsArray($aLabelRect)
    EndIf ; If ($idCurrentLabel = 0) Or ($idCurrentLabel = $_idLabel)
  Else ; NICHT auf Label geklickt.
    $idCurrentLabel = 0
    $iRetVal = 0
  EndIf ; If BitAND(_WinAPI_GetAsyncKeyState($_sHexKey), $iBitMask) <> 0

  Return $iRetVal

EndFunc ; ==> LabelWasClicked


; ==============================================================================
; OnAutoItExit
;
; Routine die aufgerufen werden soll, wenn AutoIt verlassen wird.
; Sie wurde mit "OnAutoItExitRegister('OnAutoItExit')" registriert.
; ------------------------------------------------------------------------------
Func OnAutoItExit()
  If $g_iID Then
    _WinAPI_ShellChangeNotifyDeregister($g_iID)
  EndIf
  ROT_Revoke($g_COM_ROT_Handle)
EndFunc ; ==> OnAutoItExit


; ==============================================================================
; OUDF_EmptyAllDictionaries
;
; Routine zum Leeren aller OUDF-Dictionaries.
; ------------------------------------------------------------------------------
Func OUDF_EmptyAllDictionaries()

  If $g_oDict_OUDF_Au3Paths.Count > 0 Then
    $g_oDict_OUDF_SynSets.RemoveAll  ; Syntax-Sets für OUDFs (Syntax und Datei-Pfad).
    $g_oDict_OUDF_Au3Paths.RemoveAll ; Pfade zu den au3 Dateien, aus denen OUDFs ausgelesen wurden.
    $g_oDict_OUDF_Au3Dirs.RemoveAll  ; Ordner-Pfade, in denen die ausgelesenen au3 Dateien liegen (Monitoring).
    $g_bOUDF_DictsWereEmptied = True ; Flag setzen, damit der CallTip aktualisieren werden kann,
  EndIf
EndFunc ; ==> OUDF_EmptyAllDictionaries


; ==============================================================================
; OUDF_GetIncludedFilePathsRecursive
;
; Routine zum Ermitteln aller Pfade zu User-Scripts in den Includes eines
; übergebenen Scripts. Die Ermittlung erfolgt rekursiv, bisher ohne Limit
; der Tiefe. 2020-07-16.
; ------------------------------------------------------------------------------
Func OUDF_GetIncludedFilePathsRecursive($_sFilePath)

  Local $sText
  Local $iIndex = 0, $iPathsCount = 1
  Local $aAllIncludes = 0, $aOUDF_FilePaths[1] = [$_sFilePath]

  While Not ($iIndex = $iPathsCount)

    $sText = RemoveRegionsAndComments(FileRead($aOUDF_FilePaths[$iIndex]))
    $aAllIncludes = StringRegExp($sText, '(?im)^\h*#include\h+[<"'']([^*?"''<>|]+)', 3)

    $iPathsCount = UBound($aOUDF_FilePaths)

    For $i = 0 To UBound($aAllIncludes) - 1

      Local $sPathFound = _PathFull($aAllIncludes[$i], _
        _WinAPI_PathRemoveFileSpec($aOUDF_FilePaths[$iIndex]))

      If FileExists($sPathFound) And Not $g_oDict_OUDF_Au3Paths.Exists($sPathFound) _
        And _ArraySearch($aOUDF_FilePaths, $sPathFound, 0, 0, 0, 2) < 0 Then
        $iPathsCount = _ArrayAdd($aOUDF_FilePaths, $sPathFound) ; Pfad in Array aufnehmen.
      EndIf
    Next

    $iIndex += 1
  WEnd

  Return $aOUDF_FilePaths
EndFunc  ; ==> OUDF_GetIncludedFilePathsRecursive


; ==============================================================================
; OUDF_InitializeDictionaries
;
; Routine zum Erstellen der Dictionaries und zum Einstellen des CompareMode.
; ------------------------------------------------------------------------------
Func OUDF_InitializeDictionaries()

  $g_oDict_OUDF_SynSets = ObjCreate("Scripting.Dictionary")
  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "CallTipViewer", Function "OUDF_InitializeDictionaries":' _
      & @CRLF & @CRLF & 'Error when creating the Dictionary Object.')
    Exit(10) ; 10 = Fehler beim Erzeugen des Dictionary Objekts.
  Else
    $g_oDict_OUDF_SynSets.CompareMode = 1 ; (0 = BinärMode (default), 1 = Textmode)
  EndIf ; If @error

  $g_oDict_OUDF_Au3Paths = ObjCreate("Scripting.Dictionary")
  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "CallTipViewer", Function "OUDF_InitializeDictionaries":' _
      & @CRLF & @CRLF & 'Error when creating the Dictionary Object.')
    Exit(10) ; 10 = Fehler beim Erzeugen des Dictionary Objekts.
  Else
    $g_oDict_OUDF_Au3Paths.CompareMode = 1 ; (0 = BinärMode (default), 1 = Textmode)
  EndIf ; If @error

  $g_oDict_OUDF_Au3Dirs = ObjCreate("Scripting.Dictionary")
  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "CallTipViewer", Function "OUDF_InitializeDictionaries":' _
      & @CRLF & @CRLF & 'Error when creating the Dictionary Object.')
    Exit(10) ; 10 = Fehler beim Erzeugen des Dictionary Objekts.
  Else
    $g_oDict_OUDF_Au3Dirs.CompareMode = 1 ; (0 = BinärMode (default), 1 = Textmode)
  EndIf ; If @error
EndFunc ; ==> OUDF_InitializeDictionaries


; ==============================================================================
; OUDF_ReadFromFileAndIncludes
;
; In dieser Routine werden zunächst alle Pfade aus den #includes ermittelt, die
; zu Scripts führen, die relativ zum übergebenen Script liegen. Dadurch sollten
; nur User-Scripts erfasst werden und keine aus dem offizielnen AutoIt Ordner
; "Include". Die Ermittlung ist rekursiv, d. h. es werden auch die inkludierten
; Scripts und deren inkludierten Scripts usw. durchsucht.
;
; Dann werden alle Dateien der gefundenen Pfade durchsucht auf OUDFs. Dabei
; werden die OUDFs, die Au3-Pfade und die Au3-Dirs in jeweils ein Dictionary
; gespeichert.
;
; Danach wird die Ordner-Überwachung (Monitoring) mit den neuen Au3-Dirs
; aktualisiert. In OUDF_ShellChangeNotifyRegister() wird eine vorhandene
; Registrierung aufgehoben und die neuen Pfade werden fürs Monitoring
; registriert.
; ------------------------------------------------------------------------------
Func OUDF_ReadFromFileAndIncludes($_sFilePath)

  If FileExists($_sFilePath) And Not $g_oDict_OUDF_Au3Paths.Exists($_sFilePath) Then

    Local $aFileAndIncludePaths = OUDF_GetIncludedFilePathsRecursive($_sFilePath)

    For $i = 0 To UBound($aFileAndIncludePaths) - 1
      OUDF_ReadFromFileToDictionary($aFileAndIncludePaths[$i])
    Next

    If $g_oDict_OUDF_Au3Dirs.Count > 0 Then

      OUDF_ShellChangeNotifyRegister($g_oDict_OUDF_Au3Dirs.Keys)
    EndIf
  EndIf
EndFunc ; ==> OUDF_ReadFromFileAndIncludes


; ==============================================================================
; OUDF_ReadFromFileToDictionary
;
; In dieser Routine wird jeweils 1 Au3-Datei verarbeitet.
; - Der Pfad zur Au3-Datei wird ins Au3Paths Dictionary gespeichert, das
;   verwendet wird zum Prüfen, ob Dateien schon aufgenommen wurden.
; - Der Pfad zum Dir, in dem die Au3-Datei liegt, wird ins Au3Dirs Dictionary
;   gespeichert, das verwendet wird für die Ordner-Überwachung (Monitoring).
; - Die Funktions-Köpfe werden aus der Au3-Datei auslesen und zusammen mit dem
;   Datei-Pfad als Syntax-Set in das SynSets-Dictionary gespeichert. 2020-07-17.
; ------------------------------------------------------------------------------
Func OUDF_ReadFromFileToDictionary($_sFilePath)

  Local $sLineRead, $sKey, $sItem
  Local $iDelimPos

  If Not $g_oDict_OUDF_Au3Paths.Exists($_sFilePath) Then ; <= Falls Key noch nicht existiert:
    $g_oDict_OUDF_Au3Paths.Add($_sFilePath, "")          ; FilePath als Key und leeres Item dem Au3-Pfad-Dictionary hinzufügen.

    Local $sDirPath = StringRegExpReplace($_sFilePath, "(^.*\\)(.*)", "\1") ; Dir path with backslash.
    If Not $g_oDict_OUDF_Au3Dirs.Exists($sDirPath) Then  ; <= Falls Key noch nicht existiert:
      $g_oDict_OUDF_Au3Dirs.Add($sDirPath, "")           ; DirPath als Key und leeres Item dem Au3-Dir-Dictionary hinzufügen.
    EndIf

    Local $aFuncList = OUDF_ReadFunctionHeads($_sFilePath)

    If Not @error Then

      For $i = 0 To UBound($aFuncList) - 1 Step 1

        $sLineRead = $aFuncList[$i]

        $iDelimPos = StringInStr($sLineRead, " ( ")

        If $iDelimPos > 0 Then

          $sKey  = StringLeft($sLineRead, $iDelimPos - 1) ; FuncName aus Zeile filtern.
          $sItem = $sLineRead & '"' & $_sFilePath & '"'   ; Komplette Zeile (Syntax) plus Dateipfad speichern.

          If Not $g_oDict_OUDF_SynSets.Exists($sKey) Then ; <= Falls Key noch nicht existiert:
            $g_oDict_OUDF_SynSets.Add($sKey, $sItem)      ; Key und Item (Syntax-Set) dem OUDF-Dictionary hinzufügen.
          Else
            $g_oDict_OUDF_SynSets.Item($sKey) &= @CRLF & $sItem
          EndIf
        EndIf ; If $iDelimPos > 0 Then
      Next
    EndIf ; If @error
  EndIf ; If ($_sFilePath <> "") And Not $g_oDict_OUDF_Au3Paths.Exists($_sFilePath)
EndFunc ; ==> OUDF_ReadFromFileToDictionary


; ==============================================================================
; OUDF_ReadFunctionHeads
;
; Author:         Professor Bernd.
; Date:           2020-07-06.
; Syntax:         OUDF_ReadFunctionHeads(Const $_sFilePath)
; Parameters:     $_sFilePath - the file path as a string
; Return values:  Success - an array with function names and parameter lists
;                 Failure - @error
; Description:
;
;   Routine zum Auslesen von Funktions-Namen und deren Parameter-Liste aus den
;   Funktions-Deklarationen einer Datei.
;
;   Ausgelesen werden alle gültigen Funktions-Köpfe einer Datei. Daraus werden
;   die Funktions-Namen mit zugehöriger Parameter-Liste gefiltert und in einer
;   Schreibweise dargestellt, die der aus der AutoIt Hilfe ähnelt.
;
;   - - - -
;
;   Routine to read function names and their parameter list from the function
;   declarations of a file.
;
;   All valid function headers of a file are read out. The function names
;   with the associated parameter list are filtered and displayed in a
;   notation similar to the AutoIt help.
; ------------------------------------------------------------------------------
Func OUDF_ReadFunctionHeads(Const $_sFilePath)

  Local $sAllLines = RemoveRegionsAndComments(_FileReadFast($_sFilePath)) & @CRLF

  If $sAllLines = @CRLF Then Return SetError(1, 0, '')

  $sAllLines = StringRegExpReplace($sAllLines, '\h+_\h*\R+?', ' ')

  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?mi)^\h*Volatile Func\h+.*\R?(*SKIP)(*F)|^\h*Func\h+.*\R?(*SKIP)(*F)|^\h*EndFunc.*\R?(*SKIP)(*F)|^.*\R?', "")

  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?x)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \h* (?: <> | [&\+\-\*\/=><]? = | [\(\)\*+\-\/&^<>?:] ) \h*)', ' \1 ')

  $sAllLines = StringTrimRight(StringRegExpReplace($sAllLines & ':and=And:or=Or:not=Not', _
    '(?sxi)(?: (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \b(and|or|not)\b ) (?=.*:\1=(\w+)\b)', '\2'), 22)

  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?xi)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \bconst\b )', ' Const ')
  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?xi)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \bbyref\b )', ' ByRef ')

  $sAllLines = StringRegExpReplace($sAllLines, _
    '(?x)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \h*, )', ', ')

  $sAllLines = StringRegExpReplace($sAllLines, "(\h+)", ' ')

  $sAllLines = StringRegExpReplace($sAllLines, "( = -)\h+(\d)", '\1\2')

  Local $sPattern = _
     '(?mi)^\h*(Volatile\h+)?Func\h+((?(DEFINE)(?<brackets>\((?|''(?>[^''\v]+|'''')*''|"(?>[^"\v]+|"")*"|[^\(\)\r\n"'']*|(?&brackets))*\)))\w+\h*(?&brackets)\h*(?=\R\h*EndFunc))'

  Local $aFuncs = StringRegExp($sAllLines, $sPattern, $STR_REGEXPARRAYGLOBALMATCH)

  Local $aRet[UBound($aFuncs) / 2]

  For $i = 0 To UBound($aRet) - 1
    $aRet[$i] = $aFuncs[$i * 2 + 0] & $aFuncs[$i * 2 + 1]

    $aRet[$i] = StringReplace($aRet[$i], "Volatile ", "Volatile ")
  Next

  Return SetError(@error, @extended, @error ? @error : $aRet)
EndFunc ; ==> OUDF_ReadFunctionHeads


; ==============================================================================
; OUDF_ShellChangeNotifyRegister
;
; Routine mit der die Ordner-Überwachung aufgehoben und mit neuen Ordner-Pfaden
; registriert wird.
;
; Mithilfe der Ordner-Überwachung werden in OUDF_WM_SHELLCHANGENOTIFY() alle
; Dictionaries geleert, danach in OUDF_ReadFromFileAndIncludes() mit neuen
; Pfaden gefüllt und hier wird die Ordner-Überwachung aktualisiert.
; ------------------------------------------------------------------------------
Func OUDF_ShellChangeNotifyRegister($_aPaths)

  If $g_iID Then _WinAPI_ShellChangeNotifyDeregister($g_iID)

  $g_iID = _WinAPI_ShellChangeNotifyRegister($g_hCallTipWin, $g_iWinMsg, $SHCNE_UPDATEITEM, _
    BitOR($SHCNRF_INTERRUPTLEVEL, $SHCNRF_SHELLLEVEL), $_aPaths)

  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error ' & @error & '  in Module "CallTipViewer", Function "OUDF_ShellChangeNotifyRegister":' _
      & @CRLF & @CRLF & 'The window could not be registered.')
  EndIf
EndFunc ; ==> OUDF_ShellChangeNotifyRegister


; ==============================================================================
; OUDF_ShowToolTipForSavingTheFile
;
; ------------------------------------------------------------------------------
Func OUDF_ShowToolTipForSavingTheFile($_sReceivedTxt, ByRef $_sChkShowHideTxt_Old)

  Local $aSynEdCaretPos ; Array für die internen und externen Caret-Koordinaten.
  Local $sReceivedText
  Local $aRecvdTxt

  If StringInStr(WinGetTitle($g_hPSPad), '*') Then

    $aSynEdCaretPos = SplitSynEditCaretPos()

    If $aSynEdCaretPos[0] = 4 Then

      $sReceivedText = RemoveRegionsAndComments($_sReceivedTxt)
      $sReceivedText = StringStripWS($sReceivedText, $STR_STRIPTRAILING)

      $sReceivedText = StringRegExpReplace($sReceivedText, '\h+_\h*\R+?', ' ')

      $sReceivedText = StringReplace($sReceivedText, '$', '')
      $sReceivedText = StringReplace($sReceivedText, '#', '')

      $sReceivedText = StringRegExpReplace($sReceivedText, _
        '(?x)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \h*\( )', '$')

      $sReceivedText = StringRegExpReplace($sReceivedText, _
        '(?x)( (?>"(?> [^"\v]+ | "" )*" | ''(?> [^''\v]+ | '''' )*'') (*SKIP)(*FAIL) | \h*\) )', '#')

      Local $i_C_B_Found = 1

      $aRecvdTxt = StringSplit($sReceivedText, "")

      For $i = $aRecvdTxt[0] To 1 Step -1

        If $aRecvdTxt[$i] = '$' Then $i_C_B_Found -= 1 ; '$' <= '('
        If $aRecvdTxt[$i] = '#' Then $i_C_B_Found += 1 ; '#' <= ')'

        If $i_C_B_Found = 0 Then

          $i_C_B_Found = 1

          For $x = $i - 1 To 1 Step -1

            If StringIsAlNum($aRecvdTxt[$x]) Or ($aRecvdTxt[$x] = '_') Then
              $i_C_B_Found = 0
              ExitLoop 2 ; Beide Schleifen verlassen.
            ElseIf Not ($aRecvdTxt[$x] = ' ') And Not ($aRecvdTxt[$x] = @TAB) Then
              ExitLoop 2 ; Beide Schleifen verlassen.
            EndIf
          Next
        EndIf
      Next

      If $i_C_B_Found = 0 Then
        ToolTipWithTimeOut_Show("Please save the file first.", _
          $aSynEdCaretPos[3], $aSynEdCaretPos[4] + ($g_iLineHeight * 2) + $g_iOffsetY, _
          "No CallTip available.")
        $_sChkShowHideTxt_Old = ""
      EndIf
    EndIf
  EndIf ; If StringInStr(WinGetTitle($g_hPSPad), '*')
EndFunc ; ==> OUDF_ShowToolTipForSavingTheFile


; ==============================================================================
; OUDF_WM_SHELLCHANGENOTIFY  (Ereignis-Empfänger / Event Receiver)
;
; Monitoring-Routine die von Windows aufgerufen wird, wenn sich eine Datei im
; überwachten Ordner ändert.
;
; Diese Routine wird registriert mit
;   _WinAPI_RegisterWindowMessage('SHELLCHANGENOTIFY')
;   GUIRegisterMsg($g_iWinMsg, 'OUDF_WM_SHELLCHANGENOTIFY')
;
; Wenn dieses Script beendet wird, wird in der Routine OnAutoItExit() die
; Registrierung aufgehoben und damit die Ordner-Überwachung beendet.
;
; Ordner zum Überwachen werden entfernt/hinzugefügt mit
; _WinAPI_ShellChangeNotifyDeregister und _WinAPI_ShellChangeNotifyRegister()
; in der Routine OUDF_ShellChangeNotifyRegister().
; ------------------------------------------------------------------------------
Func OUDF_WM_SHELLCHANGENOTIFY($hWnd, $iMsg, $wParam, $lParam)
  #forceref $hWnd, $iMsg, $wParam, $lParam

  OUDF_EmptyAllDictionaries()

EndFunc ; ==> OUDF_WM_SHELLCHANGENOTIFY


; ==============================================================================
; OwnPSPadWinExists
;
; Routine zum Prüfen, ob mindestens 1 Fenster existiert, das zur eigenen
; PSPad.exe gehört.
;
; Es können mehrere PSPad-"Installationen" gleichzeitig benutzt werden, z.B.
; PSPad4AutoIt3 + eine "normale" PSPad-Version zum bearbeiten von Text, usw.
; Der CallTipViewer gehört jedoch nur zu dieser (portablen) PSPad-Installation
; und soll sich dementsprechend beenden, wenn kein Fenster mehr existiert, das
; zur eigene PSPad.exe gehört. Das ist z.B. wichtig, wenn zur Übernahme von
; Einstellungen PSPad neu gestartet werden soll. 2021-03-01.
;
;   Rückgabe: True  - Es existiert mindestens 1 Fenster, das zur eigenen PSPad.exe gehört.
;             False - KEIN Fenster existiert, das zur eigenen PSPad.exe gehört.
; ------------------------------------------------------------------------------
Func OwnPSPadWinExists()

  Local $bOwnPSPadWinExists = False
  Local $sPSPadExe_Path = _PathFull(@ScriptDir & '\..\..\PSPad.exe')    ; @ScriptDir: '...\PSPad-Dir\PSPad4Au3\Au3 scripts\'

  Local $aWinLst = WinList("[CLASS:TfPSPad]")                           ; get list of PSPad windows

  For $i = 0 To $aWinLst[0][0]
    If _WinAPI_GetProcessFileName(WinGetProcess($aWinLst[$i][1])) = $sPSPadExe_Path Then  ; [$i][1]: Idx for $handle
      $bOwnPSPadWinExists = True
      ExitLoop
    EndIf

  Next

  Return $bOwnPSPadWinExists
EndFunc ; ==> OwnPSPadWinExists


; ==============================================================================
; ReadAu3Api_ToDictionary
;
; Routine zum zeilenweisen Einlesen der SciTE-au3.api in ein Dictionary-Object.
;
; Im Dictionary befinden sich dann Sets der AutoIt-Funktionsbeschreibungen. Ein
; Set entspricht einer Zeile und besteht aus Syntax-Teil und Beschreibungs-Teil.
;
; Im Dictionary besteht ein Element aus dem Key (Funktionsname) und dem Item
; (= Set, = komplette Zeile mit Syntax und Beschreibung der jeweiligen Funktion).
;
; Allerdings kann ein Item auch mehrere Sets zu einem Key enthalten, wenn ein
; Funktionsname in der au3.api mehrfach vorkommt.
;
; Beispiele zum Auslesen eines Items des Dictionary zu einer bestimmten Funktion.
;   $sStr = $g_oDict_Au3Api_SynSets.Item("Opt")     ; Die Suche nach einem Key gibt das Item zurück.
;   $sStr = $g_oDict_Au3Api_SynSets("_HexToString") ; Suche kann auch ohne ".Item" aufgerufen werden.
;
; Das Anlegen des Dictionary basiert auf einem Code-Snippet von User Musashi.
; Datum: 2020-03-15.
; Quelle: https://autoit.de/thread/86839-gro%C3%9Fe-datei-durchsuchen-optimierungsvorschl%C3%A4ge-gesucht/?postID=698721#post698721
; ------------------------------------------------------------------------------
Func ReadAu3Api_ToDictionary()

  Local $sAu3ApiPath, $sLineRead, $sKey, $sItem
  Local $hFileOpen
  Local $bEOF_IsReached
  Local $iDelimPos

  $g_oDict_Au3Api_SynSets = ObjCreate("Scripting.Dictionary")
  If @error Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
      'Error in Module "CallTipViewer", Function "Au3Api_ReadToDictionary":' _
      & @CRLF & @CRLF & 'Error when creating the Dictionary Object.')
    Exit(10) ; 10 = Fehler beim Erzeugen des Dictionary Objekts.
  Else

    $g_oDict_Au3Api_SynSets.CompareMode = 1 ; (0 = BinärMode (default), 1 = Textmode)

    $sAu3ApiPath = ReadFromPSPad4AutoIt3_INI("Paths", "SciTE_Dir") & "api\au3.api"

    If Not FileExists($sAu3ApiPath) Then $sAu3ApiPath = @ScriptDir & "\..\AutoIt3\SciTE\api\au3.api"

    $hFileOpen = FileOpen($sAu3ApiPath, BitOR($FO_READ, $FO_UTF8_NOBOM))
    If $hFileOpen = -1 Then
      MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, 'CallTipViewer', _
        'Error in Module "CallTipViewer", Function "Au3Api_ReadToDictionary":' & @CRLF _
        & @CRLF & 'Error while opening the file.'& @CRLF & @CRLF & "'" & $sAu3ApiPath & "'")
      Exit(11) ; 11 = Fehler beim Öffnen der SciTE Datei "au3.api".
    EndIf

    $bEOF_IsReached = False
    While Not $bEOF_IsReached

      $sLineRead = FileReadLine($hFileOpen)
      If @error Then $bEOF_IsReached = True

      $iDelimPos = StringInStr($sLineRead, " ( ")

      If $iDelimPos > 0 Then

        $sKey  = StringLeft($sLineRead, $iDelimPos - 1) ; FuncName aus Zeile filtern.
        $sItem = $sLineRead                             ; Komplette Zeile im Variable speichern.

        If Not $g_oDict_Au3Api_SynSets.Exists($sKey) Then  ; <= Falls Key noch nicht existiert:
          $g_oDict_Au3Api_SynSets.Add($sKey, $sItem)       ; Key und Item dem Dictionary hinzufügen.
        Else
          $g_oDict_Au3Api_SynSets.Item($sKey) &= @CRLF & $sItem
        EndIf
      EndIf ; If $iDelimPos > 0 Then
    WEnd
    FileClose($hFileOpen)
  EndIf ; If @error
EndFunc ; ==> ReadAu3Api_ToDictionary


; ==============================================================================
; ReadFromPSPad4AutoIt3_INI                             Version von 2019-12-13
;
; Daten aus PSPad4AutoIt3.ini im Unterverzeichnis "\PSPad4Au3\" lesen.
;
;   Rückgabe: Wert des Keys - oder "".
; ------------------------------------------------------------------------------
Func ReadFromPSPad4AutoIt3_INI($_sSection, $_sKey)
  Local $sRetVal = ""
  Local $sPSPad4AutoIt3_IniPath = _PathFull(@ScriptDir & '\..\PSPad4AutoIt3.ini')

  If FileExists($sPSPad4AutoIt3_IniPath) Then
    $sRetVal = IniRead($sPSPad4AutoIt3_IniPath, $_sSection, $_sKey, "")
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "File does not exist." & @CRLF & @CRLF & $sPSPad4AutoIt3_IniPath)
  EndIf

  Return($sRetVal)
EndFunc ; ==> ReadFromPSPad4AutoIt3_INI


; ==============================================================================
; ReadFromPSPad_INI                                                 2020-04-23
;
; Daten aus PSPad.ini im PSPad Verzeichnis lesen (wo die PSPad.exe liegt).
;
;   Rückgabe: Wert des Keys - oder "".
; ------------------------------------------------------------------------------
Func ReadFromPSPad_INI($_sSection, $_sKey)
  Local $sRetVal = ""
  Local $sPSPad_IniPath = _PathFull(@ScriptDir & '\..\..\PSPad.ini')

  If FileExists($sPSPad_IniPath) Then
    $sRetVal = IniRead($sPSPad_IniPath, $_sSection, $_sKey, "")
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "File does not exist." & @CRLF & @CRLF & $sPSPad_IniPath)
  EndIf

  Return($sRetVal)
EndFunc ; ==> ReadFromPSPad_INI


; ==============================================================================
; ReadFromPSPadLanguage_INI                                         2020-05-24
;
; Daten aus der jeweiligen Language-ini im PSPad\Language\ Verzeichnis lesen.
;
;   Rückgabe: Wert des Keys - oder "".
; ------------------------------------------------------------------------------
Func ReadFromPSPadLanguage_INI($_sLanguageName, $_sSection, $_sKey)
  Local $sRetVal = ""
  Local $sPSPadLanguage_IniPath = _
    _PathFull(@ScriptDir & '\..\..\Language\' & $_sLanguageName & '.ini')

  If FileExists($sPSPadLanguage_IniPath) Then
    $sRetVal = IniRead($sPSPadLanguage_IniPath, $_sSection, $_sKey, "")
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "File does not exist." & @CRLF & @CRLF & $sPSPadLanguage_IniPath)
  EndIf

  Return($sRetVal)
EndFunc ; ==> ReadFromPSPadLanguage_INI


; ==============================================================================
; RemoveRegionsAndComments
;
; Routine zum Entfernen aller #Region..#EndRegion- und Kommentar-Zeilen.
; ------------------------------------------------------------------------------
Func RemoveRegionsAndComments($_sText)

  If $_sText <> "" Then

    $_sText = StringRegExpReplace($_sText, '(?mi)^\h*(?:#Region|#EndRegion).*\R', "")

    $_sText = StringRegExpReplace($_sText, '(?m)^((?:[^''";]*([''"]).*?\g2)*[^;]*);?.*$', '$1')

    $_sText = StringRegExpReplace($_sText, _
      '(?xmi) (^\h*\# (?>cs|comments-start)\b (?sU:(?>(?R)|.)*) ^\h*\#(?>ce|comments-end)\b.*\R )', '')
  EndIf

  Return $_sText
EndFunc ; ==> RemoveRegionsAndComments


; ==============================================================================
; ResetOldVariables
;
; Routine zum Zurücksetzen der "Old"-Variablen in den Parametern (ByRef).
;
; ------------------------------------------------------------------------------
Func ResetOldVariables(ByRef $_sChkShowHideTxt_Old, ByRef $_sInpCurrentAu3Path_Old, _
  ByRef $_iFuncNameFirstCharPosX_Intern_Old)

  $_sChkShowHideTxt_Old = ""    ; Caret-Koordinaten, die vom VBScript geschickt wurden.
  $_sInpCurrentAu3Path_Old = "" ; Pfad zur aktuellen Au3, der vom VBScript geschickt wurde.

  $_iFuncNameFirstCharPosX_Intern_Old = 0

EndFunc ; ==> ResetOldVariables


; ==============================================================================
; ScrollSynSetsByArrowLabels
;
; Routine zum Scrollen, wenn auf Arrow-Labels geklickt wird.
;
; Bei Klick auf Up/Down:    Scrollen der SyntaxSets,
; bei Klick auf Left/Right: Scrollen des ChildGUIs (Syntax u Description).
;
; Rückgabe: SyntaxSet-Index und horizontale ScrollPos in den Parametern (ByRef).
; ------------------------------------------------------------------------------
Func ScrollSynSetsByArrowLabels($_iAdjustX_Arrw, $_iChildGuiWidth, $_aCallTipRect, _
  $_iSynSetsCount, ByRef $_iSynSetIdx, ByRef $_iScrollPosX)

  If BitAND(WinGetState($g_hCallTipWin), $WIN_STATE_VISIBLE) Then

    Local $bLeftOrRightWasClicked = False
    Local Const $iScrollDelay = 150

    If BitAND(GUICtrlGetState($g_LblUp), $GUI_SHOW) Then

      If LabelWasClicked($g_hCallTipWin, $g_LblUp, $_aCallTipRect[0], $_aCallTipRect[1], $iScrollDelay) Then

        $_iScrollPosX = 0 ; Beim Klicken auf Up/Down die ScrollPos für Left/Right zurücksetzen.

        If $_iSynSetIdx > 1 Then                   ; Index für das aktuellen SyntaxSet vermindern.
          $_iSynSetIdx -= 1
        EndIf

      ElseIf LabelWasClicked($g_hCallTipWin, $g_LblDown, $_aCallTipRect[0], $_aCallTipRect[1], $iScrollDelay) Then

        $_iScrollPosX = 0 ; Beim Klicken auf Up/Down die ScrollPos für Left/Right zurücksetzen.

        If $_iSynSetIdx < $_iSynSetsCount Then     ; Index für das aktuellen SyntaxSet erhöhen.
          $_iSynSetIdx += 1
        EndIf
      EndIf

    EndIf

    If BitAND(GUICtrlGetState($g_LblLeft), $GUI_SHOW) Then

      If LabelWasClicked($g_hCallTipWin, $g_LblLeft, $_aCallTipRect[0], $_aCallTipRect[1]) Then
        $bLeftOrRightWasClicked = True

        If $_iScrollPosX < 0 Then
          $_iScrollPosX += 20 ; Horizontale ScrollPos erhöhen (um ChildGUI nach rechts zurückzuschieben).
        EndIf

      ElseIf LabelWasClicked($g_hCallTipWin, $g_LblRight, $_aCallTipRect[0], $_aCallTipRect[1]) Then
        $bLeftOrRightWasClicked = True

        If Abs($_iScrollPosX) < ($_iChildGuiWidth - $_aCallTipRect[2] + $_iAdjustX_Arrw) Then
          $_iScrollPosX -= 20 ; Horizontale ScrollPos vermindern (um ChildGUI nach links zu verschieben).
        EndIf
      EndIf

      If $bLeftOrRightWasClicked Then
        $bLeftOrRightWasClicked = False

        _WinAPI_SetWindowPos($g_hChildGui, $HWND_BOTTOM, $_iAdjustX_Arrw + $_iScrollPosX, 0, _
          Default, Default, $SWP_NOSIZE) ; BitOR(... $SWP_NOACTIVATE, $SWP_NOOWNERZORDER ?

      EndIf
    EndIf ; If BitAND(GUICtrlGetState($g_LblLeft), $GUI_SHOW)
  EndIf ; If BitAND(WinGetState($g_hCallTipWin), $WIN_STATE_VISIBLE)
EndFunc ; ==> ScrollSynSetsByArrowLabels


; ==============================================================================
; ScrollSynSetsByKeyboard
;
;   Autor(s): alpines, 2020-05-25.
;             Professor Bernd (Formatierungen, Kommentare), 2020-05-26.
;
; Falls ein CallTip mit mehreren SyntaxSets angezeigt wird, kann mit
; erneutem Drücken/Festhalten des Hotkeys Shift+Ctrl+Space vorwärts
; durch die Sets gescrollt werden. Nach dem letzen Set wieder von vorne.
;
; Rückgabe: Als Parameter (ByRef).
;
;   $_iSynSetIdx:   Zum Scrollen durch die SyntaxSets.
;   $_iScrollPosX:  Zum Zurücksetzen der horizontalen ScrollPos, wenn
;                   SyntaxSets gescrollt wurden.
;   $_bScrollingOfSyntaxSetsAllowed: Damit beim ersten Drücken des
;                   Hotkeys nicht doppelt gescrollt wird.
;
; Vielen Dank an alpines, für das Erstellen dieser Routine und für's
; "Dranbleiben" bis alles lief wie gewünscht! 2020-06-06.
; ------------------------------------------------------------------------------
Func ScrollSynSetsByKeyboard($_iSynSetsCount, ByRef $_iSynSetIdx, _
  ByRef $_iScrollPosX, ByRef $_bScrollingOfSyntaxSetsAllowed)

  Local Static $bHotkeyIsHeld = False ; Neu 2020-05-25.
  Local Static $tmrHotkeyHeld = TimerInit()

  Local Const $iTimeUntilNexCallTip = 200

  If BitAND(GUICtrlGetState($g_LblUp), $GUI_SHOW) Then

    Local $bShiftPressed = _IsPressed("10", $g_hDLL)
    Local $bCtrlPressed  = _IsPressed("11", $g_hDLL)
    Local $bSpacePressed = _IsPressed("20", $g_hDLL)

    If Not $_bScrollingOfSyntaxSetsAllowed And (Not $bShiftPressed Or Not $bCtrlPressed Or Not $bSpacePressed) Then
      $_bScrollingOfSyntaxSetsAllowed = True
    ElseIf Not $_bScrollingOfSyntaxSetsAllowed Then
      Return
    EndIf

    If $bShiftPressed And $bCtrlPressed And $bSpacePressed _
      And (Not $bHotkeyIsHeld Or TimerDiff($tmrHotkeyHeld) >= $iTimeUntilNexCallTip) Then

      $bHotkeyIsHeld = True
      $tmrHotkeyHeld = TimerInit()

      $_iScrollPosX = 0                          ; ScrollPos für Left/Right zurücksetzen.

      If $_iSynSetIdx < $_iSynSetsCount Then     ; Index für das aktuellen SyntaxSet erhöhen.
        $_iSynSetIdx += 1
      ElseIf $_iSynSetIdx = $_iSynSetsCount Then ; Index beim letzten SyntaxSet zurücksetzen.
        $_iSynSetIdx = 1
      EndIf

    ElseIf (Not $bShiftPressed Or Not $bCtrlPressed Or Not $bSpacePressed) And $bHotkeyIsHeld Then
      $bHotkeyIsHeld = False
    EndIf
  EndIf ; If BitAND(GUICtrlGetState($g_LblUp), $GUI_SHOW)
EndFunc ; ==> ScrollSynSetsByKeyboard


; ------------------------------------------------------------------------------
; SetStateAndPosFor_ArrowLabels
;
; Routine für Arrow-Labels Anpassungen.
;
; Rückgabe: $iAdjustX_Arrw - Zum Anpassen der PosX von CallTipWin und
;           ChildGUI, abhängig davon, ob Arrow-Labels sichtbar sind.
;
; Falls Up/Down-Labels sichtbar sind, wird das CallTipWin entsprechend
; weiter nach links und das ChildGUI weiter nach rechts verschoben.
; ------------------------------------------------------------------------------
Func SetStateAndPosFor_ArrowLabels($_iSetsCount, $_iArrowLen, $_iChildGuiWidth, _
  $_iScreenWA_Width)

  Local $iAdjustX_Arrw = $g_iOffsetX
  Local $bUpDownArrowsAreVisible

  If ($_iSetsCount > 1) Then

    $iAdjustX_Arrw += $_iArrowLen

    GUICtrlSetPos($g_LblUp, 0)
    GUICtrlSetPos($g_LblDown, 0)

    GUICtrlSetState($g_LblUp, $GUI_SHOW)
    GUICtrlSetState($g_LblDown, $GUI_SHOW)

    $bUpDownArrowsAreVisible = True
  Else

    GUICtrlSetState($g_LblUp, $GUI_HIDE)
    GUICtrlSetState($g_LblDown, $GUI_HIDE)

    GUICtrlSetPos($g_LblUp, -1000)
    GUICtrlSetPos($g_LblDown, -1000)

    $bUpDownArrowsAreVisible = False
  EndIf

  If (($_iChildGuiWidth + $iAdjustX_Arrw + $_iArrowLen) > $_iScreenWA_Width) Then

    $iAdjustX_Arrw += $_iArrowLen

    If Not $bUpDownArrowsAreVisible Then
      GUICtrlSetPos($g_LblLeft, 0)
      GUICtrlSetPos($g_LblRight, 0)
    Else
      GUICtrlSetPos($g_LblLeft, $_iArrowLen)
      GUICtrlSetPos($g_LblRight, $_iArrowLen)
    EndIf

    GUICtrlSetState($g_LblLeft, $GUI_SHOW)
    GUICtrlSetState($g_LblRight, $GUI_SHOW)
  Else

    GUICtrlSetState($g_LblLeft, $GUI_HIDE)
    GUICtrlSetState($g_LblRight, $GUI_HIDE)

    GUICtrlSetPos($g_LblLeft, -1000)
    GUICtrlSetPos($g_LblRight, -1000)
  EndIf

  Return $iAdjustX_Arrw
EndFunc ; ==> SetStateAndPosFor_ArrowLabels


; ==============================================================================
; SetTextAndPosFor_SyntaxLabels
;
; Routine, um allen Syntax-Labels Text, Position und Maße zuzuweisen.
;
;   Die Syntax-Labels sind:
;
;   ... $g_LblSyntax, das die Syntax zur übermittelten Funktion anzeigt.
;   Es wird wird im CallTip ganz oben (PosY) angezeigt.
;
;   ... $g_Lbl_Red, das den aktuellen Parameter der übermittelten Funktion in
;   rot anzeigt. Es wird ebenfalls ganz oben (PosY) angezeigt und in der
;   (Z-Order) "über" dem Syntax-Label und mit roter Schrift. Dadurch entsteht
;   der Eindrück eines zweifarbigen Labels.
;
;   ... $g_LblDescr, das die Beschreibung zur übermittelten Funktion anzeigt.
;   Es wird unterhalb des Syntax-Labels angezeigt.
;
; Rückgabe: 1 - Wenn es Änderungen am CallTipWin gibt.
;           0 - Wenn es KEINE Änderungen am CallTipWin gibt.
;
;           Sowie die Werte in $_iChildGuiWidth und $_iChildGuiHeight (ByRef).
; ------------------------------------------------------------------------------
Func SetTextAndPosFor_SyntaxLabels($_aCurrentSyntaxSet_Parts, $_iParamNum, _
  $_bFuncNameHasChanged, ByRef $_iChildGuiWidth, ByRef $_iChildGuiHeight)

  Local $iStrWidth_Full, $iStrWidth_L, $iStrWidth_M, $iStrWidth_Descr

  Local $fFontSize_OUDF = $g_fFontSize - 0.5 ; 2021-03-27

  Local $bSyntaxHasChanged = False
  Local $bParamNumHasChanged = False
  Local $bDescriptionHasChanged = False
  Local $bColorSchemeHasChanged = False ; Zum Aktualisieren der Schriftfarbe vom Description-Label.

  Local Static $iParamNum_Old = 0

  Local $sSyntaxPart_L = $_aCurrentSyntaxSet_Parts[0]
  Local $sSyntaxPart_M = $_aCurrentSyntaxSet_Parts[1]
  Local $sSyntaxPart_R = $_aCurrentSyntaxSet_Parts[2]
  Local $sDescription  = $_aCurrentSyntaxSet_Parts[3]

  If $iParamNum_Old <> $_iParamNum Then
    $iParamNum_Old = $_iParamNum
    $bParamNumHasChanged = True
  EndIf

  If $_iParamNum = 0 Then

    Local $sTooManyParams = "  ---- Too many params! ----   "

    $sSyntaxPart_L = StringLeft($sSyntaxPart_L, StringInStr($sSyntaxPart_L, "("))
    $sSyntaxPart_M = $sTooManyParams
    $sSyntaxPart_R = ""
  EndIf

  Local $sSyntax_Full = $sSyntaxPart_L & $sSyntaxPart_M & $sSyntaxPart_R

  If $g_oDict_COM_CallTip.Item("CheckColorScheme") = True Then
    $g_oDict_COM_CallTip.Item("CheckColorScheme") = False
    $bColorSchemeHasChanged = True
    ColorScheme_ReadFromIniAndSetColorsForControls()
  EndIf

  If $g_bFontHasChanged Then
    $g_bFontHasChanged = False
    $_bFuncNameHasChanged = True
  EndIf

  If ($sSyntax_Full <> GUICtrlRead($g_LblSyntax)) Or $_bFuncNameHasChanged Then

    GUICtrlSetData($g_LblSyntax, $sSyntax_Full)

    $iStrWidth_Full = _StringSize($sSyntax_Full, $g_fFontSize, $FW_NORMAL, _
      $GUI_FONTNORMAL, $g_sFontName)[2]

    GUICtrlSetPos($g_LblSyntax, 0, 0, $iStrWidth_Full, $g_iLineHeight)

    $bSyntaxHasChanged = True
  EndIf

  If ($sSyntaxPart_M <> GUICtrlRead($g_Lbl_Red)) Or $bSyntaxHasChanged _
    Or $bParamNumHasChanged Then

    GUICtrlSetData($g_Lbl_Red, $sSyntaxPart_M)

    $iStrWidth_L = _StringSize($sSyntaxPart_L, $g_fFontSize, $FW_NORMAL, _
      $GUI_FONTNORMAL, $g_sFontName)[2]

    $iStrWidth_M = _StringSize($sSyntaxPart_M, $g_fFontSize, $FW_NORMAL, _
      $GUI_FONTNORMAL, $g_sFontName)[2]

    GUICtrlSetPos($g_Lbl_Red, $iStrWidth_L, 0, $iStrWidth_M, $g_iLineHeight)
  EndIf

  If ($sDescription <> GUICtrlRead($g_LblDescr)) Or $_bFuncNameHasChanged _
    Or $bSyntaxHasChanged Or $bColorSchemeHasChanged Then

    If StringRight(StringStripWS($sDescription, $STR_STRIPTRAILING), 5) = '.au3"' Then

      GUICtrlSetFont($g_LblDescr, $fFontSize_OUDF, $FW_NORMAL, $GUI_FONTNORMAL, $g_sFontName)
      GUICtrlSetColor($g_LblDescr, $g_hexDescriptionColor_OUDF)

      $iStrWidth_Descr = _StringSize($sDescription, $fFontSize_OUDF, $FW_NORMAL, _
        $GUI_FONTNORMAL, $g_sFontName)[2]

    Else ; Falls es eine Au3.api Funktion ist.
      GUICtrlSetFont($g_LblDescr, $g_fFontSize, $FW_NORMAL, $GUI_FONTNORMAL, $g_sFontName)
      GUICtrlSetColor($g_LblDescr, $g_hexDescriptionColor_Au3Api)

      $iStrWidth_Descr = _StringSize($sDescription, $g_fFontSize, $FW_NORMAL, _
        $GUI_FONTNORMAL, $g_sFontName)[2]
    EndIf

    GUICtrlSetData($g_LblDescr, $sDescription)

    GUICtrlSetPos($g_LblDescr, 0, $g_iLineHeight, $iStrWidth_Descr, $g_iLineHeight)

    $bDescriptionHasChanged = True
  EndIf

  If $bSyntaxHasChanged Or $bDescriptionHasChanged Then

    $iStrWidth_Full = _StringSize($sSyntax_Full, $g_fFontSize, $FW_NORMAL, _
      $GUI_FONTNORMAL, $g_sFontName)[2]

    If $iStrWidth_Full > $iStrWidth_Descr Then
      $_iChildGuiWidth = $iStrWidth_Full - $g_iOffsetX
    Else
      $_iChildGuiWidth = $iStrWidth_Descr - $g_iOffsetX
    EndIf

    $_iChildGuiHeight = ($g_iLineHeight * 2) + $g_iOffsetY

    GUISetState(@SW_HIDE, $g_hCallTipWin)
  EndIf

  If $bSyntaxHasChanged Or $bDescriptionHasChanged Or $bParamNumHasChanged Then
    Return 1
  Else
    Return 0
  EndIf
EndFunc ; ==> SetTextAndPosFor_SyntaxLabels


; ==============================================================================
; SplitSynEditCaretPos
;
; Routine zum Auslesen der SynEdit-CaretPos, die vom VBScript in die Caption
; der ChechBox "ShowHide" übertragen wurde.
; [0] enthält die Anzahl der Elemente.
; [1] u [2]: enthalten interne X u Y
; [3] u [4]: enthalten externe X u Y
;
; Es werden nun die internen und externen Koordinaten vom VBScript gesendet.
;
; Die Koordinaten werden benutzt, um in der Hauptschleife Code nur auszuführen,
; wenn sie sich verändert haben: "If $sChkShowHideTxt_Old <> $sChkShowHideTxt ..."
;
; Interne X/Y werden auch benötigt, falls das Caret am linken oder rechten Rand des
; SynEdit "auf der Stelle wandert". Wenn das Caret z. B. am rechten Rand ankommt und
; per Tastatur noch weiter nach rechts verschoben wird, verschiebt sich der sichtbare
; Bereich auch nach rechts, aber die externe Pos bleibt unverändert! Eine Änderung
; der CaretPos bliebe unbemerkt, wenn nur die externen Koordinaten geprüft würden!
;
; Die externen X/Y werden benutzt, um das CallTipWin zu positionieren.
;
; [Für das bisschen Code wäre keine eigene Routine nötig, aber es ist
;  eine gute Möglichkeit, die ganzen Infos unterzubringen. 2020-04-20.]
; ------------------------------------------------------------------------------
Func SplitSynEditCaretPos()

  Local $sChkShowHide_Text  ; Text der CheckBox "ShowHide".

  $sChkShowHide_Text = $g_sCaretPosCoordinates

  Return StringSplit($sChkShowHide_Text, "|")

EndFunc ; ==> SplitSynEditCaretPos


; ==============================================================================
; SplitSyntaxAndDescription
;
; Ein Syntax-Set besteht aus einem Syntax- und zugehörigen Description-Text.
; Beide Teile werden später im Code im jeweiligen Label angezeigt. Dabei soll
; für die Syntax der jeweils aktuelle Parameter in roter Schrift angezeigt
; werden. Um das zu erreichen, wird ein Label mit roter Schrift über dem
; Syntax-Label angezeigt (Z-Order!).
;
; Dieser Routine wird ein String übergeben, der 1 Syntax-Set enthält. Die
; Routine splitet das Set in Syntax und Beschreibung, und jeden Syntax-Text
; in 3 Teile: Links, Mitte, Rechts.
;
; Die Rückgabe erfolgt als Array ("$asTxt[4]") mit folgenden 4 Elementen:
;
;  [0] TextPart-L:    Linker Syntax-Teil, der vor dem roten Label steht.
;
;  [1] TextPart-M:    Mittlerer Syntax-Teil, der in einem roten Label (in der
;                     Z-Order) über dem Syntax-Label gezeigt wird.
;
;  [2] TextPart-R:    Rechter Syntax-Teil, um den vollständigen Syntax-Text
;                     zusammenfügen zu können (in SetTextAndPosFor_SyntaxLabels()).
;                     Der vollständige Syntax-Text wird im Syntax-Label angezeigt.
;
;  [3] TextPart-Desc: Der Beschreibungs-Text, der in einem eigenen Label
;                     unterhalb des Syntax-Labels angezeigt wird.
; ------------------------------------------------------------------------------
Func SplitSyntaxAndDescription($_sSyntaxAndDescription, $_iParamNum)

  Local $iDelimPos_1, $iDelimPos_2, $iDelimPos_CB ; $iDelimPos_CB = closing bracket
  Local $asTxt[4] ; Array für Rückgabe-Werte.

  If $_iParamNum < 2 Then
    $iDelimPos_1 = StringInStr($_sSyntaxAndDescription, ' ( ') + 1 ; Erstes Vorkommen von links + 1.
  Else
    $iDelimPos_1 = StringInStr($_sSyntaxAndDescription, ',', 0, $_iParamNum - 1)
  EndIf

  $iDelimPos_2 = StringInStr($_sSyntaxAndDescription, ',', 0, $_iParamNum)

  If StringRight(StringStripWS($_sSyntaxAndDescription, $STR_STRIPTRAILING), 5) = '.au3"' Then
    $iDelimPos_CB = StringInStr($_sSyntaxAndDescription, '"', 0, -2) - 3
  Else

    $iDelimPos_CB = StringInStr($_sSyntaxAndDescription, ' ) ')

    If $iDelimPos_CB = 0 Then $iDelimPos_CB = StringInStr($_sSyntaxAndDescription, ') ') - 1
  EndIf

  If ($iDelimPos_2 = 0) Or ($iDelimPos_2 > $iDelimPos_CB) Then $iDelimPos_2 = $iDelimPos_CB

  If StringMid($_sSyntaxAndDescription, $iDelimPos_2 - 1, 1) = '[' Then
    $iDelimPos_2 = $iDelimPos_2 - 1
  Else
    If StringMid($_sSyntaxAndDescription, $iDelimPos_2 - 1, 1) = ']' Then
      $iDelimPos_2 = StringInStr($_sSyntaxAndDescription, ']')
    EndIf
  EndIf

  $asTxt[0] = StringLeft($_sSyntaxAndDescription, $iDelimPos_1)                     ; SyntaxPart_L

  $asTxt[1] = StringMid($_sSyntaxAndDescription, $iDelimPos_1 + 1, _
    $iDelimPos_2 - $iDelimPos_1 - 1)                                                ; SyntaxPart_M

  $asTxt[2] = StringMid($_sSyntaxAndDescription, $iDelimPos_2, _
    $iDelimPos_CB - $iDelimPos_2 + StringLen(' ) '))                                ; SyntaxPart_R

  $asTxt[3] = StringMid($_sSyntaxAndDescription, $iDelimPos_CB + StringLen(' ) '))
  $asTxt[3] = " " & $asTxt[3] & " "                                                 ; Description

  Return($asTxt)

EndFunc ; ==> SplitSyntaxAndDescription


; ==============================================================================
; ToolTipWithTimeOut_Show und, ToolTipWithTimeOut_Stop
;
; Zwei Routinen zum Anzeigen eines ToolTips mit Timeout. Der ToolTip wird
; mithilfe eines Adlib-Timers nach dem Timeout geschlossen.
; ------------------------------------------------------------------------------
Func ToolTipWithTimeOut_Show($_sText, $_iX = Default, $_iY = Default, $_sTitel = "", _
  $_iTimeout = 2000, $_iIcon = $TIP_INFOICON, $_iOptions = $TIP_CENTER)

  ToolTip($_sText, $_iX, $_iY, $_sTitel, $_iIcon, $_iOptions) ; ToolTip zeigen.
  AdlibRegister("ToolTipWithTimeOut_Stop", $_iTimeout)        ; Adlib-Timer starten.
EndFunc ; ==> ToolTipWithTimeOut_Show

;
; Adlib-Event: ToolTip und Adlib-Timer beim ersten Event ausschalten.
Func ToolTipWithTimeOut_Stop()
    ToolTip("")                                               ; ToolTip verstecken.
    AdlibUnRegister("ToolTipWithTimeOut_Stop")                ; Adlib-Timer stoppen.
EndFunc ; ==> ToolTipWithTimeOut_Stop

