; #AutoIt3Wrapper_Au3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w- 7

; = Info ===========================================================================================
; Title           : FileTypesRegister.au3
; --------------------------------------------------------------------------------------------------
;
; Version (vnots) : 0.0.21
; Date            : 2021-03-27
; Author(s)       : Professor Bernd
;
; Requirements    :
; Language        :
; Description     : Zuordnen/registrieren der Dateitypen ".au3" und ".ppr" zu/für PSPad4AutoIt3.
; ==================================================================================================

  #cs
    ----------------------------------------------------------------------------
   | Die folgenden Informationen stammen aus meinem Thread
   | "Dateitypen für eigenes Programm registrieren"
   | https://autoit.de/thread/87282-dateitypen-für-eigenes-programm-registrieren
   | Beitrag #16, vom 2021-03-08.
   | https://autoit.de/thread/87282-dateitypen-f%C3%BCr-eigenes-programm-registrieren/?postID=703132#post703132
   |
   | Die Infos habe ich zusammengetragen aus Recherchen, MS Docs und vielen
   | eigenen Tests. 2021-03-11.
    ----------------------------------------------------------------------------

    Übersetzter Auszug aus  MS Docs "HKEY_CLASSES_ROOT Key"
    https://docs.microsoft.com/en-us/windows/win32/sysinfo/hkey-classes-root-key
    https://docs.microsoft.com/en-us/windows/win32/sysinfo/merged-view-of-hkey-classes-root

    HKEY_CLASSES_ROOT                   <= bietet eine Ansicht der Registry, die die Informationen
                                            aus den beiden folgenden Quellen zusammenführt.
    HKEY_CURRENT_USER\Software\Classes  <= enthält Einstellungen, die nur für den interaktiven
                                            Benutzer gelten.
    HKEY_LOCAL_MACHINE\Software\Classes <= enthält Standardeinstellungen, die für alle Benutzer
                                            auf dem lokalen Computer gelten können.


    HKEY_CLASSES_ROOT merged also die Einstellungen aus den beiden anderen Registry-Zweigen.
    Dabei gilt, dass HKCU Keys die Keys aus HKLM überschreiben. Das bedeutet aber NICHT,
    dass man seinen Kram einfach in HKCU schreiben kann und keiner kann einem was! AutoIt
    hat in HKLM weitere Schlüssel angelegt, die dann wiederum das Verhalten "übertrumpft",
    das man mit den Schlüsseln von HKCU erreichen wollte!

    Hier meine Überlegungen, um Einträge in die Registry zu schreiben, damit .au3
    Dateien per Doppelklick mit PSPad geöffnet werden.

    Möglich wäre das Schreiben in die oben genannten 3 Schlüssel: HKCR, HKCU und HKLM.
    Um unerwünschte Ergebnisse zu vermeiden, vor denen die genannten MS Docs warnen,
    will ich HKCR NICHT benutzen. Bleiben also noch HKCU und HKLM. - Vorerst habe ich
    HKCU im Sinn, was folgende Vorteile hat:

      1. Es werden KEINE Admin-Rechte benötigt, um Einträge zu schreiben.
      2. PSPad schreibt mit seiner eigenen Registrier-Funktion seine Einträge in
          HKCU, weshalb sich dessen Einträge dann mit meinen "vertragen". Wer weiß
          was passiert, wenn ich in HKLM schreiben würde und PSPad in HKCU schreibt!? ;-)
      3. Wenn ich in HKCU schreibe, werden keine Schlüssel in HKLM überschrieben,
          geändert oder gelöscht! Dadurch ist ein Rückgängigmachen möglich, bei
          dem der vorherigen Zustand wiederhergestellt werden kann. (AutoIt3
          schreibt seine Einstellungen in HKLM.)

    Für unsere Zwecke habe ich mir angesehen, was AutoIt und PSPad in die Registry
    schreiben. (SciTE fügt zwar eigene Schlüssel hinzu, ändert aber nichts an den
    vorhandenen und wird von mir hier nicht extra erwähnt.)

    - AutoIt schreibt in HKLM. Für uns relevant sind die 2 Einträge: ".au3" und "AutoIt3Script".
    - PSPad  schreibt in HKCU. Für uns relevant sind die 2 Einträge: ".au3" und "au3file".

    Laut MS Docs sollten die Einträge in HKCU die Einträge in HKLM überschreiben. Das
    tun sie auch, aber nur, wenn sie in beiden Zweigen vorhanden sind. Das trifft aber
    für "AutoIt3Script" NICHT zu. Damit unsere Einträge für PSPad wirksam werden, muss
    somit auch in HKCU ein Eintrag für "AutoIt3Script" erstellt werden, mit PSPad als
    Anwendung in "Shell\Open\Commend". Somit sind für PSPad 3 Einträge zu schreiben:

    ".au3"
    "au3file"
    "AutoIt3Script"

    Ich liebe Ordnung! :) Mit diesen Einträge wäre es wie o.g. möglich, beim Deregistrieren
    diese 3 Einträge in HKCU zu löschen und dadurch den vorherigen Zustand wiederherzustellen,
    sodass z.B. .au3 Dateien per Doppelklick wieder mit SciTE geöffnet würden.

    2021-03-08, 2021-03-11.


    ---- AU3 ----

    Die konkreten 3 (+ DefaultIcon) Schlüssel in HKEY_CURRENT_USER (HKCU) zum
    Registrieren per AutoIt-Code:  ($sPSPad_Path = Pfad zur eigenen PSPad.exe)

      ; Dateiendung ".au3" als Dateityp "au3file" registrieren.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\.au3", "", "REG_SZ", "au3file")

      ; FileType "au3file" mit dem Icon 2 von PSPad.exe registrieren. Das Icon wird im Win-Explorer für ".au3"-Dateien gezeigt.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\au3file\DefaultIcon", "", "REG_SZ", $sPSPad_Path & ',2')

      ; FileType "au3file" zum Öffnen (z.B. per Doppelklick) mit PSPad registrieren.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\au3file\Shell\Open\Command", "", "REG_SZ", '"' & $sPSPad_Path & '" "%1"')

      ; FileType "AutoIt3Script" zum Öffnen (z.B. per Doppelklick) mit PSPad registrieren.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\AutoIt3Script\Shell\Open\Command", "", "REG_SZ", '"' & $sPSPad_Path & '" "%1"')


    ---- PPR ----

    Für ".ppr"-Dateien genügen 2 Schlüssel (+ DefaulIcon), da AutoIt keine ".ppr"-Dateien kennt/registriert.
    AutoIt-Code zum Registrieren:  ($sPSPad_Path = Pfad zur eigenen PSPad.exe)

      ; Dateiendung ".ppr" als Dateityp "PSPad.project" registrieren.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\.ppr", "", "REG_SZ", "PSPad.project")

      ; FileType "PSPad.project" mit dem Icon 1 von PSPad.exe registrieren. Das Icon wird im Win-Explorer für ".ppr"-Dateien gezeigt.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\PSPad.project\DefaultIcon", "", "REG_SZ", $sPSPad_Path & ',2')

      ; FileType "PSPad.project" zum Öffnen (z.B. per Doppelklick) mit PSPad registrieren.
      RegWrite("HKEY_CURRENT_USER\Software\Classes\PSPad.project\Shell\Open\Command", "", "REG_SZ", '"' & $sPSPad_Path & '" "%1"')

    2021-03-08, 2021-03-11.
  #ce


#NoTrayIcon
#include <ComboConstants.au3>
#include <File.au3>
#include <GUIConstantsEx.au3>
#include <Misc.au3>               ; Für z.B. _Singleton().
#include <MsgBoxConstants.au3>
#include <StaticConstants.au3>
#include <WinAPISysWin.au3>
#include <WindowsConstants.au3>

_Singleton("PSPad4AutoIt3_FileTypesRegister")

Opt("MustDeclareVars", 1)

OnAutoItExitRegister('OnAutoItExit')

Global $g_hPSPad ; Wird auch in OnAutoItExit() benutzt.

Main_Func()


; ==============================================================================
; Main_Func
;
; Main function. ;-)
; ------------------------------------------------------------------------------
Func Main_Func()

  Local $sMsgGuiTitle                 = "PSPad4AutoIt3 FileTypesRegister"

  Local $sTitle_English               = "Register file types for PSPad4AutoIt3"
  Local $sTitle_German                = "Dateitypen registrieren für PSPad4AutoIt3"

  Local $sLblDeregistered_English     = "not" & @CRLF & "registered"    ; 'Deregistered' Text EN.
  Local $sLblDeregistered_German      = "nicht" & @CRLF & "registriert" ; 'Deregistered' Text DE.
  Local $sLblRegistered_English       = "registered"                    ; 'Registered'   Text EN.
  Local $sLblRegistered_German        = "registriert"                   ; 'Registered'   Text DE.

  Local $sAU3_InfoTxt_English         = "(recommended) " & @CRLF _
                                        & "+ Au3 files can be opened by double click with PSPad4Au3." & @CRLF _
                                        & "+ Examples in the AutoIt help can be opened directly in PSPad4Au3."
  Local $sAU3_InfoTxt_German          = "(empfohlen) " & @CRLF _
                                        & "+ Au3-Dateien können per Doppelklick mit PSPad4Au3 geöffnet werden." & @CRLF _
                                        & "+ Beispiele in der AutoIt-Hilfe können direkt in PSPad4Au3 geöffnet werden."
  Local $sPPR_InfoTxt_English         = "(recommended) " & @CRLF _
                                        & "+ PSPad project files can be opened by double click with PSPad4Au3." & @CRLF _
                                        & "+ All opened files will be automatically reopened at the next PSPad start." _
                                        & " - Manual saving is NOT necessary!"
  Local $sPPR_InfoTxt_German          = "(empfohlen) " & @CRLF _
                                        & "+ PSPad-Projekt-Dateien können per Doppelklick mit PSPad4Au3 geöffnet werden." & @CRLF _
                                        & "+ Alle geöffneten Dateien werden beim nächsten PSPad-Start automatisch wieder geöffnet." _
                                        & " - Manuelles Speichern ist NICHT nötig!"

  Local $sBtnDeregister_English       = "Deregister selected"
  Local $sBtnDeregister_German        = "Deregistriere ausgewählte"
  Local $sBtnRegister_English         = "Register selected"
  Local $sBtnRegister_German          = "Registriere ausgewählte"

  Local $hGUI
  Local $cmbLanguage
  Local $lblTitle, $lblAU3_Info, $lblPPR_Info, $lblAU3_Registered, $lblPPR_Registered
  Local $chkAU3, $chkPPR
  Local $btnDeregister, $btnRegister

  Local $nMsg

  Local $bDebugIsOn ;= True

  $g_hPSPad = WinGetHandle("[active]")  ; Parent-Fenster für die eigene GUI.

  If _WinAPI_IsClassName($g_hPSPad, "TfPSPad") Then

    $hGUI = GUICreate("", 586, 300, -1, -1, BitOR($WS_CAPTION, $WS_POPUPWINDOW), -1, $g_hPSPad)
    WinSetTitle($hGUI, "", $sMsgGuiTitle)
    GUISetFont(9, 0, 0, "Segoe UI", $hGUI) ; "Segoe UI", "MS Sans Serif"

    $lblTitle = GUICtrlCreateLabel("", 10, 10, 452, 34, BitOR($SS_CENTER, $SS_CENTERIMAGE))
    If $bDebugIsOn Then GUICtrlSetStyle(-1, Default, $WS_EX_STATICEDGE) ; <== Nur für Debug.
    GUICtrlSetFont(-1, 14, 800, 0, "Segoe UI") ; "Segoe UI", "MS Sans Serif"
    GUICtrlSetData(-1, $sTitle_English)

    $cmbLanguage = GUICtrlCreateCombo("", 462, 16, 100, 25, _
      BitOR($CBS_DROPDOWNLIST, $CBS_AUTOHSCROLL, $WS_VSCROLL)) ; ComboBox read only.
    GUICtrlSetData(-1, "English|German", "English")
    GUICtrlSetFont(-1, 8.5)

    $chkAU3 = GUICtrlCreateCheckbox(".au3", 24, 72, 64)
    GUICtrlSetState($chkAU3, $GUI_CHECKED)

    $lblAU3_Registered = GUICtrlCreateLabel("registered", 10, 98, 80, 54, $SS_CENTER)
    If $bDebugIsOn Then GUICtrlSetStyle(-1, Default, $WS_EX_STATICEDGE) ; <== Nur für Debug.

    $lblAU3_Info = GUICtrlCreateLabel("", 112, 72, 456, 80)
    If $bDebugIsOn Then GUICtrlSetStyle(-1, Default, $WS_EX_STATICEDGE) ; <== Nur für Debug.
    GUICtrlSetData(-1, $sAU3_InfoTxt_English)

    $chkPPR = GUICtrlCreateCheckbox(".ppr", 24, 152, 64)
    GUICtrlSetState($chkPPR, $GUI_CHECKED)

    $lblPPR_Registered = GUICtrlCreateLabel("not registered", 10, 178, 80, 54, $SS_CENTER)
    If $bDebugIsOn Then GUICtrlSetStyle(-1, Default, $WS_EX_STATICEDGE) ; <== Nur für Debug.

    $lblPPR_Info = GUICtrlCreateLabel("", 112, 152, 456, 92)
    If $bDebugIsOn Then GUICtrlSetStyle(-1, Default, $WS_EX_STATICEDGE) ; <== Nur für Debug.
    GUICtrlSetData(-1, $sPPR_InfoTxt_English)

    $btnDeregister = GUICtrlCreateButton($sBtnDeregister_English, 24, 248, 252, 38)
    $btnRegister = GUICtrlCreateButton($sBtnRegister_English, 308, 248, 252, 38)

    ControlFocus($hGUI, "", $lblTitle)

    CheckFileTypesAreRegisteredForOwnPSPad($cmbLanguage, $lblAU3_Registered, $lblPPR_Registered, _
      $sLblDeregistered_English, $sLblRegistered_English, $sLblDeregistered_German, $sLblRegistered_German)

    GUISetState(@SW_SHOW, $hGUI)

    WinSetState($g_hPSPad, "", @SW_DISABLE) ; Parent-Fenster disablen.

    While 1
      $nMsg = GUIGetMsg()
      Switch $nMsg

        Case $GUI_EVENT_CLOSE
          ExitLoop

        Case $cmbLanguage

          Switch GUICtrlRead($cmbLanguage)

            Case "English"
              GUICtrlSetData($lblTitle, $sTitle_English)
              GUICtrlSetData($lblAU3_Info, $sAU3_InfoTxt_English)
              GUICtrlSetData($lblPPR_Info, $sPPR_InfoTxt_English)
              GUICtrlSetData($btnDeregister, $sBtnDeregister_English)
              GUICtrlSetData($btnRegister, $sBtnRegister_English)

            Case "German"
              GUICtrlSetData($lblTitle, $sTitle_German)
              GUICtrlSetData($lblAU3_Info, $sAU3_InfoTxt_German)
              GUICtrlSetData($lblPPR_Info, $sPPR_InfoTxt_German)
              GUICtrlSetData($btnDeregister, $sBtnDeregister_German)
              GUICtrlSetData($btnRegister, $sBtnRegister_German)

          EndSwitch

          CheckFileTypesAreRegisteredForOwnPSPad($cmbLanguage, $lblAU3_Registered, $lblPPR_Registered, _
            $sLblDeregistered_English, $sLblRegistered_English, $sLblDeregistered_German, $sLblRegistered_German, False)

        Case $btnDeregister, $btnRegister

          GUICtrlSetState($chkAU3, $GUI_DISABLE)
          GUICtrlSetState($chkPPR, $GUI_DISABLE)
          GUICtrlSetState($btnDeregister, $GUI_DISABLE)
          GUICtrlSetState($btnRegister, $GUI_DISABLE)

          GUISetCursor(15, 1, $hGUI) ; 15 = WAIT (Sanduhr)

          If $nMsg = $btnDeregister Then ; "Deregister"
            If GUICtrlRead($chkAU3) = $GUI_CHECKED Then FileTypeDeregisterForOwnPSPad(".au3")
            If GUICtrlRead($chkPPR) = $GUI_CHECKED Then FileTypeDeregisterForOwnPSPad(".ppr")
          ElseIf $nMsg = $btnRegister Then ; "Register"
            If GUICtrlRead($chkAU3) = $GUI_CHECKED Then FileTypeRegisterForOwnPSPad(".au3")
            If GUICtrlRead($chkPPR) = $GUI_CHECKED Then FileTypeRegisterForOwnPSPad(".ppr")
          EndIf

          GUICtrlSetState($chkAU3, $GUI_UNCHECKED)
          GUICtrlSetState($chkPPR, $GUI_UNCHECKED)

          ClearWindowsIconCache()

          CheckFileTypesAreRegisteredForOwnPSPad($cmbLanguage, $lblAU3_Registered, $lblPPR_Registered, _
            $sLblDeregistered_English, $sLblRegistered_English, $sLblDeregistered_German, $sLblRegistered_German)

          GUISetCursor(2, 1, $hGUI) ; 2 = ARROW (normaler Cursor)

          GUICtrlSetState($chkAU3, $GUI_ENABLE)
          GUICtrlSetState($chkPPR, $GUI_ENABLE)

        Case $chkAU3, $chkPPR
          If GUICtrlRead($chkAU3) = $GUI_CHECKED Or GUICtrlRead($chkPPR) = $GUI_CHECKED Then
            GUICtrlSetState($btnDeregister, $GUI_ENABLE )
            GUICtrlSetState($btnRegister, $GUI_ENABLE )
          Else
            GUICtrlSetState($btnDeregister, $GUI_DISABLE)
            GUICtrlSetState($btnRegister, $GUI_DISABLE)
          EndIf

      EndSwitch
    WEnd
  Else ; Falls das Script NICHT von PSPad gestartet wurde.
    MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, "PSPad4AutoIt3 - " & @ScriptName, _
      "Function 'Main_Func'" & @CRLF & @CRLF & "This script must be started from PSPad.")
  EndIf ; If _WinAPI_IsClassName($g_hPSPad, "TfPSPad")
EndFunc ; ==> Main_Func


; ==============================================================================
; CheckFileTypesAreRegisteredForOwnPSPad
;
; Routine zum Ermitteln, ob AU3-/PPR-Dateien in der Registry mit PSPad4AutoIt3
; verbunden sind und zum Setzen des Textes für die 'Registered'-Labels.
; ------------------------------------------------------------------------------
Func CheckFileTypesAreRegisteredForOwnPSPad($_cmbLanguage, $_lblAU3_Registered, $_lblPPR_Registered, _
  $_sLblDeregistered_English, $_sLblRegistered_English, $_sLblDeregistered_German, $_sLblRegistered_German, _
  $bUpdateFromRegistry = True)

  Local Static $bIsRegistered_AU3, $bIsRegistered_PPR

  If $bUpdateFromRegistry Then $bIsRegistered_AU3 = FileTypeIsRegisteredForOwnPSPad(".au3")

  If $bUpdateFromRegistry Then $bIsRegistered_PPR = FileTypeIsRegisteredForOwnPSPad(".ppr")

  Switch GUICtrlRead($_cmbLanguage)

    Case "English"

      If Not $bIsRegistered_AU3 Then
        GUICtrlSetData($_lblAU3_Registered, $_sLblDeregistered_English)
        GUICtrlSetColor($_lblAU3_Registered, 0xCC0000) ; mitteldunkel Rot
      Else ; AU3-Dateien sind registriert.
        GUICtrlSetData($_lblAU3_Registered, $_sLblRegistered_English)
        GUICtrlSetColor($_lblAU3_Registered, 0x00CC00) ; mittleres Grün
      EndIf

      If Not $bIsRegistered_PPR Then
        GUICtrlSetData($_lblPPR_Registered, $_sLblDeregistered_English)
        GUICtrlSetColor($_lblPPR_Registered, 0xCC0000) ; mitteldunkel Rot
      Else ; PPR-Dateien sind registriert.
        GUICtrlSetData($_lblPPR_Registered, $_sLblRegistered_English)
        GUICtrlSetColor($_lblPPR_Registered, 0x00CC00) ; mittleres Grün
      EndIf

    Case "German"

      If Not $bIsRegistered_AU3 Then
        GUICtrlSetData($_lblAU3_Registered, $_sLblDeregistered_German)
        GUICtrlSetColor($_lblAU3_Registered, 0xCC0000) ; mitteldunkel Rot
      Else ; AU3-Dateien sind registriert.
        GUICtrlSetData($_lblAU3_Registered, $_sLblRegistered_German)
        GUICtrlSetColor($_lblAU3_Registered, 0x00CC00) ; mittleres Grün
      EndIf

      If Not $bIsRegistered_PPR Then
        GUICtrlSetData($_lblPPR_Registered, $_sLblDeregistered_German)
        GUICtrlSetColor($_lblPPR_Registered, 0xCC0000) ; mitteldunkel Rot
      Else ; PPR-Dateien sind registriert.
        GUICtrlSetData($_lblPPR_Registered, $_sLblRegistered_German)
        GUICtrlSetColor($_lblPPR_Registered, 0x00CC00) ; mittleres Grün
      EndIf

  EndSwitch
EndFunc ; ==> CheckFileTypesAreRegisteredForOwnPSPad


; ==============================================================================
; ClearWindowsIconCache
;
; Routine zum Bereinigen des Windows Icon Cache und zum Aktualisieren der Icons
; des Windows Explorers.
;
; ------------------------------------------------------------------------------
Func ClearWindowsIconCache()

  Local $iErrors = 0

  If @OSVersion = "WIN_10" Then
    If @OSArch = "X86" Then ; x86
      RunWait(@COMSPEC & " /c ie4uinit.exe -show", "", @SW_HIDE)
    Else ; "IA64","X64"
      RunWait(@ComSpec & " /c " & @WindowsDir & "\Sysnative\ie4uinit.exe -show", "", @SW_HIDE)
    EndIf
    $iErrors += @error
  Else
    If @OSArch = "X86" Then ; x86
      RunWait(@COMSPEC & " /c ie4uinit.exe -ClearIconCache", "", @SW_HIDE)
    Else ; "IA64","X64"
      RunWait(@ComSpec & " /c " & @WindowsDir & "\Sysnative\ie4uinit.exe -ClearIconCache", "", @SW_HIDE)
    EndIf
    $iErrors += @error
  EndIf

  If $iErrors > 0 Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "Function 'ClearWindowsIconCache'" & @CRLF & @CRLF & "Error cleaning the Windows icon cache.")
  EndIf
EndFunc ; ==> ClearWindowsIconCache


; ==============================================================================
; FileTypeDeregisterForOwnPSPad
;
; Routine zum Deregistrieren einer Datei-Endungen für PSPad4Au3.
; ------------------------------------------------------------------------------
Func FileTypeDeregisterForOwnPSPad($_sExt)

  Local $iErrors = 0
  Local $sRegKey
  Local $sFileType = ""
  Local $sOwnPSPad_Path = _PathFull(@ScriptDir & '\..\..\PSPad.exe')

  If FileExists($sOwnPSPad_Path) Then

    If StringLeft($_sExt, 1) <> "." Then $_sExt = "." & $_sExt

    If $_sExt = ".au3" Then $sFileType = "au3file"
    If $_sExt = ".ppr" Then $sFileType = "PSPad.project"
    If $sFileType = "" Then Return

    $sRegKey = "HKEY_CURRENT_USER\Software\Classes\" & $_sExt
    If RegKeyExists($sRegKey) Then
      RegDelete($sRegKey)
      $iErrors += @error
    EndIf

    $sRegKey = "HKEY_CURRENT_USER\Software\Classes\" & $sFileType
    If RegKeyExists($sRegKey) Then
      RegDelete($sRegKey)
      $iErrors += @error
    EndIf

    If $_sExt = ".au3" Then
      $sRegKey = "HKEY_CURRENT_USER\Software\Classes\AutoIt3Script"
      If RegKeyExists($sRegKey) Then
        If RegRead($sRegKey & "\Shell\Open\Command", "") = '"' & $sOwnPSPad_Path & '" "%1"' Then
          RegDelete($sRegKey)
          $iErrors += @error
        EndIf
      EndIf
    EndIf

    If $iErrors > 0 Then
      MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
        "Function 'FileTypeDeregisterForOwnPSPad'" & @CRLF & @CRLF & "Error deleting the registry keys.")
    EndIf
  Else ; Falls die eigene PSPad.exe NICHT existiert.
    $iErrors = 1
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "Function 'FileTypeDeregisterForOwnPSPad'" & @CRLF & @CRLF _
      & "File does not exist." & @CRLF & @CRLF & '"' & $sOwnPSPad_Path & '"')
  EndIf ; If FileExists($sOwnPSPad_Path)

  Return $iErrors
EndFunc ; ==> FileTypeDeregisterForOwnPSPad


; ==============================================================================
; FileTypeIsRegisteredForOwnPSPad
;
; Routine zum Ermitteln, ob AU3- oder PPR-Dateien in der Registry eingetragen
; und für die eigene PSPad.exe registriert sind.
;
; Die Einträge, mit denen PSPad für ".au3" und ".ppr" Dateien registiert werden,
; bestehen aus folgenden Schlüsselpaaren:
;
; "HKEY_CURRENT_USER\"
; ------------------------------------------------------------------------------
Func FileTypeIsRegisteredForOwnPSPad($_sExt)

  Local $sFileType, $sRegisteredProg, $sOwnPSPad_Path
  Local $bReturn = False

  $sOwnPSPad_Path = _PathFull(@ScriptDir & '\..\..\PSPad.exe')

  If FileExists($sOwnPSPad_Path) Then

    If StringLeft($_sExt, 1) <> "." Then $_sExt = "." & $_sExt

    $sFileType = RegRead("HKEY_CURRENT_USER\Software\Classes\" & $_sExt, "")
    If Not @error Then
      $sRegisteredProg = RegRead("HKEY_CURRENT_USER\Software\Classes\" & $sFileType & "\Shell\Open\Command", "")
      If Not @error Then
        If $sRegisteredProg = '"' & $sOwnPSPad_Path & '" "%1"' Then
          $bReturn = True
        EndIf
        If $bReturn And $_sExt = ".au3" Then
          $sRegisteredProg = RegRead("HKEY_CURRENT_USER\Software\Classes\AutoIt3Script\Shell\Open\Command", "")
          If Not @error Then
            If $sRegisteredProg <> '"' & $sOwnPSPad_Path & '" "%1"' Then
              $bReturn = False
            EndIf
          EndIf
        EndIf
      EndIf
    EndIf
  Else ; Falls die eigene PSPad.exe NICHT existiert.
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "Function 'FileTypeIsRegisteredForOwnPSPad'" & @CRLF & @CRLF _
      & "File does not exist." & @CRLF & @CRLF & '"' & $sOwnPSPad_Path & '"')
  EndIf ; If FileExists($sOwnPSPad_Path)

  Return $bReturn
EndFunc ; ==> FileTypeIsRegisteredForOwnPSPad


; ==============================================================================
; FileTypeRegisterForOwnPSPad
;
; Routine zum Registrieren einer Datei-Endungen für PSPad4Au3.
; ------------------------------------------------------------------------------
Func FileTypeRegisterForOwnPSPad($_sExt)

  Local $iErrors = 0
  Local $sIconIdx
  Local $sFileType = ""
  Local $sOwnPSPad_Path = _PathFull(@ScriptDir & '\..\..\PSPad.exe')

  If FileExists($sOwnPSPad_Path) Then

    If StringLeft($_sExt, 1) <> "." Then $_sExt = "." & $_sExt

    If $_sExt = ".au3" Then
      $sIconIdx = 2
      $sFileType = "au3file"
    ElseIf $_sExt = ".ppr" Then
      $sIconIdx = 1
      $sFileType = "PSPad.project"
    Else
      Return
    EndIf

    RegWrite("HKEY_CURRENT_USER\Software\Classes\" & $_sExt, "", "REG_SZ", $sFileType)
    $iErrors += @error

    RegWrite("HKEY_CURRENT_USER\Software\Classes\" & $sFileType & "\DefaultIcon", _
      "", "REG_SZ", $sOwnPSPad_Path & "," & $sIconIdx) ; <= Pfad NICHT mit Quotes umschließen.
    $iErrors += @error

    RegWrite("HKEY_CURRENT_USER\Software\Classes\" & $sFileType & "\Shell\Open\Command", _
      "", "REG_SZ", '"' & $sOwnPSPad_Path & '" "%1"') ; <= Pfade mit Quotes umschließen.
    $iErrors += @error

    If $_sExt = ".au3" Then
      RegWrite("HKEY_CURRENT_USER\Software\Classes\AutoIt3Script\Shell\Open\Command", _
        "", "REG_SZ", '"' & $sOwnPSPad_Path & '" "%1"') ; <= Pfad mit Quotes umschließen.
      $iErrors += @error
    EndIf

    If $iErrors > 0 Then
      MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
        "Function 'FileTypeRegisterForOwnPSPad'" & @CRLF & @CRLF & "Error writing the registry keys.")
    EndIf
  Else ; Falls die eigene PSPad.exe NICHT existiert.
    $iErrors = 1
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "Function 'FileTypeRegisterForOwnPSPad'" & @CRLF & @CRLF _
      & "File does not exist." & @CRLF & @CRLF & '"' & $sOwnPSPad_Path & '"')
  EndIf ; If FileExists($sOwnPSPad_Path)

  Return $iErrors
EndFunc ; ==> FileTypeRegisterForOwnPSPad


; ==============================================================================
; OnAutoItExit
;
; Routine die aufgerufen werden soll, wenn das Script beendet wird.
; Sie wurde mit "OnAutoItExitRegister('OnAutoItExit')" registriert.
; ------------------------------------------------------------------------------
Func OnAutoItExit()
  WinSetState($g_hPSPad, "", @SW_ENABLE) ; Wichtig! Sonst ist Parent-Fenster unbenutzbar!
EndFunc ; ==> OnAutoItExit


; ==============================================================================
; RegKeyExists
;
; Routine zum Prüfen, ob ein bestimmter Schlüssel in der Registry existiert.
; Dabei ist es unwichtig, ob der Schlüssel Unterschlüssel hat und ob für den
; Werteintrag "(Standard)" des Schlüssels einen Wert festgelegt wurde.
;
; Quelle: jguinch, 2014-09-10.
; https://www.autoitscript.com/forum/topic/164189-test-if-registry-key-exsists/?do=findComment&comment=1197267
; Modifiziert von Professor Bernd, 2021-03-09.
; ------------------------------------------------------------------------------
Func RegKeyExists($_sKeyname)
    RegRead($_sKeyname, "")
    Return (@error <= 0) ; If RegRead() @error is 0 or -1 or -2, then the registry key exists.
EndFunc ; == RegKeyExists

