; #AutoIt3Wrapper_Au3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w- 7

; = Info ===========================================================================================
; Title           : IniRefresher.au3
; --------------------------------------------------------------------------------------------------
;
; Version (vnots) : 0.0.9
; Date            : 2021-05-01
; Author(s)       : AspirinJunkie, Professor Bernd
;
; Requirements    :
; Language        :
; Description     : Erneuern von User-Inis mithilfe von Global-Inis. Die Ini-Struktur
;                     wird neu geschrieben und die User-Einstellungen werden behalten.
;                   Renewing user inis using global inis. The ini structure is
;                     rewritten and the user settings are kept.
;
; Original-Code von AspirinJunkie, 2021-04-23.
; https://autoit.de/thread/87338-gemischte-ini-datei-updaten-ideen-gesucht/?postID=703643#post703643
; Modifiziert und angepasst an Pau3 von Professor Bernd. 2021-04-24, 2021-04-29, ...
; ==================================================================================================

  #cs

    Der IniRefresher dient dazu, eine User-Ini mithilfe einer Global-Ini zu
    aktualisieren und zu formatieren.

    Die Global-Ini und die User-Ini werden eingelesen. Die Global-Ini wird in
    einen MergedContent kopiert und Werte von Schlüsseln, die in beiden Inis
    existieren, werden im MergedContent durch die Werte der User-Ini ersetzt.

    Dann wird der MergedContent mit der User-Ini verglichen und wenn sie sich
    unterscheiden, wird die User-Ini mit dem MergedContent überschrieben.
    Danach enthält die User-Ini die Struktur und Reihenfolge der Global-Ini,
    alle Einstellungen, die der User gesetzt hatte, alle neu hinzugekommenen
    Schlüssel und die Versionsbezeichnung ganz am Anfang der User-Ini.

    Kurz ausgedrückt: Die Global-Ini wird fast 1:1 in die User-Ini kopiert,
    lediglich die Einstellungen des Users werden übernommen.

    2021-05-01.
  #ce


#NoTrayIcon
#include <File.au3>

Main_Func()


; ==============================================================================
; Main_Func
;
; Main function. ;-)
; ------------------------------------------------------------------------------
Func Main_Func()

  Local $sPau3GlobalIni_Path = _PathFull("..\Bak\Global\Pau3Global.ini", @ScriptDir) ; Relative paths must NOT start with "\"!
  Local $sPau3UserIni_Path = _PathFull("..\PSPad4AutoIt3.ini", @ScriptDir)
  _IniRefresh($sPau3GlobalIni_Path, $sPau3UserIni_Path)

  Local $sTidyGlobalIni_Path = _PathFull("..\Bak\Global\TidyGlobal.ini", @ScriptDir)
  Local $sTidyUserIni_Path = _PathFull("..\AutoIt3\SciTE\Tidy\Tidy.ini", @ScriptDir)
  _IniRefresh($sTidyGlobalIni_Path, $sTidyUserIni_Path)
EndFunc ; ==> Main_Func


; ==============================================================================
; _IniCombine
;
; Diese Routine führt den Inhalt der Global-Ini ($_sPrimaryIni_Path) und den der
; User-Ini ($_sValuesIni_Path) so zusammen, dass der neue Inhalt fast genau dem
; der Global-Ini entspricht, außer dass vorhandene Schlüssel-Werte der User-Ini
; übernommen werden.
;
; Rückgabe: Der zusammengeführte Inhalt aus Global-Ini und User-Ini, falls sich
;           der zusammengeführte Inhalt und der Inhalt der User-Ini unterscheiden.
;
;           Ansonsten "".
;
; Autor: AspirinJunkie, modifiziert von Professor Bernd.
; ------------------------------------------------------------------------------
Func _IniCombine(Const $_sPrimaryIni_Path, Const $_sValuesIni_Path)

  Local $sResult = "", $sLine, $sValuesIni, $sSection = "", $sKey = "", $sValue = ""
  Local $hPrimaryIni
  Local $aTmp
  Local $oSec = ObjCreate("Scripting.Dictionary")

  $hPrimaryIni = FileOpen($_sPrimaryIni_Path)
  If @error Then Return SetError(1, @error, "")

  $sValuesIni = FileRead($_sValuesIni_Path)
  If @error Then Return SetError(2, @error, "")

  Do
    $sLine = FileReadLine($hPrimaryIni)
    If @error Then ExitLoop

    $aTmp = StringRegExp($sLine, '^\[(.+)\]', 1)
    If Not @error Then ; Sektionsname gefunden.
      $sSection = $aTmp[0]
      $oSec = _IniReadSection($sValuesIni, $sSection, True)
      If @error Then $oSec = ObjCreate("Scripting.Dictionary")

    EndIf

    $aTmp = StringRegExp($sLine, '^(?!;)([^=]+)=(.*)$', 1)
    If Not @error Then ; Key-Value-Zeile gefunden.
      $sKey = $aTmp[0]
      $sValue = $aTmp[1]
      If $oSec.Exists($sKey) Then
        $sResult &= $sKey & "=" & ($oSec($sKey) = "" ? $sValue : $oSec($sKey)) & @CRLF

      Else
        $sResult &= $sLine & @CRLF
      EndIf
    Else ; Zeile enthält sonstiges (Leerzeile, Sectionheader, Kommentar).
      $sResult &= $sLine & @CRLF
    EndIf ; If Not @error
  Until 0
  FileClose($hPrimaryIni)

  If $sValuesIni == $sResult Then $sResult = ""

  Return $sResult
EndFunc ; ==> _IniCombine


; ==============================================================================
; _IniReadSection
;
; Routine zum Auslesen einer kompletten Sektion aus einer Ini-Datei.
;
; Rückgabe
;
;   Erfolg: Alle Zeilen der Sektion, die ein "=" enthalten (auch Kommentare),
;           wahlweise als Array oder Dictionary.
;
;   Fehler: False + eigener Error-Code in @error + RegEx-Error-Code in @extended.
;
; Autor: AspirinJunkie, modifiziert von Professor Bernd.
; ------------------------------------------------------------------------------
Func _IniReadSection($_sFilePath, $_sSection, Const $_bRetAsDict = False)

  Local $sFile
  If FileExists($_sFilePath) Then
    $sFile = FileRead($_sFilePath)
  Else
    $sFile = $_sFilePath
  EndIf

  Local $aSectionRaw = StringRegExp($sFile, '(?m)^\[' & _RegExEscape($_sSection) & '\].*\R([^\[]+)', 1) ; von Aspirinjunkie, 2021-04-25.
  If @error Then Return SetError(2, @error, False)

  Local $aKeyValues = StringRegExp($aSectionRaw[0], '(?m)^([^=\v]+)=(.*)$', 4)
  If @error Then Return SetError(3, @error, False)

  If $_bRetAsDict Then
    Local $oRet = ObjCreate("Scripting.Dictionary")
    For $aPair In $aKeyValues
      $oRet($aPair[1]) = $aPair[2]
    Next
    Return $oRet
  Else
    Local $aRet[UBound($aKeyValues)][2], $iC = 0
    For $aPair In $aKeyValues
      $aRet[$iC][0] = $aPair[1]
      $aRet[$iC][1] = $aPair[2]
      $iC += 1
    Next
    Return $aRet
  EndIf
EndFunc ; ==> _IniReadSection


; ==============================================================================
; _IniRefresh
;
; Routine, die die anderen Routinen kapselt und das Schreiben in die User-Ini
; druchführt, falls Unterschiede zwischen zusammengeführtem Inhalt und dem
; Inhalt der User-Ini gefunden wurden. (Details siehe "_IniCombine()".)
;
; Rückgabe: Keine.
;
; Autor: Professor Bernd.
; ------------------------------------------------------------------------------
Func _IniRefresh(Const $_sPrimaryIni_Path, Const $_sValuesIni_Path)

  Local $sErrMsg = ""
  Local $sMergedContent

  If Not FileExists($_sPrimaryIni_Path) Then $sErrMsg = '"' & $_sPrimaryIni_Path & '"' & @CRLF & @CRLF
  If Not FileExists($_sValuesIni_Path) Then
    FileWrite($_sValuesIni_Path, FileRead($_sPrimaryIni_Path))
    If Not FileExists($_sValuesIni_Path) Then $sErrMsg &= $sErrMsg & '"' & $_sValuesIni_Path & '"' & @CRLF

  EndIf

  If $sErrMsg <> "" Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
        "Files not found: " & @CRLF & @CRLF & $sErrMsg)
    Exit ; Script hier beenden.
  EndIf

  $sMergedContent = _IniCombine($_sPrimaryIni_Path, $_sValuesIni_Path)

  If $sMergedContent <> "" Then

    If StringInStr(FileGetAttrib($_sValuesIni_Path), "R") > 0 Then FileSetAttrib($_sValuesIni_Path, "-R")

    FileDelete($_sValuesIni_Path)

    FileWrite($_sValuesIni_Path, $sMergedContent)

    If Not (FileRead($_sValuesIni_Path) == $sMergedContent) Then
      MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
        "Error writing the ini file:" & @CRLF & @CRLF & $_sValuesIni_Path & @CRLF & @CRLF & "Write protection?")
    EndIf
  EndIf
EndFunc ; ==> _IniRefresh


; ==============================================================================
; _RegExEscape
;
; Die Routine stellt escaped Strings zur Verfügung, welche in RegEx-Pattern
; integriert werden sollen:
;
; Rückgabe: Escaped Strings.
;
; Autor: AspirinJunkie
; ------------------------------------------------------------------------------
Func _RegExEscape($_sString)
  Return "\Q" & StringRegExpReplace($_sString, '\\E', '\\E\\\\E\\Q') & "\E"
EndFunc ; ==> _RegExEscape
