; #AutoIt3Wrapper_Au3Check_Parameters=-q -d -w 1 -w 2 -w 3 -w- 4 -w 5 -w 6 -w- 7

; = Info ===========================================================================================
; Title           : VersionsChecker.au3
; --------------------------------------------------------------------------------------------------
;
; Version (vnots) : 0.0.79
; Date            : 2021-04-13
; Author(s)       : Professor Bernd
;
; Requirements    :
; Language        :
; Description     : Prüfung der Grundvoraussetzungen für das PSPad4AutoIt3 Improvement Kit.
; ==================================================================================================

#cs ----------------------------------------------------------------------------

  "VersionsChecker.au3" hat nur eine einzige Funktionalität, die "nach außen
  geführt" wird: Das Prüfen der Mindest-Versionsnummer von verschiedenen Dateien.
  Die Mindest-Versionsnummern befinden sich in der "VersionsChecker.lst".

  Diese Funktionalität kann nur benutzt werden, indem "VersionsChecker.au3" stand-alone
  ausgeführt wird. (Ein Inkludieren in ein anderes Script ist derzeit nicht vorgesehen.)

  Start-Parameter:

    Derzeit kann für Parameter 1 immer nur 1 der folgenden Werte übergeben werden.
    Die Parameter-Werte können NICHT kombiniert werden!
    (Bei "/TitleDownlHelper" wird als Kombi im Param 2 der Titel übergeben.)

    $CmdLine[1]:  "/ShowVerInfo"

                  Falls angegeben, wird ein Fenster mit Infos zu den geprüften
                  Dateien angezeigt.

                  Ansonsten wird das Fenster mit Infos zu den geprüften Dateien
                  nur angezeigt, wenn Fehler auftreten. Fehler sind: Datei
                  existiert NICHT oder Mindest-Versionsnummer NICHT erfüllt.

      oder        "/TitleDownlHelper"

                  Falls angegeben, wird auch der Param "$CmdLine[2]" ausgewertet,
                  der dann den Titel-Text vom DownloadHelper enthält.

    $CmdLine[2]:  Falls in $CmdLine[1] "/TitleDownlHelper" angegeben wurde, kann
                  hier der Titel-Text vom DownloadHelper angegeben werden.

                  Der Titel vom DownloadHelper wird in "DownloadHelperIsParent()"
                  benutzt, um das DownloadHelper-Fenster zu identifizieren.

  Exit-Code (Wichtig: Entspricht NICHT der Rückgabe der Main-Funktion!)

    Erfolg: 0 - KEINE Fehler und ALLE MinVers erfüllt.
    Fehler: 1 - KEIN Fehler in MinVer von "AutoIt3.exe", aber Fehler bei anderen Dateien.
            2 - Fehler in MinVer von "AutoIt3.exe", aber KEINE Fehler bei anderen Dateien.
            3 - Fehler in MinVer von "AutoIt3.exe" UND Fehler bei anderen Dateien.


  Aufruf-Beispiele:

    "...\AutoIt3.exe" "...\VersionsChecker.au3"
    "...\AutoIt3.exe" "...\VersionsChecker.au3" "/ShowVerInfo"
    "...\AutoIt3.exe" "...\VersionsChecker.au3" "/TitleDownlHelper" "Titel-des-DownloadHelpers"

#ce ;---------------------------------------------------------------------------


#NoTrayIcon
#include <Array.au3>
#include <File.au3>
#include <Misc.au3>
#include <MsgBoxConstants.au3>
#include <String.au3>
#include <WinAPIFiles.au3>

#include <ComboConstants.au3>     ; Für z.B. ComboBox read only.
#include <GUIConstantsEx.au3>
#include <StaticConstants.au3>
#include <WinAPISysWin.au3>
#include <WindowsConstants.au3>

_Singleton("PSPad4AutoIt3->VersionsChecker")

Opt("MustDeclareVars", 1)

OnAutoItExitRegister('OnAutoItExit')

Global Const $g_sErrMsg_VerNum_OK = "OK"
Global Const $g_sErrMsg_VersionError = "Version error"
Global Const $g_sErrMsg_FileNotExists = "File not exists"

Global $g_sTitleDownlHelper

Global $g_sTitleArrDispl = "Version info of the required files for PSPad4AutoIt3"

Global $g_hMsgGUI
Global $g_hParent_VerChk

Global $g_iRet, $g_iErrNum ; Zum Auswerten von @error

$g_iRet = VersionCheckFor_PSPad4AutoIt3_Files()
$g_iErrNum = @error

If $g_iRet = 1 Then
  Exit 0 ; KEINE Fehler aufgetreten und alle MinVers erfüllt.
Else
  Exit $g_iErrNum
EndIf


; Main-Function
; ==============================================================================
; VersionCheckFor_PSPad4AutoIt3_Files
;
; Versionen der PSPad4AutoIt3 Dateien prüfen.
;
; Rückgabe
;
;   Erfolg: 1 - KEINE Fehler und ALLE MinVers erfüllt.
;   Fehler: 0 - Fehler
;               @error: 1 - KEIN Fehler in MinVer von "AutoIt3.exe", aber Fehler bei anderen Dateien.
;                       2 - Fehler in MinVer von "AutoIt3.exe", aber KEINE Fehler bei anderen Dateien.
;                       3 - Fehler in MinVer von "AutoIt3.exe" UND Fehler bei anderen Dateien.
; ------------------------------------------------------------------------------
Func VersionCheckFor_PSPad4AutoIt3_Files()

  Local $hFile                ; Zum Einlesen der VersionsChecker.lst.

  Local $sAutoIt3_Dir         ; Pfad zur AutoIt3.exe die der User installiert hat.
  Local $sPSPad4Au3_SciTE_Dir ; Pfad zum SciTE-Ordner im PSPad4Au3-Ordner für mitgeführte AutoIt3Wrapper.au3.
  Local $sPSPad_Dir
  Local $sBaseDir             ; Basis-Pfad: Entweder $sAutoIt3_Dir, $sSciTE_Dir oder $sPSPad_Dir.
  Local $sFilePath            ; Pfad zur Datei, die überprüft werden soll.
  Local $sMinVerLstFile_Path  ; Pfad zur VersionsChecker.lst.
  Local $sLine = ""           ; Zeile, die aus der VersionsChecker.lst ausgelesen wird.

  Local $aLine = 0            ; Array für das Aufsplitten von $sLine.
  Local $aVerInfo[1][5] = [["state", "min version", "file version", "file name", "file dir"]]

  Local $iMinVerNum
  Local $iErrNum = 0, $iErrNumExt = 0, $iRetVal = 0

  Local $bShowVerInfo = False

  If $CmdLine[0] > 0 Then
    If $CmdLine[1] = "/ShowVerInfo" Then
      $bShowVerInfo = True
    ElseIf $CmdLine[1] = "/TitleDownlHelper" Then
      If $CmdLine[0] > 1 Then $g_sTitleDownlHelper = $CmdLine[2]
    EndIf
  EndIf

  $sPSPad_Dir = _PathFull('..\..\', @ScriptDir)
  $sAutoIt3_Dir = ReadFromPSPad4AutoIt3_INI("Paths", "AutoIt3_Dir")
  $sPSPad4Au3_SciTE_Dir = $sPSPad_Dir & "PSPad4Au3\AutoIt3\SciTE\" ; Für mitgeführte AutoIt3Wrapper.au3.

  $sMinVerLstFile_Path = $sPSPad_Dir & "PSPad4Au3\Au3 scripts\VersionsChecker.lst"
  $hFile = FileOpen($sMinVerLstFile_Path, $FO_READ)

  If $hFile = -1 Then
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, @ScriptName, _
      "Message from Modul  '" & @ScriptName & "'" & @CRLF _
      & "function  'VersionCheckFor_PSPad4AutoIt3_Files()':" & @CRLF & @CRLF _
      & "An error occurred while reading the file:" & @CRLF & @CRLF _
      & "Beim Lesen der Datei ist ein Fehler aufgetreten:" & @CRLF _
      & @CRLF & @CRLF & Chr(34) & $sMinVerLstFile_Path & Chr(34))
  Else
    Local $bEOF_IsReached = False

    While Not $bEOF_IsReached
      $sLine = FileReadLine($hFile)

      If @error Then ; Dateiende erreicht.
        $bEOF_IsReached = True
      Else
        If StringLeft(StringStripWS($sLine, $STR_STRIPLEADING), StringLen("PSPad4AutoIt3")) = "PSPad4AutoIt3" Then

          $aLine = StringSplit($sLine, "|", $STR_NOCOUNT) ; 0-basiertes Array.

          $g_sTitleArrDispl &= "  " & $aLine[1]
        EndIf

        If StringLeft(StringStripWS($sLine, $STR_STRIPLEADING), StringLen("BaseDir_")) = "BaseDir_" Then

          $aLine = StringSplit($sLine, "|", $STR_NOCOUNT) ; 0-basiertes Array.

          If $aLine[0] = "BaseDir_PSPad" Then ; Das am häufigsten vorkommende BaseDir zuerst prüfen.
            $sBaseDir = $sPSPad_Dir
          ElseIf $aLine[0] = "BaseDir_PSPad4Au3_SciTE" Then
            $sBaseDir = $sPSPad4Au3_SciTE_Dir
          ElseIf $aLine[0] = "BaseDir_AutoIt3" Then
            $sBaseDir = $sAutoIt3_Dir
          EndIf

          $sFilePath = $sBaseDir & $aLine[1] ; Dateipfad zusammensetzen.
          $iMinVerNum = $aLine[2]

          If (_WinAPI_PathStripPath($sFilePath) = "AutoIt3.exe") And Not FileExists($sFilePath) Then
          Else
            _ArrayAdd($aVerInfo, GetVerInfoString($sFilePath, $iMinVerNum))
          EndIf
        EndIf ; If StringLeft(StringStripWS($sLine, $STR_STRIPLEADING), StringLen("BaseDir_")) = "BaseDir_"
      EndIf ; If @error
    WEnd
  EndIf ; If $hFile = -1

  FileClose($hFile)

  $iRetVal = CheckVerInfoArrayForErrorMessages($aVerInfo)
  $iErrNum = @error

  If $iRetVal = 0 Then
    $bShowVerInfo = False
  EndIf

  If ($bShowVerInfo = True) Then

    AdlibRegister('CloseArrDisplayIfNotActive')

    _ArrayDisplay($aVerInfo, $g_sTitleArrDispl, "", $ARRAYDISPLAY_NOROW)
  EndIf

  Return SetError($iErrNum, $iErrNumExt, $iRetVal)
EndFunc ; ==> VersionCheckFor_PSPad4AutoIt3_Files


; =========================================================================
; CheckVerInfoArrayForErrorMessages
;
; VerInfo-Array durchsuchen nach Fehlern.
;
; Rückgabe
;
;   Erfolg: 1 - KEINE Fehler und ALLE MinVers erfüllt.
;   Fehler: 0 - @error: 1 - KEIN Fehler in MinVer von "AutoIt3.exe", aber Fehler bei anderen Dateien.
;                       2 - Fehler in MinVer von "AutoIt3.exe", aber KEINE Fehler bei anderen Dateien.
;                       3 - Fehler in MinVer von "AutoIt3.exe" UND Fehler bei anderen Dateien.
; -------------------------------------------------------------------------
Func CheckVerInfoArrayForErrorMessages($_aVerInfo)

  Local $sErrMsg_English, $sErrMsg_German
  Local $iErrNum = 0, $iErrNumExt = 0
  Local $bErrInAu3ExeMinVer = False

  For $i = 1 To UBound($_aVerInfo, $UBOUND_ROWS) - 1

    If StringInStr($_aVerInfo[$i][0], $g_sErrMsg_FileNotExists) Then

      $sErrMsg_English = "Warning! - Minimum requirements not met." & @CRLF _
        & @CRLF _
        & "One or more program files are missing." & @CRLF _
        & @CRLF _
        & "PSPad itself can be used normally, but some of the special PSPad4AutoIt3 " _
        & "functions are not available." & @CRLF _
        & @CRLF _
        & "Please install the missing program files."

      $sErrMsg_German = "Warnung! - Mindestanforderungen nicht erfüllt." & @CRLF _
        & @CRLF _
        & "Eine oder mehrere Programm-Dateien fehlen." & @CRLF _
        & @CRLF _
        & "PSPad selbst kann zwar normal benutzt werden, aber einige der speziellen " _
        & "PSPad4AutoIt3 Funktionen sind nicht verfügbar." & @CRLF _
        & @CRLF _
        & "Bitte installieren Sie die fehlenden Programm-Dateien."

      $iErrNum = BitOR($iErrNum, 1)
      ExitLoop

    ElseIf StringInStr($_aVerInfo[$i][0], $g_sErrMsg_VersionError) Then

      If $_aVerInfo[$i][3] = "AutoIt3.exe" Then

        $sErrMsg_English = "The specified AutoIt3.exe does not meet the minimum version required." _
          & @CRLF & @CRLF _
          & "Version of the specified AutoIt3.exe:  " & $_aVerInfo[$i][2] & @CRLF _
          & "Minimum version required:  " & $_aVerInfo[$i][1]

        $sErrMsg_German = "Die angegebene AutoIt3.exe erfüllt nicht die benötigte Mindestversion." _
          & @CRLF & @CRLF _
          & "Version der angegebenen AutoIt3.exe:  " & $_aVerInfo[$i][2] & @CRLF _
          & "Benötigte Mindestversion:  " & $_aVerInfo[$i][1]

        $bErrInAu3ExeMinVer = True
        $iErrNum = BitOR($iErrNum, 2)

      Else ; Der Versions-Fehler betrifft eine andere Datei.      ----------

        $sErrMsg_English = "Warning! - Minimum requirements not met." & @CRLF _
          & @CRLF _
          & "One or more program files do not meet the minimum version number." & @CRLF _
          & @CRLF _
          & "PSPad itself can be used normally, but some of the special PSPad4AutoIt3 " _
          & "functions are not available or work incorrectly." & @CRLF _
          & @CRLF _
          & "Please update the respective program files."

          $sErrMsg_German = "Warnung! - Mindestanforderungen nicht erfüllt." & @CRLF _
          & @CRLF _
          & "Eine oder mehrere Programm-Dateien erfüllen nicht die Mindest-Versionsnummer." & @CRLF _
          & @CRLF _
          & "PSPad selbst kann zwar normal benutzt werden, aber einige der speziellen " _
          & "PSPad4AutoIt3 Funktionen sind nicht verfügbar oder arbeiten fehlerhaft." & @CRLF _
          & @CRLF _
          & "Bitte aktualisieren Sie die betreffenden Programm-Dateien."

        $iErrNum = BitOR($iErrNum, 1)
      EndIf ; If $_aVerInfo[$i][3] = "AutoIt3.exe"                --------------

      ExitLoop
    EndIf
  Next

  If $iErrNum = 0 Then
    Return(1)
  Else ; Fehler aufgetreten.
    MsgGui_CreateAndShow($_aVerInfo, $bErrInAu3ExeMinVer, $sErrMsg_English, $sErrMsg_German)
    Return SetError($iErrNum, $iErrNumExt, 0)
 EndIf
EndFunc ; ==> CheckVerInfoArrayForErrorMessages


; ==============================================================================
; CloseArrDisplayIfNotActive
;
; Routine zum Schließen des "_ArrayDisplay()"-Dialogs.
; Sie wurde mit "AdlibRegister()" registriert.
;
; Ein "_ArrayDisplay()"-Dialog hat eine eigene Schleife, die solange läuft, bis
; der Dialog geschlossen wird. Deshalb kann der Dialog NICHT aus einer Schleife
; heraus geschlossen werden, die den Dialog aufgerufen hat, weil die aufrufende
; Schleife solang pausiert, bis der Dialog geschlossen ist.
;
; Das Schließen des "_ArrayDisplay()"-Dialogs ist aber mithilfe eine Timers
; möglich. (Vermutlich würde es auch funktionieren, wenn man das Script auf den
; OnEvent-Modus umstellen würde.)
; ------------------------------------------------------------------------------
Func CloseArrDisplayIfNotActive()
  If WinExists($g_sTitleArrDispl) And Not WinActive($g_sTitleArrDispl) Then
    WinClose($g_sTitleArrDispl)
    AdlibUnRegister('CloseArrDisplayIfNotActive')
  EndIf
EndFunc ; CloseArrDisplayIfNotActive


; =========================================================================
; FileVerIsGreaterOrEqualMinVer
;
; Vergleichen der Datei Version mit Minimum Version.
; -------------------------------------------------------------------------
Func FileVerIsGreaterOrEqualMinVer($_sMinVersion, $_sFileVersion) ; boolean

  Local $aMinVer = StringSplit($_sMinVersion, ".")
  Local $aFileVer = StringSplit($_sFileVersion, ".")
  Local $bFileVerIsGreaterOrEqual = True
  Local $sErrMsg = ""

  If $aMinVer[0] <> $aFileVer[0] Then _
    $sErrMsg = "Version numbers have a different quantity of blocks." & @CRLF

  If ($_sMinVersion = "") Then _
    $sErrMsg = $sErrMsg & "Error in minimum version." & @CRLF

  If ($_sFileVersion = "") Then _
    $sErrMsg = $sErrMsg & "Error reading the file version." & @CRLF

  If $sErrMsg <> "" Then
    MsgBox($MB_SYSTEMMODAL, @ScriptName, _
      "Message from function 'FileVerIsGreaterOrEqualMinVer()':" _
      & @CRLF & @CRLF & $sErrMsg)
    $bFileVerIsGreaterOrEqual = False
  Else
    If IsArray($aMinVer) And IsArray($aFileVer) Then
      For $i = 1 To $aFileVer[0]
        If Int($aMinVer[$i]) > Int($aFileVer[$i]) Then
          $bFileVerIsGreaterOrEqual = False
          ExitLoop
        ElseIf Int($aMinVer[$i]) < Int($aFileVer[$i]) Then
          ExitLoop
        EndIf
      Next
    EndIf
  EndIf

  Return($bFileVerIsGreaterOrEqual)
EndFunc ; ==> FileVerIsGreaterOrEqualMinVer


; =========================================================================
; GetFileVersionOfExeOrScript
;
; Ermitteln der Version einer Exe- oder Script-Datei.
;
;   Rückgabe: Datei Version (Format: 0.0.0.0) Anzahl Blöcke variiert.
;
; (Funktions-Name verlängert, um Verwechslung mit Standard-Funktionen zu vermeiden.)
; -------------------------------------------------------------------------
Func GetFileVersionOfExeOrScript($_sFilePath)

  Local $hFile
  Local $sLine
  Local $sKeyWord
  Local $sDelimiter
  Local $iDelimPos
  Local $aChunks

  Switch StringRight($_sFilePath, 4)

    Case ".exe"
      Return(FileGetVersion($_sFilePath)) ; Für Exe's VerNum zurückgeben und Funktion hier verlassen.

    Case ".vbs"
      $sKeyWord = "Const module_ver"
      $sDelimiter = "="

    Case ".au3"
      If _WinAPI_PathStripPath($_sFilePath) = "AutoIt3Wrapper.au3" Then

        $aChunks = StringSplit(FileRead($_sFilePath, 10240), @CRLF & "=") ; Die ersten 10.240 Zeichen auslesen.

        For $n = 2 To UBound($aChunks) - 1

          If StringStripWS($aChunks[$n - 1], $STR_STRIPALL) = "GlobalConst$VERSION" Then
            Return StringStripWS(StringReplace($aChunks[$n], '"', ''), $STR_STRIPALL)
          EndIf

          If StringStripWS($aChunks[$n - 1], $STR_STRIPLEADING + $STR_STRIPTRAILING) _
            = "#AutoIt3Wrapper_Res_Fileversion" Then
            Return StringStripWS(StringReplace($aChunks[$n], '"', ''), $STR_STRIPALL)
          EndIf
        Next
      Else ; Meine eigenen au3 Scripts.
        $sKeyWord = "Version (vnots)"
        $sDelimiter = ":"
      EndIf

    Case Else
      MsgBox($MB_SYSTEMMODAL + $MB_ICONINFORMATION, @ScriptName, _
        'Only for "Exe", "VBS" and "Au3" files.')
  EndSwitch

  $hFile = FileOpen($_sFilePath, $FO_READ)

  If $hFile = -1 Then
    MsgBox($MB_SYSTEMMODAL, @ScriptName, _
      "Message from function 'GetFileVersionOfExeOrScript()':" & @CRLF & @CRLF _
      & "An error occurred while reading the file." & @CRLF & @CRLF _
      & "Beim Lesen der Datei ist ein Fehler aufgetreten." & @CRLF _
      & @CRLF & @CRLF & Chr(34) & $_sFilePath & Chr(34))
  Else
    Local $bEOF_IsReached = False
    While Not $bEOF_IsReached
      $sLine = FileReadLine($hFile)                     ; Zeile auslesen.
      If @error Then $bEOF_IsReached = True             ; Dateiende erreicht.

      $iDelimPos = StringInStr($sLine, $sDelimiter)
      If StringInStr($sLine, $sKeyWord) And ($iDelimPos > 0) Then
        $sLine = StringMid($sLine, $iDelimPos + 1)      ; Text nach Trennzeichen ermitteln.
        $sLine = StringReplace($sLine, '"', '')         ; Alle Doppel-Anführungszeichen entfernen
        $sLine = StringStripWS($sLine, $STR_STRIPALL )  ; Alle Leerzeichen u Co entfernen (Punkte bleiben!).
        ExitLoop
      EndIf
    WEnd
  EndIf

  FileClose($hFile)

  Return($sLine)
EndFunc ; ==> GetFileVersionOfExeOrScript


; Der String wird mit Trennzeichen versehen, mit denen die Infos per
; _ArrayAdd in die Spalten einer Reihe eines VerInfoArray eingefügt werden.
; -------------------------------------------------------------------------
Func GetVerInfoString($_sFilePath, $_sMinVersion)

  Local $sFileVer
  Local $sVerInfoStr
  Local $sFileName = StringMid($_sFilePath, StringInStr($_sFilePath, '\', 0, -1) + 1)
  Local $sFileDir = StringLeft($_sFilePath, StringInStr($_sFilePath, "\", 0, -1))

  If FileExists($_sFilePath) Then

    $sFileVer = GetFileVersionOfExeOrScript($_sFilePath)

    If (FileVerIsGreaterOrEqualMinVer($_sMinVersion, $sFileVer) = True) Then
      $sVerInfoStr = $g_sErrMsg_VerNum_OK
    Else
      $sVerInfoStr = "- - " & $g_sErrMsg_VersionError & " - -"
    EndIf

  Else ; If file not exists.
    $sVerInfoStr = "- - " & $g_sErrMsg_FileNotExists & " - -"
  EndIf

  $sVerInfoStr = $sVerInfoStr & "|" & $_sMinVersion & "|" & $sFileVer _
    & "|" & $sFileName & "|" & $sFileDir

  Return($sVerInfoStr)
EndFunc ; ==> GetVerInfoString


; ==============================================================================
; DownloadHelperIsParent
;
; Routine zum Ermitteln, ob das DownloadHelper-Fenster das Parent-Fenster der
; Msg-GUI (VersionsChecker) ist.
;
; Geprüft wird vom Parent-Fenster der Msg-GUI, ob der Titel mit dem vom
; DownloadHelper per Start-Parameter übermittelten Titel übereinstimmt, ob es
; sich um ein Fenster der "AutoIt v3 GUI"-Klasse handelt und ob das Parent-
; Fenster des Parent-Fensters (DownloadHelper) ein PSPad-Fenster ist.
;
; Das ergibt eine höhere Sicherheit, das richtige DownloadHelper-Win zu finden.
; Es könnte z.B. durchaus ein Explorer-Win mit dem gleichen Titel geben.
; ------------------------------------------------------------------------------
Func DownloadHelperIsParent()
  If (WinGetTitle($g_hParent_VerChk) = $g_sTitleDownlHelper) _
      And _WinAPI_IsClassName($g_hParent_VerChk, "AutoIt v3 GUI") _
      And _WinAPI_IsClassName(_WinAPI_GetParent($g_hParent_VerChk), "TfPSPad") Then
    Return(1)
  Else
    Return(0)
  EndIf

EndFunc ; ==> DownloadHelperIsParent


; ==============================================================================
; MsgGui_CreateAndShow
;
; ------------------------------------------------------------------------------
Func MsgGui_CreateAndShow($_aVerInfo, $_bErrInAu3ExeMinVer, $_ErrMsg_English, $_ErrMsg_German)

  Local $sMsgGuiTitle           = "PSPad4AutoIt3 VersionsChecker"

  Local $sTitle_English         = "VersionsChecker"

  Local $sVersionsInfo_English  = "Show versions info"
  Local $sVersionsInfo_German   = "Versions-Infos zeigen"
  Local $sDownlHelper_English   = "Change AutoIt3 path"
  Local $sDownlHelper_German    = "AutoIt3-Pfad ändern"

  Local $lblTitle, $cmbLanguage, $lblMainInfo
  Local $btnVersionsInfo, $btnDownlHelper

  Local $sComboTxt, $nMsg
  Local $aDnlHlpPos
  Local $iGuiClientWidth, $iBtnWith, $iBtnLeft

  $g_hParent_VerChk = WinGetHandle("[active]")  ; Parent-Fenster für Msg-GUI.

  If _WinAPI_IsClassName($g_hParent_VerChk, "TfPSPad") Or DownloadHelperIsParent() Then

    $g_hMsgGUI = GUICreate("", 586, 300, -1, -1, BitOR($WS_CAPTION, $WS_POPUPWINDOW), -1, $g_hParent_VerChk)
    WinSetTitle($g_hMsgGUI, "", $sMsgGuiTitle)

    If DownloadHelperIsParent() Then
      $aDnlHlpPos = WinGetPos($g_hParent_VerChk)
      If IsArray($aDnlHlpPos) Then WinMove($g_hMsgGUI, "", $aDnlHlpPos[0], $aDnlHlpPos[1])
    EndIf

    $lblTitle = GUICtrlCreateLabel("", 128, 12, 328, 34, BitOR($SS_CENTER, $SS_CENTERIMAGE))
    GUICtrlSetFont(-1, 16, 800, 0, "Segoe UI") ; "Segoe UI", "MS Sans Serif"
    GUICtrlSetColor(-1, 0xFF6600) ; dunkel orange.
    GUICtrlSetData(-1, $sTitle_English)

    $cmbLanguage = GUICtrlCreateCombo("", 462, 16, 100, 25, _
      BitOR($CBS_DROPDOWNLIST, $CBS_AUTOHSCROLL, $WS_VSCROLL)) ; ComboBox read only.
    GUICtrlSetData(-1, "English|German", "English")

    $lblMainInfo = GUICtrlCreateLabel("", 24, 72, 538, 160)             ; Wenn wenig Text im Label, etwas tiefer setzen.
    If Not $_bErrInAu3ExeMinVer Then GUICtrlSetPos(-1, -1, 56, -1, 192) ; Wenn viel Text im Label, höher setzen und Höhe vergrößern.
    GUICtrlSetFont(-1, 11, 400, 0, "Segoe UI") ; "Segoe UI", "MS Sans Serif"
    GUICtrlSetData(-1, $_ErrMsg_English)

    $iBtnWith = 252
    $btnVersionsInfo = GUICtrlCreateButton($sVersionsInfo_English, 24, 248, $iBtnWith, 38)
      $iGuiClientWidth = WinGetClientSize($g_hMsgGUI)[0]
      $iBtnLeft = ($iGuiClientWidth - $iBtnWith) / 2
      GUICtrlSetPos($btnVersionsInfo, $iBtnLeft)

    $btnDownlHelper = GUICtrlCreateButton($sDownlHelper_English, 308, 248, 252, 38)
    GUICtrlSetState(-1, $GUI_HIDE)

    If DownloadHelperIsParent() And $CmdLine[0] > 2 Then
      $sComboTxt = $CmdLine[3]                  ; Übermittelte Sprache vom DownloadHelper übernehmen.
      GUICtrlSetState($cmbLanguage, $GUI_HIDE)  ; ComboBox verstecken, weil Änderungen durch den User (derzeit)
      GUICtrlSetData($cmbLanguage, $sComboTxt)  ;   NICHT zurück zum DownloadHelper übermittelt werden können.
    EndIf

    Switch GUICtrlRead($cmbLanguage)

      Case "English"
        GUICtrlSetData($lblMainInfo, $_ErrMsg_English)
        GUICtrlSetData($btnVersionsInfo, $sVersionsInfo_English)
        GUICtrlSetData($btnDownlHelper, $sDownlHelper_English)

      Case "German"
        GUICtrlSetData($lblMainInfo, $_ErrMsg_German)
        GUICtrlSetData($btnVersionsInfo, $sVersionsInfo_German)
        GUICtrlSetData($btnDownlHelper, $sDownlHelper_German)

    EndSwitch

    ControlFocus($g_hMsgGUI, "", $lblTitle)

    GUISetState(@SW_SHOW, $g_hMsgGUI)

    WinSetState($g_hParent_VerChk,"",@SW_DISABLE) ; Parent-Fenster disablen.

    If DownloadHelperIsParent() Then _WinAPI_ShowWindow($g_hParent_VerChk, @SW_HIDE)

    While 1
      $nMsg = GUIGetMsg()
      Switch $nMsg

        Case $GUI_EVENT_CLOSE

          ExitLoop

        Case $cmbLanguage

          Switch GUICtrlRead($cmbLanguage)

            Case "English"
              GUICtrlSetData($lblMainInfo, $_ErrMsg_English)
              GUICtrlSetData($btnVersionsInfo, $sVersionsInfo_English)
              GUICtrlSetData($btnDownlHelper, $sDownlHelper_English)

            Case "German"
              GUICtrlSetData($lblMainInfo, $_ErrMsg_German)
              GUICtrlSetData($btnVersionsInfo, $sVersionsInfo_German)
              GUICtrlSetData($btnDownlHelper, $sDownlHelper_German)

          EndSwitch

        Case $btnVersionsInfo

          AdlibRegister('CloseArrDisplayIfNotActive')

          _ArrayDisplay($_aVerInfo, $g_sTitleArrDispl, "", $ARRAYDISPLAY_NOROW)

        Case $btnDownlHelper

          ExitLoop

      EndSwitch
    WEnd
  EndIf ; If _WinAPI_IsClassName($g_hParent_VerChk, "TfPSPad") Or DownloadHelperIsParent()
EndFunc ; ==> MsgGui_CreateAndShow


; ==============================================================================
; OnAutoItExit
;
; Routine die aufgerufen werden soll, wenn das Script beendet wird.
; Sie wurde mit "OnAutoItExitRegister('OnAutoItExit')" registriert.
; ------------------------------------------------------------------------------
Func OnAutoItExit()

  Local $aVerChkPos

  WinSetState($g_hParent_VerChk,"", @SW_ENABLE) ; Wichtig! Sonst ist Parent-Fenster unbenutzbar!

  If DownloadHelperIsParent() Then

    $aVerChkPos = WinGetPos($g_hMsgGUI)
    If IsArray($aVerChkPos) Then
      WinMove($g_hParent_VerChk, "", $aVerChkPos[0], $aVerChkPos[1])
    EndIf

   _WinAPI_ShowWindow($g_hParent_VerChk, @SW_SHOW)
  EndIf
EndFunc ; ==> OnAutoItExit


; =========================================================================
; ReadFromPSPad4AutoIt3_INI                         Version von 2019-12-13.
;
; Daten aus PSPad4AutoIt3.ini lesen.
;
;   Rückgabe: Wert des Keys - oder "".
; -------------------------------------------------------------------------
Func ReadFromPSPad4AutoIt3_INI($_sSection, $_sKey)
  Local $sRetVal = ""
  Local $sPSPad4AutoIt3_IniPath = _PathFull(@ScriptDir & '\..\PSPad4AutoIt3.ini')

  If FileExists($sPSPad4AutoIt3_IniPath) Then
    $sRetVal = IniRead($sPSPad4AutoIt3_IniPath, $_sSection, $_sKey, "")
  Else
    MsgBox($MB_SYSTEMMODAL + $MB_ICONWARNING, "PSPad4AutoIt3 - " & @ScriptName, _
      "File does not exist." & @CRLF & @CRLF & $sPSPad4AutoIt3_IniPath)
  EndIf

  Return($sRetVal)
EndFunc ; ==> ReadFromPSPad4AutoIt3_INI

