Option Explicit

' = Info ===========================================================================================
' Title           : AutoIt3_CallTip.vbs
' --------------------------------------------------------------------------------------------------
'
Const module_name = "AutoIt3_CallTip"
Const module_ver  = "0.0.56"
Const module_desc = "Routines for displaying CallTips using CallTipViewer.au3"
' Date            : 2021-04-30
' Author(s)       : Professor Bernd
'
' Requirements    :
' Language        :
' ==================================================================================================


' global constants (gc_...)
' Const gc_sExample = "Hello world!"
Const gc_sTitel_CallTipWin = "PSPad4AutoIt3_CallTipViewer"
Const gc_sCOM_ROT_Name = "PSPad4AutoIt3_COM_DictForCallTips"


' WICHTIG! Globale Variablen können NICHT im Init() initialisiert werden!
' =======  (Aber sie können hier im globalen Bereich initialisiert werden.
'           Der globale Bereich wird noch VOR der Init() Routine ausgeführt!)
'
' global variables
' Dim g_bExample
' Dim g_oCOM_Dict ' COM Objekt (Dictionary), das im ROT (Running Object Table) registiert wird.





' ==============================================================================
' CallTip_GetLinesLeftAndAboveTheCaret
'
' Routine zum Auslesen der Zeilen links vom Caret und der Zeilen darüber.
'
' Die Routine ermittelt erst den Text der aktuellen Zeile, vom Zeilenanfang
' bis zum Caret. Dann wird geprüft, ob die Zeile darüber ein " _", also ein
' Lcc = "Line continuation character" (Zeilenfortsetzungszeichen) enthält.
' Falls ja, wird die Zeile vor dem ermitteltetn Text eingefügt, dann die
' nächste Zeile darüber geprüft, usw. 2020-04-26.
' ------------------------------------------------------------------------------
Function CallTip_GetLinesLeftAndAboveTheCaret(o_bj)

  Dim aLinesAll

  Dim sCollectedLines
  Dim sCurrentLine
  Dim sPreviousCharacter
  Dim sStrippedRest

  Dim iCaretLine
  Dim iInStringStyle
  Dim iLinePos

  Dim bValidLCCFound
  Dim bOnlyWhitespacesBeforeLCC
  Dim bValidSequenceAfterLCC

  aLinesAll = Split(o_bj.Text(), vbCRLF, -1, vbTextCompare)

  If UBound(aLinesAll) < 0 Then Exit Function

  iCaretLine = o_bj.caretY() - 1

  sCollectedLines = Mid(aLinesAll(iCaretLine), 1, o_bj.caretX() - 1)

  Do
    iCaretLine = iCaretLine - 1
    If iCaretLine < 0 Then Exit Do
    If (InStr(aLinesAll(iCaretLine), " _") = 0) _
      And (InStr(aLinesAll(iCaretLine), vbTab & "_") = 0) Then Exit Do

    sCurrentLine = aLinesAll(iCaretLine)
    iInStringStyle = 0
    bValidLCCFound = False

    For iLinePos = 1 To Len(sCurrentLine)
      Select Case Mid(sCurrentLine, iLinePos, 1)
        Case """"
          If iInStringStyle = 1 Then
            iInStringStyle = 0
          ElseIf iInStringStyle = 2 Then
          ElseIf iInStringStyle = 0 Then
            iInStringStyle = 1
          End If

        Case "'"
          If iInStringStyle = 2 Then
            iInStringStyle = 0
          ElseIf iInStringStyle = 1 Then

          ElseIf iInStringStyle = 0 Then
            iInStringStyle = 2
          End If

        Case ";", "#"
          If iInStringStyle = 0 Then
            bValidLCCFound = False
            Exit For
          End If

        Case "_"

          If (iInStringStyle = 0) And (iLinePos > 1)Then
            sPreviousCharacter = Mid(sCurrentLine, iLinePos - 1, 1)

            bOnlyWhitespacesBeforeLCC = (Trim(Replace(Mid(sCurrentLine, 1, iLinePos - 1), vbTab, "")) = "")

            bValidSequenceAfterLCC = False

            sStrippedRest = Trim(Replace(Right(sCurrentLine, Len(sCurrentLine) - (iLinePos + 1) + 1), vbTab, ""))

            If sStrippedRest = "" Or Left(sStrippedRest, 1) = ";" Then bValidSequenceAfterLCC = True

            If (sPreviousCharacter = " " Or sPreviousCharacter = vbTab) _
              And Not bOnlyWhitespacesBeforeLCC And bValidSequenceAfterLCC Then
              bValidLCCFound = True
              Exit For
            End If
          End If
      End Select
    Next

    If bValidLCCFound = False Then Exit Do

    sCollectedLines = sCurrentLine & vbCRLF & sCollectedLines
  Loop

  CallTip_GetLinesLeftAndAboveTheCaret = sCollectedLines
End Function ' ==> CallTip_GetLinesLeftAndAboveTheCaret


' ==============================================================================
' CaretX_TTS  (kurz für "CaretX_TabsToSpaces")
'
' Routine zum Ermitteln des virtuellen CaretX, das dem entspricht, wenn
' die Tabs zu Spaces konvertiert und an den TabStopps ausgerichtet sind.
' Das heißt, die Tabs werden durch soviele Spaces ersetzt, dass sich alle
' Wörter an der gleichen Position befinden, an der sie der Editor beim
' Text mit Tabs angezeigt hat.
'
' Das virtuelle CaretX enthält auch den Abstand zw. Zeilenende und Caret. 2020-10-29.
' ------------------------------------------------------------------------------
Function CaretX_TTS(o_bj)

  Dim sLineFormStartToCaret
  Dim iVirtualSpaces

  sLineFormStartToCaret = Left(o_bj.lineText, o_bj.CaretX - 1)

  sLineFormStartToCaret = TabsToSpacesEx(sLineFormStartToCaret, o_bj)

  iVirtualSpaces = o_bj.CaretX - (Len(o_bj.lineText) + 1)

  If iVirtualSpaces < 0 Then iVirtualSpaces = 0

  CaretX_TTS = Len(sLineFormStartToCaret) + 1 + iVirtualSpaces ' Caret-Pos ist 1-basiert, also 1 dazuzählen.
End Function ' ==> CaretX_TTS


' ==============================================================================
' TabsToSpacesEx
'
' Routine zum Entfernen des abschließenden CRLF vom konvertierten String,
' falls der Original-String ohne CRLF endet.
'
' Auf meine Anfrage (am 2020-04-16) hin hat Jan Fiala in PSPad's VBScript die
' Funktion "tabsToSpaces()" eingebaut. Bisher (2020-10-28) ist die Funktion
' undokumentiert.
'
' Leider fügt "tabsToSpaces()" ein CRLF am Ende des konvertierten Strings ein,
' falls der Original-String ohne CRLF endet. Wofür das gedacht ist, ist mir
' nicht bekannt und aus meiner Sicht ist das ein Bug.
'
' Meine Funktion "TabsToSpacesEx()" entfernt lediglich das abschließende CRLF
' vom konvertierten String, falls der Original-String ohne CRLF endet.
'
' Hint: Das Konvertieren von Tabs zu Spaces hier im VBScript durchzuführen,
'       hat den Vorteil, dass "tabsToSpaces" automatisch die Tab-Weite
'       benutzt, die in PSPad eingestellt ist. 2020-10-29.
' ------------------------------------------------------------------------------
Function TabsToSpacesEx(s_Text, o_bj)

  Dim sTmp

  sTmp = tabsToSpaces(s_Text)

  If Right(s_Text, Len(vbCRLF)) <> vbCRLF Then
    If Right(sTmp, Len(vbCRLF)) = vbCRLF Then
      sTmp = Left(sTmp, Len(sTmp) - Len(vbCRLF))
    End If
  End If

  TabsToSpacesEx = sTmp
End Function ' ==> TabsToSpacesEx


' ==============================================================================
' CallTip_Show
'
' Anzeigen eines Call-Tips bei einer Funktion.
' ------------------------------------------------------------------------------
Sub CallTip_Show()

  Dim obj, oCOM_Dict
  Dim sTitel_PSPadWin, sTitel_Tmp
  Dim sCallTipViewerPath
  Dim sCommandLine
  Dim sLinesLeftAndAboveTheCaret ' Text links vom Caret und alle Zeilen darüber, die ein LCC haben.
  Dim iCaretX_Old, iCaretY_Old
  Dim iCaretPos_X, iCaretPos_Y
  Dim iCaretPos_X_Old, iCaretPos_Y_Old

  Dim aCaretPosExtern ' Array mit externen X- und Y-Koordinaten.

  Dim bLoopIsRunning
  bLoopIsRunning = False

  iCaretX_Old = 0 : iCaretPos_X = 0 : iCaretPos_X_Old = 0
  iCaretY_Old = 0 : iCaretPos_Y = 0 : iCaretPos_Y_Old = 0

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    CallTip_Start() ' CallTipViewer neustarten, falls er NICHT (mehr) läuft (z. B. wg Fehler).

    Set obj = NewEditor()
    obj.assignActiveEditor()

    Set oCOM_Dict = GetObject(gc_sCOM_ROT_Name)

    If oCOM_Dict.Item("ShowCallTip") = True Then Exit Sub

    If (obj.caretY < obj.topLineIndex()) _
      Or (obj.caretY >= (obj.topLineIndex() + obj.linesInWindow())) Then Exit Sub

    sLinesLeftAndAboveTheCaret = CallTip_GetLinesLeftAndAboveTheCaret(obj)

    sLinesLeftAndAboveTheCaret = TabsToSpacesEx(sLinesLeftAndAboveTheCaret, obj)

    If InStr(1, sLinesLeftAndAboveTheCaret, "(", vbTextCompare) = 0 Then Exit Sub

    iCaretX_Old = 0 ' CaretX ist 1-basierend, dadurch kann man gut auf CaretX = 0 prüfen.
    iCaretY_Old = obj.caretY
    aCaretPosExtern = Split(oCOM_Dict.Item("CaretPosExtern"), "|")
    iCaretPos_X_Old = aCaretPosExtern(0) ' absolute screen coordinates
    iCaretPos_Y_Old = aCaretPosExtern(1) ' absolute screen coordinates

    Do ' Loop While oCOM_Dict.Item("ShowCallTip") = True

      aCaretPosExtern = Split(oCOM_Dict.Item("CaretPosExtern"), "|")
      iCaretPos_X = aCaretPosExtern(0) ' absolute screen coordinates
      iCaretPos_Y = aCaretPosExtern(1) ' absolute screen coordinates

      If bLoopIsRunning = True Then

        If (iCaretPos_X_Old <> iCaretPos_X) And (iCaretX_Old = obj.caretX) Then Exit Do
        If iCaretPos_Y_Old <> iCaretPos_Y Then Exit Do

        If (obj.caretY >= (obj.topLineIndex() + obj.linesInWindow())) Then Exit Do

        If iCaretY_Old <> obj.caretY Then Exit Do

        If oCOM_Dict.Item("PSPadEditorPath") <> getVarValue("%FullFileName%") Then Exit Do
      End If

      If iCaretX_Old <> obj.caretX Then

        iCaretX_Old = obj.caretX      ' Aktuelles internes CaretX speichern.
        iCaretPos_X_Old = iCaretPos_X ' Aktuelles externes CaretX speichern.

        sLinesLeftAndAboveTheCaret = CallTip_GetLinesLeftAndAboveTheCaret(obj)
        oCOM_Dict.Item("LinesLeftAndAbove") = TabsToSpacesEx(sLinesLeftAndAboveTheCaret, obj)
        oCOM_Dict.Item("CaretPosCoordinates") = _
          CaretX_TTS(obj) & "|" & obj.caretY & "|" & iCaretPos_X & "|" & iCaretPos_Y
        oCOM_Dict.Item("PSPadEditorPath") = getVarValue("%FullFileName%")
      End If ' If iCaretX_Old <> obj.caretX

      If bLoopIsRunning = False Then
        bLoopIsRunning = True
        oCOM_Dict.Item("ShowCallTip") = True
      End If

      Sleep(10) ' Hauptschleife
    Loop While oCOM_Dict.Item("ShowCallTip") = True

    oCOM_Dict.Item("ShowCallTip") = False

  End If ' If LCase(getVarValue("%Ext%")) = LCase(".au3")
End Sub ' ==> CallTip_Show


' ==============================================================================
' CallTip_Start
'
' Routine zum Starten des CallTipViewer.
'
' Der CallTipViewer läuft im Hintergrund. Erst wenn der Shortcut für
' CallTip_Show gedrückt wird, wird auch das CallTipWin gezeigt. Der
' CallTipViewer läuft derzeit solange, bis Windows beendet wird. 2020-04-24.
' ------------------------------------------------------------------------------
Sub CallTip_Start()

  Dim oShell, oCOM_Dict
  Dim sAutoIt3ExePath_Own, sCallTipViewerPath, sWorkingDir
  Dim sCommandLine, sErrMsgTxt
  Dim iErrNum

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sCallTipViewerPath = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\CallTipViewer.au3"

  sWorkingDir = getVarValue("%Temp%")

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sCallTipViewerPath) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sCallTipViewerPath & """" & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'CallTip_Start'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Sub
  End If

  Set oShell = CreateObject("Wscript.Shell")

  On Error resume next  ' Fehlerbehandlung einschalten => es werden KEINE Fehler gezeigt.

  Set oCOM_Dict = GetObject(gc_sCOM_ROT_Name)

  On Error GoTo 0       ' Fehlerbehandlung ausschalten => Fehler werden wieder gezeigt.

  If IsObject(oCOM_Dict) = False Then

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sCallTipViewerPath & """" _
      & " " & gc_sTitel_CallTipWin _
      & " " & gc_sCOM_ROT_Name

    oShell.CurrentDirectory = sWorkingDir ' Aktuelles WorkingDirectory ändern.
    oShell.Run(sCommandLine)              ' Programm ausführen.

    CallTip_WaitUntil_COM_Server_Exists(10)

  End If ' If IsObject(oCOM_Dict) = False
End Sub ' ==> CallTip_Start


' ==============================================================================
' CallTip_WaitUntil_COM_Server_Exists
'
' Routine die wartet, bis der COM Server gestartet ist und das IROT COM Objekt
' existiert.
' ------------------------------------------------------------------------------
Sub CallTip_WaitUntil_COM_Server_Exists(i_Seconds)

  Dim oCOM_Dict
  Dim iStartTime

  On Error resume next

  iStartTime = Timer()
  Do
    Sleep(10)

    If (Timer() - iStartTime) >= i_Seconds Then Exit Do

    Set oCOM_Dict = GetObject(gc_sCOM_ROT_Name)

  Loop Until IsObject(oCOM_Dict) = True ' Alternative: "IsObject()" prüfen.

End Sub ' ==> CallTip_WaitUntil_COM_Server_Exists


' ==============================================================================
' Init
'
' name "Init" is required, its called automatically during initialization to create menu items
' ------------------------------------------------------------------------------
Sub Init()

  Dim sAu3Menu
  Dim oCOM_Dict

  CallTip_Start()

  Set oCOM_Dict = GetObject(gc_sCOM_ROT_Name)
  oCOM_Dict.Item("CheckColorScheme") = True

  sAu3Menu = ""

  addMenuItem "CallTip show"                  , sAu3Menu, "CallTip_Show"               , "Shift+Ctrl+Space"
End Sub ' ==> Init
