Option Explicit

' = Info ===========================================================================================
' Title           : AutoIt3_CompRun_LogWin.vbs
' --------------------------------------------------------------------------------------------------
'
Const module_name = "AutoIt3_CompRun_LogWin"
Const module_ver  = "0.0.36"
Const module_desc = "Routines for compiler run and LogWin commands."
' Date            : 2021-04-30
' Author(s)       : Professor Bernd
'
' Requirements    :
' Language        :
' ==================================================================================================


' global constants (gc_...)
' Const gc_sExample = "Hello world!"
'
' Erkennungswort, das von CompilerRunner ins Log-Win geschrieben wird,
' und an dem hier erkannt werden kann, ob CompilerRunner läuft.
Const gc_sIdentifierForCompRun = "CompilerRunner"


' WICHTIG! Globale Variablen können NICHT im Init() initialisiert werden!
' =======  (Aber sie können hier im globalen Bereich initialisiert werden.
'           Der globale Bereich wird noch VOR der Init() Routine ausgeführt!)
'
' global variables                    :  initialize
'
Dim g_sLastUsedAu3ForCompilerRunner   :  g_sLastUsedAu3ForCompilerRunner = ""
Dim g_sLogWinShowAfterRunAndCompile   :  g_sLogWinShowAfterRunAndCompile = "1"
Dim g_bCompileWasCalled               :  g_bCompileWasCalled = False
Dim g_bF4WasCalledByCompilerRunner    :  g_bF4WasCalledByCompilerRunner = False


' ==============================================================================
' Au3_Stop
'
' Stoppen eines au3-Scripts mithilfe des Hotkeys Ctrl+Break von AutoIt3Wrapper.
'
' Diese Funktion wird per Shortcut Ctrl+Pause aufgerufen (siehe Init-Funktion)
' und sendet Ctrl+Break, das vom AutoIt3Wrapper ausgewertet wird, um das
' laufende Au3 Script zu beenden. Zudem wird diese Funktion per Doppelklick
' auf " -- Stop -- " in den Fav-Scripts aufgerufen.
' ------------------------------------------------------------------------------
Sub Au3_Stop()

  Dim oWshShell

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  oWshShell.SendKeys "^{BREAK}" ' Ctrl+Break wird von AutoIt3Wrapper im System registriert.

  Set oWshShell = nothing
End Sub ' ==> Au3_Stop


' ==============================================================================
' Au3_Run
'
' Ausführen eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_Run()
  Call AutoIt3_RunCompilerRunner("/Run")
End Sub ' ==> Au3_Run


' ==============================================================================
' Au3_Check
'
' Syntax-Check eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_Check()
  Call AutoIt3_RunCompilerRunner("/Check")
End Sub ' ==> Au3_Check


' ==============================================================================
' Au3_Compile
'
' Kompilieren eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_Compile()
  g_bCompileWasCalled = True
  Call AutoIt3_RunCompilerRunner("/Compile")
End Sub ' ==> Au3_Compile


' ==============================================================================
' Au3_CompileDialog
'
' Kompilieren (mit Dialog) eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_CompileDialog()
  g_bCompileWasCalled = True
  Call AutoIt3_RunCompilerRunner("/CompileDialog")
End Sub ' ==> Au3_CompileDialog


' ==============================================================================
' Au3_RunBeta
'
' Ausführen eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_RunBeta()
  Echo("""Beta Run"" not yet available.")
End Sub ' ==> Au3_RunBeta


' ==============================================================================
' Au3_CheckBeta
'
' Syntax-Check eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_CheckBeta()
  Echo("""Beta Check Syntax"" not yet available.")
End Sub ' ==> Au3_CheckBeta


' ==============================================================================
' Au3_CompileBeta
'
' Kompilieren eines au3-Scripts mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub Au3_CompileBeta()
  Echo("""Beta Compile"" not yet available.")
End Sub ' ==> Au3_CompileBeta


' ==============================================================================
' AutoIt3_RunCompilerRunner
'
' Ausführen eines au3-Scripts mithilfe eines externen Programms.
'
' s_TypToRun: "/Run", "/Check", "/Compile", "/CompileDialog"
' ------------------------------------------------------------------------------
Sub AutoIt3_RunCompilerRunner(s_TypToRun)

  Dim obj, oWshShell
  Dim sAutoIt3ExePath_User, sAutoIt3ExePath_Own, sAutoIt3WrapperPath, sCompilerRunnerPath
  Dim sActiveAu3Path
  Dim sCommandLine, sErrMsgTxt, sLine, sLineExtern
  Dim sPSPadLanguage, sLangIniPath, sReadOnlyTitle, sReadOnlyMsg
  Dim iRetVal, iLnsCnt, iTopLineIdx

  If LCase(getVarValue("%Ext%")) <> LCase(".au3") Then
    Echo("Only available for .au3 files.")
    Exit Sub
  End If

  sAutoIt3ExePath_User = AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
    module_name & ".vbs", "AutoIt3_RunCompilerRunner()") & "AutoIt3.exe"
  sAutoIt3ExePath_User = Replace(sAutoIt3ExePath_User, """", "")
  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = False Then
    Call AutoIt3___Base.DownloadHelper_Show()
    Exit Sub
  End If

  If logLinesCount > 0 Then
    If Left(logGetLine(0), Len(gc_sIdentifierForCompRun)) = gc_sIdentifierForCompRun Then
      Echo("'CompilerRunner' is already running" & vbCRLF & vbCRLF & vbCRLF _
        & "Message form: " & module_name & ".vbs" & vbCRLF & vbCRLF _
        & "Running script:" & vbCRLF _
        & """" & g_sLastUsedAu3ForCompilerRunner & """")
      Exit Sub
    Else
      logClear()
    End If
  End If

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sAutoIt3WrapperPath = getVarValue("%PSPath%") _
    & "PSPad4Au3\AutoIt3\SciTE\AutoIt3Wrapper\AutoIt3Wrapper.au3"

  sCompilerRunnerPath = getVarValue("%PSPath%") _
    & "PSPad4Au3\Au3 scripts\CompilerRunner.au3"

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sAutoIt3WrapperPath) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sAutoIt3WrapperPath & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sCompilerRunnerPath) = False Then _
     sErrMsgTxt = sErrMsgTxt & """" & sCompilerRunnerPath & """" & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'AutoIt3_RunCompilerRunner'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Sub
  End If

  Set obj = NewEditor()
  obj.assignActiveEditor()

  iTopLineIdx = obj.topLineIndex()

  If obj.caretX() = 1 Then
    obj.command("ecRight")
    obj.command("ecLeft")
  Else ' obj.caretX() > 1
    obj.command("ecLeft")
    obj.command("ecRight")
  End If

  obj.topLineIndex(iTopLineIdx)                                      ' 2020-03-11.

    sActiveAu3Path = getVarValue("%FullFileName%")

    If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = True Then

      If (AutoIt3___Base.FileReadOnly_IsSet(sActiveAu3Path) = True) Then

        sPSPadLanguage = AutoIt3___Base.INI_IniRead(getVarValue("%PSPath%") _
          & "PSPad.ini", "Config", "Localization", "ENGLISH")

        sLangIniPath = getVarValue("%PSPath%") & "Language\" & sPSPadLanguage & ".ini"

        sReadOnlyTitle = AutoIt3___Base.INI_IniRead(sLangIniPath, _
          "General Strings", "rs_Warning", "Warning")

        sReadOnlyMsg = AutoIt3___Base.INI_IniRead(sLangIniPath, _
          "General Strings", "rs_AttribInfo", "File has set attributes: ")
        sReadOnlyMsg = Replace(sReadOnlyMsg, "%s", "", 1, -1, vbTextCompare) ' TextCompare = case insensitive.
        sReadOnlyMsg = sReadOnlyMsg & AutoIt3___Base.INI_IniRead(sLangIniPath, _
          "Action List", "aReadOnly_Caption", "Read Only") & vbCRLF
        sReadOnlyMsg = sReadOnlyMsg & AutoIt3___Base.INI_IniRead(sLangIniPath, _
          "General Strings", "rs_SaveFileChangesQuestion", "Save Changes Made to File?")

        If MsgBox(sReadOnlyMsg, vbSystemModal + vbExclamation + vbYesNo, sReadOnlyTitle) = vbYes Then
          obj.saveFile() ' Speichert OHNE Nachfrage, auch wenn die Datei ReadOnly ist!
        Else ' Falls der User "Nein" oder das "X" zum Schließen klickt.
          Exit Sub
        End If
      Else ' Die Datei ist NICHT ReadOnly.
        obj.saveFile() ' Speichert OHNE Nachfrage, auch wenn die Datei ReadOnly ist!
      End If ' If AutoIt3___Base.FileReadOnly_IsSet(sActiveAu3Path) = True
    Else ' Die Datei existiert NICHT.
      runPSPadAction("aSaveAs")
    End If ' If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = True

  sActiveAu3Path = getVarValue("%FullFileName%")

  If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = True Then

    g_sLogWinShowAfterRunAndCompile = AutoIt3___Base.INI_Pau3IniRead("LogWin", "ShowAfterCompile", _
      module_name & ".vbs", "AutoIt3_RunCompilerRunner()")
    If (g_sLogWinShowAfterRunAndCompile <> "0") And (g_sLogWinShowAfterRunAndCompile <> "1") Then
      g_sLogWinShowAfterRunAndCompile = "1"
      Call AutoIt3___Base.INI_Pau3IniWrite("LogWin", "ShowAfterCompile", g_sLogWinShowAfterRunAndCompile, _
        module_name & ".vbs", "AutoIt3_RunCompilerRunner()")
    End If

    If g_sLogWinShowAfterRunAndCompile <> "0" Then LogWin_Show()

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sCompilerRunnerPath & """" _
      & " " & s_TypToRun _
      & " " & """" & sActiveAu3Path & """" _
      & " " & """" & getVarValue("%PSPath%") & """" _
      & " " & gc_sIdentifierForCompRun ' Erkennungswort für VBScript, dass CompilerRunner läuft.

    iRetVal = oWshShell.Run(sCommandLine)

    g_sLastUsedAu3ForCompilerRunner = sActiveAu3Path

    g_bF4WasCalledByCompilerRunner = True
  End If
End Sub ' ==> AutoIt3_RunCompilerRunner


' ==============================================================================
' CheckAutoIt3Wrapper_AutoIncrFileVer
'
' Prüfen ob im Au3 die AutoIt3Wrapper Direktive für AutoIncrement gesetzt ist.
' Falls ja, speichert AutoIt3Wrapper extern im Au3 die erhöhte Versionsnummer.
' Diese Routine aktualisiert den aktiven Editor so, dass zum einen die neue
' Versionsnummer angezeigt wird, und zum anderen die "UnDo's" erhalten bleiben.
'
' Aufruf der Funktion erfolgt in "LogWin_JumpThroughErrors()".
'
' Hinweis: Das ginge viel einfacher mit reloadFile(), dann würden jedoch alle
'          "UnDo's" und "ReDo's" gelöscht.
' ------------------------------------------------------------------------------
Sub CheckAutoIt3Wrapper_AutoIncrFileVer()

  Dim fso, oFile
  Dim obj, objTmp
  Dim sLine, sLineExtern
  Dim iLnsCnt, iOld_X, iOld_Y
  Dim bAutoIncrement

  Const iMaxLns = 100 ' Die ersten 100 Zeilen durchsuchen dürfte genügen.
  Const ForReading = 1, ForWriting = 2, ForAppending = 8

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    Set obj = NewEditor()
    obj.assignActiveEditor()
    obj.activate()

    iLnsCnt = 0
    bAutoIncrement = False

    For Each sLine In obj ' access to editor lines line by line
      iLnsCnt = iLnsCnt + 1
      If iLnsCnt > iMaxLns Then Exit For

      If InStr(1, sLine, "#AutoIt3Wrapper_Res_Fileversion=", vbTextCompare) > 0 Then
        iOld_X = obj.caretX() ' Aktuelle Caret ...
        iOld_Y = obj.caretY() ' ... Position merken.
      End If

      If InStr(1, sLine, "#AutoIt3Wrapper_Res_Fileversion_AutoIncrement=y", vbTextCompare) > 0 Then
        bAutoIncrement = True
      End If

      If (bAutoIncrement = True) And (iOld_X > 0) And (iOld_Y > 0) Then Exit For
    Next

    If (bAutoIncrement = True) Then

      Set fso = CreateObject("Scripting.FileSystemObject")
      Set oFile = fso.OpenTextFile(getVarValue("%FullFileName%"), ForReading)

      iLnsCnt = 0

      Do While Not oFile.AtEndOfStream
        iLnsCnt = iLnsCnt + 1
        If iLnsCnt > iMaxLns Then Exit Do

        sLineExtern = oFile.ReadLine  ' Zeile aus externer Datei einlesen.

        If InStr(1, sLineExtern, "#AutoIt3Wrapper_Res_Fileversion=", vbTextCompare) > 0 Then

          obj.caretY(iLnsCnt)         ' Zeile mit der Fileversion im Editor aktiv setzen.
          sLine = obj.lineText()      ' Zeil aus Editor einlesen.

          If (InStr(1, sLine, "#AutoIt3Wrapper_Res_Fileversion=", vbTextCompare) > 0) _
            And (sLine <> sLineExtern) Then

            obj.lineText(sLineExtern) ' Zeile im Editor durch externe Zeile ersetzen.
            obj.modified(False)       ' Editor auf "gespeichert" setzen.
            Exit Do
          End If
        End If
      Loop

      oFile.Close

      If iOld_X > 0 Then obj.caretX(iOld_X)
      If iOld_Y > 0 Then obj.caretY(iOld_Y)

    End If ' bAutoIncrement = True
  End If ' If LCase(getVarValue("%Ext%")) = LCase(".au3")
End Sub ' ==> CheckAutoIt3Wrapper_AutoIncrFileVer


' ==============================================================================
' LogWin_AutoCloseIfNoError
'
' Routine zum automatischen Schließen des LogWin, falls keine Fehler*
' aufgetreten sind (*Fehler, Wanungen, Debug-Meldungen).
'
' Die Einstellungen dazu werden aus der Pau3-Ini ("PSPad4AutoIt3.ini")
' gelesen Sektion "[LogWin]". 2021-04-23, Beispiel mit Standard-Werten:
'
'   [LogWin]
'   AutoCloseDelay=2
'   ; Close the LogWin automatically if no errors occur ...
'   AutoCloseIfNoError=1
'   ; ... but not if debug messages occur.
'   AutoCloseButNotIfDbgMsg=1
' ------------------------------------------------------------------------------
Sub LogWin_AutoCloseIfNoError(b_ErrWarnMsgFound, b_DebugMsgFound)

  Dim sLogWinAutoCloseIfNoError
  Dim sLogWinAutoCloseButNotIfDbgMsg
  Dim sLogWinAutoCloseDelay
  Dim sTmpDelay

  Dim bLogWinCloseDelayed : bLogWinCloseDelayed = False

  sLogWinAutoCloseDelay = AutoIt3___Base.INI_Pau3IniRead("LogWin", "AutoCloseDelay", _
    module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  sTmpDelay = sLogWinAutoCloseDelay
  On Error resume next  ' Fehlerbehandlung einschalten => es werden KEINE Fehler gezeigt.
  If CInt(sLogWinAutoCloseDelay) < CInt(0) Then sLogWinAutoCloseDelay = "0"
  If CInt(sLogWinAutoCloseDelay) > CInt(10) Then sLogWinAutoCloseDelay = "10"
  If Err.Number <> 0 Then sLogWinAutoCloseDelay = "2" ' Standard: 2 Sekunden.
  On Error GoTo 0       ' Fehlerbehandlung ausschalten => Fehler werden wieder gezeigt.
  If sTmpDelay <> sLogWinAutoCloseDelay Then
    Call AutoIt3___Base.INI_Pau3IniWrite("LogWin", "AutoCloseDelay", sLogWinAutoCloseDelay, _
      module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  End If

  sLogWinAutoCloseIfNoError = AutoIt3___Base.INI_Pau3IniRead("LogWin", "AutoCloseIfNoError", _
    module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  If (sLogWinAutoCloseIfNoError <> "0") And (sLogWinAutoCloseIfNoError <> "1") Then
    sLogWinAutoCloseIfNoError = "1"
    Call AutoIt3___Base.INI_Pau3IniWrite("LogWin", "AutoCloseIfNoError", sLogWinAutoCloseIfNoError, _
      module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  End If

  sLogWinAutoCloseButNotIfDbgMsg = AutoIt3___Base.INI_Pau3IniRead("LogWin", "AutoCloseButNotIfDbgMsg", _
    module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  If (sLogWinAutoCloseButNotIfDbgMsg <> "0") And (sLogWinAutoCloseButNotIfDbgMsg <> "1") Then
    sLogWinAutoCloseButNotIfDbgMsg = "1"
    Call AutoIt3___Base.INI_Pau3IniWrite("LogWin", "AutoCloseButNotIfDbgMsg", sLogWinAutoCloseButNotIfDbgMsg, _
      module_name & ".vbs", "LogWin_AutoCloseIfNoError()")
  End If

  If sLogWinAutoCloseIfNoError = "1" And b_ErrWarnMsgFound = False Then
    If sLogWinAutoCloseButNotIfDbgMsg = "0" Or (sLogWinAutoCloseButNotIfDbgMsg = "1" And b_DebugMsgFound = False) Then
      bLogWinCloseDelayed = True
    End If
  End If

  If bLogWinCloseDelayed = True And logIsVisible() = True Then

    AutoIt3___Base.WaitForMilliSecs(sLogWinAutoCloseDelay * 1000) ' sec to ms

    If logLinesCount > 0 Then
      If Left(logGetLine(0), Len(gc_sIdentifierForCompRun)) <> gc_sIdentifierForCompRun Then
        LogWin_Hide()
      End If
    End If

  End If ' If bLogWinCloseDelayed = True And logIsVisible() = True
End Sub ' ==> LogWin_AutoCloseIfNoError


' ==============================================================================
' LogWin_Clear
'
' Log-Fenster leeren.
' ------------------------------------------------------------------------------
Sub LogWin_Clear()
  logClear()
  g_sLastUsedAu3ForCompilerRunner = ""
End Sub ' ==> LogWin_Clear


' ==============================================================================
' LogWin_Hide
'
' Log-Fenster verstecken.
' ------------------------------------------------------------------------------
Sub LogWin_Hide()
  If logIsVisible() = True Then runPSPadAction("aLogWindow")
End Sub ' ==> LogWin_Hide


' ==============================================================================
' LogWin_Show
'
' Log-Fenster anzeigen und Tab "Log" aktivieren.
' ------------------------------------------------------------------------------
Sub LogWin_Show()

  If logLinesCount() = 0 Then
    logAddLine()  ' Log-Win falls nötig erstellen, und zeigen.
  Else
    Call logSetLine(0, logGetLine(0))
  End If
End Sub ' ==> LogWin_Show


' ==============================================================================
' LogWin_Toggle
'
' Log-Fenster Sichtbarkeit umschalten (show/hide).
'
' Diese Routine habe ich erstellt, um im Init() einen zweiten Shortcut für das
' Togglen (abwechselndes Zeigen und Verstecken) des LogWins vergeben zu können.
' Für VBScript-Shortcuts kann man Befehle NICHT direkt angeben.
'   Ctrl+L: PSPad Standard  => Eingerichtet in den PSPad Shortcuts (KeyMap.ini).
'   F8:     SciTE4AutoIt3   => Eingerichtet im Init() dieses Scripts.
' ------------------------------------------------------------------------------
Sub LogWin_Toggle()
  runPSPadAction("aLogWindow")
End Sub ' ==> LogWin_Toggle


' ==============================================================================
' LogWin_JumpThroughErrors (F4)
'
' Durchlaufen der Fehlerzeilen im Log-Win und highlighten der Fehlerzeilen
' im Editor.
' ------------------------------------------------------------------------------
Sub LogWin_JumpThroughErrors()

  Dim obj

  Dim sLine
  Dim sAu3Path
  Dim sRowColmnInfo

  Dim iOldLineIdx, iLineIdx, iIdx
  Dim iErrLineX, iErrLineY
  Dim iCommaPos

  Dim bEditorExists     : bEditorExists = False
  Dim bContainsDebugMsg : bContainsDebugMsg = False
  Dim bErrWarnMsgFound  : bErrWarnMsgFound = False
  Dim bDebugMsgFound    : bDebugMsgFound = False

  If (g_bCompileWasCalled = True) Then ' Nur nach "Compile..." 1x ausführen.
    g_bCompileWasCalled = False
    CheckAutoIt3Wrapper_AutoIncrFileVer() ' Tauscht nur 1 Zeile aus = wenig Flickern.
  End If

  If (logLinesCount() > 1) Then

    Set obj = NewEditor()

    If g_sLogWinShowAfterRunAndCompile <> "0" Then LogWin_Show()

    sAu3Path = g_sLastUsedAu3ForCompilerRunner ' LogWin_Clear() setzt die Variable zurück.

    If sAu3Path <> "" Then

      obj.assignActiveEditor()
      If obj.fileName() = sAu3Path Then
        bEditorExists = True
        obj.activate()
      Else
        For iIdx = 0 To editorsCount() - 1
          obj.assignEditorByIndex(iIdx)
          If obj.fileName() = sAu3Path Then
            bEditorExists = True
            obj.activate() ' Bewirkt auch Wechsel von anderem zu diesem Editor-Tab.
            Exit For
          End If
        Next
      End If
    End If

    If bEditorExists = True Then

      iLineIdx = logLineIndex() ' Wenn nichts markiert: -1
      iOldLineIdx = iLineIdx
      If iOldLineIdx = -1 Then iOldLineIdx = logLinesCount() - 1

      Do
        If iLineIdx < (logLinesCount() - 1) Then
          iLineIdx = iLineIdx + 1
        Else ' letzte Zeile
          iLineIdx = 0
        End If

        sLine = logGetLine(iLineIdx)

        If (InStr(1, sLine, ") : error: ", vbTextCompare) > 0) _
            Or (InStr(1, sLine, ") : warning: ", vbTextCompare) > 0) Then

          bErrWarnMsgFound = True

          sRowColmnInfo = AutoIt3___Base.StringBetween(sLine, ".au3" & Chr(34) & ">(", ") : ")
          Exit Do

        ElseIf InStr(1, sLine, "@@ Debug>(", vbTextCompare) > 0 Then

          bDebugMsgFound = True

          If g_bF4WasCalledByCompilerRunner = False Then

            bContainsDebugMsg = True

            sRowColmnInfo = AutoIt3___Base.StringBetween(sLine, "@@ Debug>(", ") : ")
            Exit Do
          End If
        End If
      Loop Until iLineIdx = iOldLineIdx ' Absicherung gg. Endlosschleife.

      If sRowColmnInfo <> "" Then

        If logLineIndex() > iLineIdx Then logLineIndex(0)
        logLineIndex(iLineIdx)            ' Im Log-Win die gefundene Zeile markieren.

        If obj.caretX() = 1 Then obj.command("ecRight") ' Falls schon auf Pos 1, dann 1x nach rechts.
        obj.caretX(1)                                   ' Setzen auf Pos 1.

        iCommaPos = InStr(1, sRowColmnInfo, ",", vbTextCompare)
        iErrLineX = Mid(sRowColmnInfo, iCommaPos + 1)    ' Spaltennummer
        iErrLineY = Mid(sRowColmnInfo, 1, iCommaPos - 1) ' Zeilennummer

        obj.caretX(iErrLineX)             ' Im Editor das Caret an die Fehlerstelle setzen (Spalte).
        obj.caretY(iErrLineY)             ' Im Editor das Caret an die Fehlerstelle setzen (Zeile).

        If bContainsDebugMsg = False Then ' Debug-Meldungen nicht highlighten wenn F4 gedrückt wird. ...
          obj.markEditorLine(iErrLineY)   ' ... Funktioniert jedoch weiterhin mit Doppel-Klick ins LogWin.
        End If
      End If
    End If ' bEditorExists

    logSetParser("*>(%L,%C) : %E")

    g_bF4WasCalledByCompilerRunner = False

    Call LogWin_AutoCloseIfNoError(bErrWarnMsgFound, bDebugMsgFound)
  End If ' If (logLinesCount() > 1)
End Sub ' ==> LogWin_JumpThroughErrors


' ==============================================================================
' Init
'
' name "Init" is required, its called automatically during initialization to create menu items
' ------------------------------------------------------------------------------
Sub Init()

  Dim sAu3Menu, sAu3_Run

  sAu3Menu = ""
  sAu3_Run = "  Run and Compile"

  addMenuItem "Au3 -    Stop"                 , sAu3_Run, "Au3_Stop"                   , "Ctrl+Pause" ' = Shortcut Ctrl+Break von AutoIt3Wrapper
  addMenuItem "Au3 -   Run"                   , sAu3_Run, "Au3_Run"                    , "F5"
  addMenuItem "Au3 -  Check Syntax"           , sAu3_Run, "Au3_Check"                  , "Ctrl+F5"
  addMenuItem "Au3 -  Compile"                , sAu3_Run, "Au3_Compile"                , "F7"
  addMenuItem "Au3 -  Compile Dialog"         , sAu3_Run, "Au3_CompileDialog"          , "Ctrl+F7"

  addMenuItem "LogWin - clear log window"     , sAu3Menu, "LogWin_Clear"               , "Shift+Ctrl+L"
  addMenuItem "LogWin - jump through errors"  , sAu3Menu, "LogWin_JumpThroughErrors"   , "F4"
  addMenuItem "LogWin - show/hide"            , sAu3Menu, "LogWin_Toggle"              , "F8"
End Sub ' ==> Init
