Option Explicit

' = Info ===========================================================================================
' Title           : AutoIt3__Main.vbs
' --------------------------------------------------------------------------------------------------
'
Const module_name = "AutoIt3__Main"
Const module_ver  = "0.0.292"
Const module_desc = "Collection of helper functions for PSPad4AutoIt3."  'script description
' Date            : 2021-05-02
' Author(s)       : Professor Bernd
'
' Requirements    :
' Language        :
' ==================================================================================================


' global constants (gc_...)
' Const gc_sExample = "Hello world!"


' WICHTIG! Globale Variablen können NICHT im Init() initialisiert werden!
' =======  (Aber sie können hier im globalen Bereich initialisiert werden.
'           Der globale Bereich wird noch VOR der Init() Routine ausgeführt!)
'
' global variables
' Dim g_KodaFirstStartDone


' ==============================================================================
' FileVerIsGreaterOrEqualMinVer
'
' Routine zum Vergleichen der Datei Version mit der Minimum Version.
' ------------------------------------------------------------------------------
Function FileVerIsGreaterOrEqualMinVer(s_MinVersion, s_FileVersion)

  Dim aMinVer, aFileVer
  Dim iIdx
  Dim sErrMsg
  Dim bFileVerIsGreaterOrEqual

  sErrMsg = ""
  bFileVerIsGreaterOrEqual = True

  If (s_MinVersion = "") Then sErrMsg = sErrMsg & "Error in minimum version." & vbCRLF

  If (s_FileVersion = "") Then sErrMsg = sErrMsg & "Error in file version." & vbCRLF

  If sErrMsg = "" Then
    aMinVer = Split(s_MinVersion, ".")
    aFileVer = Split(s_FileVersion, ".")

    If (UBound(aMinVer) > -1) And (UBound(aFileVer) > -1) Then
      If UBound(aMinVer) <> UBound(aFileVer) Then
        sErrMsg = sErrMsg &  "Version numbers have a different quantity of blocks." & vbCRLF
      End If
    End If
  End If ' If sErrMsg = ""

  If sErrMsg <> "" Then
    bFileVerIsGreaterOrEqual = False

    If LCase(s_MinVersion) <> "file not found" Then
      Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
        & "Function: 'FileVerIsGreaterOrEqualMinVer'" & vbCRLF & vbCRLF & sErrMsg)
    End If
  Else ' MinVer mit FileVer vergleichen.

    If (UBound(aMinVer) > -1) And (UBound(aFileVer) > -1) Then
      For iIdx = 0 To UBound(aFileVer)
        If CInt(aMinVer(iIdx)) > CInt(aFileVer(iIdx)) Then
          bFileVerIsGreaterOrEqual = False
          Exit For
        ElseIf CInt(aMinVer(iIdx)) < CInt(aFileVer(iIdx)) Then
          Exit For
        End If
      Next
    End If
  End If

  FileVerIsGreaterOrEqualMinVer = bFileVerIsGreaterOrEqual
End Function ' ==> FileVerIsGreaterOrEqualMinVer


' ==============================================================================
' GetAutoIt3exe_MinVersion
'
' Routine zum Auslesen der Mindest-Version für die "AutoIt3.exe" aus der
' Datei "VersionsChecker.lst".
'
' Rückgabe  Erfolg: Die ausgelesene Mindest-Version der "AutoIt3.exe".
'           Fehler: "file not found".
' ------------------------------------------------------------------------------
Function GetAutoIt3exe_MinVersion()

  Dim fso, oFile
  Dim sMinVer, sVerCheckerLst_Path, sLine
  Dim aMinVer

  Const ForReading = 1, ForWriting = 2, ForAppending = 8

  sVerCheckerLst_Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\VersionsChecker.lst"

  If AutoIt3___Base.FileExistsEx(sVerCheckerLst_Path) = False Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'GetAutoIt3exe_MinVersion'" & vbCRLF & vbCRLF _
      & "File not found: " & vbCRLF & vbCRLF & sVerCheckerLst_Path)

    sMinVer = "file not found"
  Else
    Set fso = CreateObject("Scripting.FileSystemObject")
    Set oFile = fso.OpenTextFile(sVerCheckerLst_Path, ForReading)

    Do While Not oFile.AtEndOfStream

      sLine = LTrim(oFile.ReadLine)  ' Zeile aus externer Datei einlesen, führende Leerzeichen entfernen.

      aMinVer = Split(sLine, "|")

      If UBound(aMinVer) > 1 Then
        If aMinVer(0) = "BaseDir_AutoIt3" And aMinVer(1) = "AutoIt3.exe" Then
          sMinVer = aMinVer(2)
          Exit Do
        End If
      End If
    Loop

    oFile.Close
  End If

  GetAutoIt3exe_MinVersion = sMinVer
End Function ' ==> GetAutoIt3exe_MinVersion


' ==============================================================================
' StrIsEmptyOrOnlyWhitespace
'
' Author: Geert Bellekens (original code), Professor Bernd.
' Quelle: https://stackoverflow.com/a/47553896
'
' Checks if this string is empty or has only whitespace characters.
' ------------------------------------------------------------------------------
Function StrIsEmptyOrOnlyWhitespace(s_String) ': boolean

  Dim bRetVal
  Dim sNewString

  bRetVal = False

  sNewString = Replace(s_String, vbTab, "")
  sNewString = Replace(sNewString, vbNewline, "")

  If Len(Trim(sNewString)) = 0 Then bRetVal = True

  StrIsEmptyOrOnlyWhitespace = bRetVal
End Function ' ==> StrIsEmptyOrOnlyWhitespace


Sub AboutDialog_Run()

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sAboutDialogAu3_Path, sCommandLine, sErrMsgTxt

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sAboutDialogAu3_Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\AboutDialog.au3"

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sAboutDialogAu3_Path) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sAboutDialogAu3_Path & """" & vbCRLF & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'AboutDialog_Run'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Sub
  End If

  sCommandLine = """" & sAutoIt3ExePath_Own & """" _
    & " " & """" & sAboutDialogAu3_Path & """"

  Call oWshShell.Run(sCommandLine)

End Sub ' ==> AboutDialog_Run


' ==============================================================================
' AutoIt3Help_Helper
'
' Routine zum Aufrufen der sprachspezifischen Hilfe (Language-Help)
' mithilfe eines externen Programms/.au3-Scripts.
'
' Für au3-Dateien wird die Hilfe mittels "AutoIt3Help.exe" (im AutoIt3-
' Install-Ordner des Users) aufgerufen, die scheinbar in der Hilfedatei
' zusätzliche Funktionen ermöglicht, wie das Öffnen von Code-Beispielen
' im Editor, und das Speichern ins Clipboard.
'
' Dieses VBScript ruft derzeit ein au3-Script auf, das dann die Hilfe
' aufruft, wie oben beschrieben. Das hat den Vorteil, dass damit das
' Hilfe-Fenster in den Vordergrund geholt wird (SetForegroundWindow).
'
' Das ist nützlich, da sich das Hilfe-Fenster gerne mal hinter andere
' Fenster schiebt.
' ------------------------------------------------------------------------------
Sub AutoIt3Help_Helper()

  Dim oWshShell
  Dim obj
  Dim sAutoIt3ExePath_User, sAutoIt3ExePath_Own, sAu3Path
  Dim sSearchWord, sErrMsgTxt

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  Set obj = NewEditor()
  obj.assignActiveEditor()
  obj.activate()

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    sAutoIt3ExePath_User = AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
      module_name & ".vbs", "AutoIt3Help_Helper()") & "AutoIt3.exe"
    sAutoIt3ExePath_User = Replace(sAutoIt3ExePath_User, """", "")
    If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = False Then
      Call AutoIt3___Base.DownloadHelper_Show()
      Exit Sub
    End If

    sSearchWord = obj.selText
    If sSearchWord = "" Then sSearchWord = getVarValue("%Word%")

    sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

    sAu3Path = getVarValue("%PSPath%") _
      & "PSPad4Au3\Au3 scripts\HelpForAutoIt3_Helper.au3"

    If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
      sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
    If AutoIt3___Base.FileExistsEx(sAu3Path) = False Then _
       sErrMsgTxt = sErrMsgTxt & """" & sAu3Path & """" & vbCRLF

    If sErrMsgTxt <> "" Then
      Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
        & "Function: 'AutoIt3Help_Helper'" & vbCRLF & vbCRLF _
        & "Dateien nicht gefunden: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Else
      oWshShell.Run("""" & sAutoIt3ExePath_Own & """" & " " & """" & sAu3Path & """" _
        & " " & sSearchWord)
    End If

  Else ' Aktuelle Datei ist KEINE au3.

    runPSPadAction("aLangHelp")
  End If
End Sub ' ==> AutoIt3Help_Helper


' ==============================================================================
' CheckAutoItAndSciTE_OfTheUser
'
' Routine zum Überprüfen, ob "AutoIt3.exe" und "SciTE.exe" existieren, die vom
' User installiert wurden. - Die Prüfungen erfolgen stumm (OHNE Fehlermeldungen).
'
' --------------------------------------------------------------------------
' Hier in dieser Funktion wird sichergestellt, dass in der PSPad4AutoIt3.ini alle
' Pfade OHNE Anführungszeichen und MIT abschließendem Backslash gespeichert werden.
' --------------------------------------------------------------------------
'
' "AutoIt3.exe"
' Etwa seit 2021-01-20 ist PSPad4Au3 voll portabel, d.h. alle Funktionen laufen
' OHNE Programme außerhalb des PSPad-Ordners, sogar der CallTipViewer! Aber
' natürlich wird eine AutoIt-Installation des Users benötigt, um die Funktionen
' "Run, Compile, Check, ..." ausführen zu können.
'
' "SciTE.exe"
' Etwa seit 2020-09-22 werden die benötigten SciTE-Dateien im PSPad4Au3-Ordner
' mitgeführt ("...\PSPad4Au3\AutoIt3\SciTE\"). Die SciTE-Installation des Users
' wird hier ermittelt, damit z.B. der CallTipViewer den Pfad zur "au3.api" des
' Users auslesen kann, da sie in der vom User bevorzugten Sprache sein könnte
' oder zu seiner evtl. älteren AutoIt-Version passt. Falls KEINE gefunden wird,
' wird die mitgeführte "au3.api" im Pau3-Ordner benutzt.
'
' Die Pfade werden automatisch ermittelt. Durchsucht werden Einträge in der
' Pau3-Ini von PSPad4Au3, der Standard-Pfad von AutoIt und die Registry.
' Falls ein gültiger Pfad gefunden wird, wird er in die Pau3-Ini eingetragen.
' Falls KEIN gültiger Pfad gefunden wird, wird der entsprechende Eintrag in der
' Pau3-Ini auf leer gesetzt.
'
' Wenn der Eintrag für das AutoIt3_Dir in der Pau3-Ini leer ist, wird bei jedem
' Aufruf von "Run, Compile, ..." der DownloadHelper aufgerufen, der außer dem
' AutoIt3-Download auch eine Möglichkeit bietet, eine "AutoIt3.exe" des Users
' manuell per DateiÖffnen-Dialog auszuwählen, auch von AutoIt-portable.
'
' Hier in dieser Routine erfolgt KEINE User-Abfrage, weil ich es nervig finde,
' beim ersten oder gar bei jedem Start eines neuen Programms direkt mit Abfragen
' überhäuft zu werden!
' ------------------------------------------------------------------------------
Sub CheckAutoItAndSciTE_OfTheUser(s_Au3HelpMenu)

  Dim fso, oWshShell
  Dim sAutoIt3ExePath_User, sSciTEExePath_User, sPSPad4AutoIt3_IniPath
  Dim sAutoIt_Dir, sSciTE_Dir, sMissingFile, sErrMsgTxt, sMinVer, sFileVer
  Dim bFilesFound

  Const sRegKey = "HKEY_LOCAL_MACHINE\SOFTWARE\AutoIt v3\AutoIt\InstallDir"

  bFilesFound = True

  Set fso = WScript.CreateObject("Scripting.FileSystemObject")
  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sPSPad4AutoIt3_IniPath = getVarValue("%PSPath%") & "PSPad4Au3\PSPad4AutoIt3.ini"

  If AutoIt3___Base.FileExistsEx(sPSPad4AutoIt3_IniPath) = False Then

    Call AutoIt3___Base.INI_Pau3IniWrite("Paths", "AutoIt3_Dir", "", _
      module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()")

    If AutoIt3___Base.FileExistsEx(sPSPad4AutoIt3_IniPath) = False Then Exit Sub
  End If

  sAutoIt3ExePath_User = ""
  sAutoIt3ExePath_User = AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
    module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()") & "AutoIt3.exe"

  sAutoIt3ExePath_User = Replace(sAutoIt3ExePath_User, """", "")

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = False Then     ' Pfad aus Pau3-Ini existiert NICHT.
    sAutoIt3ExePath_User = "C:\Program Files (x86)\AutoIt3\AutoIt3.exe"

    If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = False Then   ' Standard-Pfad existiert NICHT.
      On Error resume next  ' Fehlerbehandlung einschalten => es werden KEINE Fehler gezeigt.
      sAutoIt3ExePath_User = oWshShell.RegRead(sRegKey) & "\AutoIt3.exe"
      On Error GoTo 0       ' Fehlerbehandlung ausschalten => Fehler werden wieder gezeigt.
    End If
  End If ' If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = False

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = True Then      ' Pfad existiert.

    sFileVer = fso.GetFileVersion(sAutoIt3ExePath_User)
    sMinVer = GetAutoIt3exe_MinVersion()

    If FileVerIsGreaterOrEqualMinVer(sMinVer, sFileVer) = True Then

      sAutoIt_Dir = AutoIt3___Base.ExtractFilePath(sAutoIt3ExePath_User) & "\"

      If AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
        module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()") <> sAutoIt_Dir Then

        Call AutoIt3___Base.INI_Pau3IniWrite("Paths", "AutoIt3_Dir", sAutoIt_Dir, _
          module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()")
      End If
    Else ' Datei-Version ist kleiner als die Mindest-Version!

      addMenuItem "DownloadHelper for AutoIt3", s_Au3HelpMenu, "DownloadHelper_CallFromBaseModule" ' Ohne Shortcut

      If AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
        module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()") <> "" Then

        Call AutoIt3___Base.INI_Pau3IniWrite("Paths", "AutoIt3_Dir", "", _
          module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()")
      End If
    End If
  Else                                                                  ' Pfad existiert NICHT.
    bFilesFound = False
  End If ' If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_User) = True

  sSciTEExePath_User = ""

  If bFilesFound = True Then
    sSciTEExePath_User = AutoIt3___Base.INI_Pau3IniRead("Paths", "SciTE_Dir", _
      module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()") & "SciTE.exe"

    sSciTEExePath_User = Replace(sSciTEExePath_User, """", "")

    If AutoIt3___Base.FileExistsEx(sSciTEExePath_User) = False Then     ' Pfad aus Pau3-Ini existiert NICHT.
      sSciTEExePath_User = "C:\Program Files (x86)\AutoIt3\SciTE\SciTE.exe"

      If AutoIt3___Base.FileExistsEx(sSciTEExePath_User) = False Then   ' Standard-Pfad existiert NICHT.
        On Error resume next  ' Fehlerbehandlung einschalten => es werden KEINE Fehler gezeigt.
        sSciTEExePath_User = oWshShell.RegRead(sRegKey) & "\SciTE\SciTE.exe"
        On Error GoTo 0       ' Fehlerbehandlung ausschalten => Fehler werden wieder gezeigt.
      End If
    End If

    If AutoIt3___Base.FileExistsEx(sSciTEExePath_User) = True Then      ' Pfad existiert.

      sSciTE_Dir = AutoIt3___Base.ExtractFilePath(sSciTEExePath_User) & "\"

      If sSciTE_Dir <> AutoIt3___Base.INI_Pau3IniRead("Paths", "SciTE_Dir", _
        module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()") Then

        Call AutoIt3___Base.INI_Pau3IniWrite("Paths", "SciTE_Dir", sSciTE_Dir, _
          module_name & ".vbs", "CheckAutoItAndSciTE_OfTheUser()")
      End If
    End If
  End If ' If bFilesFound = True
End Sub ' ==> CheckAutoItAndSciTE_OfTheUser


' ==============================================================================
' CheckFileVersions_PSPad4AutoIt3
'
' Versionen der PSPad4AutoIt3-Dateien prüfen mithilfe eines externen Programms.
' ------------------------------------------------------------------------------
Sub CheckFileVersions_PSPad4AutoIt3(s_ShowVerInfo)

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sAu3Path
  Dim sCommandLine, sErrMsgTxt
  Dim iRetVal

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sAu3Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\VersionsChecker.au3"

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sAu3Path) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sAu3Path & """" & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'CheckFileVersions_PSPad4AutoIt3'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
  Else ' Dateien wurden gefunden.

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sAu3Path & """" _
      & " " & s_ShowVerInfo

    Call oWshShell.Run(sCommandLine)
  End If
End Sub ' ==> CheckFileVersions_PSPad4AutoIt3


' ==============================================================================
' CheckReadOnlyAndProvideACopyWithoutReadOnly
'
' Wenn aus der AutoIt Hilfe ein Example geöffnet wird, ist es ReadOnly. Um im
' Editor Änderungen zuzulassen, ohne das Original zu verändern, wird eine
' Kopie in Win-Temp erstellt und statt dem Original in den Editor geladen.
'
' Diese Routine wird in Init() aufgerufen. Somit funktioniert dieses Konzept
' nur, wenn in den PSPad-Einstellungen mehrere PSPad-Instanzen erlaubt sind.
' Dadurch wird bei jedem Aufruf aus der AutoIt-Hilfe eine PSPad-Instanz gestartet.
' ------------------------------------------------------------------------------
Sub CheckReadOnlyAndProvideACopyWithoutReadOnly()
  Dim obj, fso
  Dim sTmpDirSubFolder, sAutoIt3Dir, sActiveAu3Path, sLeftPartOfActiveAu3Path ' sAutoIt3HlpExamplesPath
  Dim sTmpDirSubSubDir_Extras, sAuotIt3_HlpExtrasDir

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    sAutoIt3Dir = AutoIt3___Base.INI_Pau3IniRead("Paths", "AutoIt3_Dir", _
      module_name & ".vbs", "CheckReadOnlyAndProvideACopyWithoutReadOnly()") ' Inkl. abschließendem "\".

    sActiveAu3Path = getVarValue("%FullFileName%")

    sLeftPartOfActiveAu3Path = Left(sActiveAu3Path, Len(sAutoIt3Dir & "Examples"))

    If LCase(sLeftPartOfActiveAu3Path) = LCase(sAutoIt3Dir & "Examples") Then

      Set fso = CreateObject("Scripting.FileSystemObject")

      sTmpDirSubFolder = getVarValue("%Temp%") & "\PSPad4AutoIt3_HelpExamples_Temp\"

      If fso.FolderExists(sTmpDirSubFolder) = False Then fso.CreateFolder(sTmpDirSubFolder)

      If fso.FolderExists(sTmpDirSubFolder) = True Then

        Set obj = NewEditor()
        obj.assignActiveEditor()

        obj.text(tabsToSpaces(obj.text))

        obj.saveFileAs(sTmpDirSubFolder & getVarValue("%FileName%"))

        obj.openFile(sTmpDirSubFolder & getVarValue("%FileName%"))

        sAuotIt3_HlpExtrasDir = sAutoIt3Dir & "Examples\Helpfile\Extras\"

        sTmpDirSubSubDir_Extras = sTmpDirSubFolder & "Extras\"

        If fso.FolderExists(sTmpDirSubSubDir_Extras) = False Then

          fso.CreateFolder(sTmpDirSubSubDir_Extras)

          If fso.FolderExists(sTmpDirSubSubDir_Extras) = True Then

            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.ani", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.exe", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.dll", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.bmp", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.cur", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.emf", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.ico", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.jpg", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.xls", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.doc", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.png", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.ttf", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.txt", sTmpDirSubSubDir_Extras
            fso.CopyFile sAuotIt3_HlpExtrasDir & "*.wav", sTmpDirSubSubDir_Extras

          End If ' If fso.FolderExists(sTmpDirSubSubDir_Extras)
        End If ' If fso.FolderExists(sTmpDirSubSubDir_Extras) = False
      End If ' If fso.FolderExists(sTmpDirSubFolder)
    End If ' If sLeftPartOfActiveAu3Path = sAutoIt3Dir
  End If ' If LCase(getVarValue("%Ext%")) = LCase(".au3")
End Sub ' ==> CheckReadOnlyAndProvideACopyWithoutReadOnly


' ==============================================================================
' CheckVeryFirstStart
'
' Aktionen beim allerersten Start von PSPad4AutoIt3 ausführen.
' ------------------------------------------------------------------------------
Sub CheckVeryFirstStart()
  If AutoIt3___Base.INI_Pau3IniRead("General", "Pau3FirstStartAfterInstallIsDone", _
      module_name & ".vbs", "CheckVeryFirstStart()") <> "1" Then
    Call AutoIt3___Base.INI_Pau3IniWrite("General", "Pau3FirstStartAfterInstallIsDone", "1", _
      module_name & ".vbs", "CheckVeryFirstStart()")
    ReadMe_Show() ' Tips and shortcuts
  End If
End Sub ' ==> CheckVeryFirstStart


' ==============================================================================
' DebugTo_General
'
' Auslesen des Wortes (Variablenname) unter dem Cursor oder des markierten
' Textes, und Einfügen einer entsprechenden DebugLine im Editor. Wenn das
' au3-Script ausgeführt wird, wird die DebugLine im s_OutputDevice angezeigt.
'
'   s_OutputDevice: "Console", "MsgBox".
'
' Diese Routine funktioniert auch bei Zeilen mit Zeilenfortsetzungzeichen.
' ------------------------------------------------------------------------------
Sub DebugTo_General(s_OutputDevice)

  Dim obj         ' Editor-Objekt.
  Dim sVarName    ' Die Variable am Cursor, für die eine Debug-Meldung erstellt werden soll.
  Dim sDebugLine  ' Die Zeile, die im Editorfenster eingefügt werden soll.
  Dim sLine       ' Text der jeweiligen Zeile.
  Dim sWSpaces    ' Whitespace Zeichen am Zeilenanfang (Leerzeichen oder Tabulatoren).

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    Set obj = NewEditor()
    obj.assignActiveEditor()
    obj.activate()

    sVarName = obj.selText()
    If (InStr(1, obj.lineText(), sVarName, vbTextCompare) = 0) Then
      Call MsgBox("Place the cursor in a variable, or select the desired " _
        & "text by marking it and press the shortcut." & vbCRLF & vbCRLF _
        & "You can only select text within one line.", _
        vbSystemModal + vbInformation, "DebugToConsole / DebugToMsgbox")
      Exit Sub
    End If

    If sVarName = "" Then sVarName = getVarValue("%Word%")

    If sVarName = "" Then
      Call MsgBox("Place the cursor in a variable, or select the desired " _
        & "text by marking it and press the shortcut.", _
        vbSystemModal + vbInformation, "DebugToConsole / DebugToMsgbox")
    Else

      If s_OutputDevice = "Console" Then
        sDebugLine = "ConsoleWrite('@@ Debug(' & @ScriptLineNumber & ') : " _
          & sVarName & " = ' & " & sVarName _
          & " & @CRLF & '>Error code: ' & @error & @CRLF) ;### Debug Console"

      ElseIf s_OutputDevice = "MsgBox" Then
        sDebugLine = _
          "MsgBox(262144, 'Debug line ~' & @ScriptLineNumber, 'Selection:' & @CRLF & '" _
          & sVarName & "' & @CRLF & @CRLF & 'Return:' & @CRLF & " & sVarName _
          & ") ;### Debug MSGBOX"
      End If

      obj.caretX(1)                 ' Cursor ganz an den Zeilenanfang setzen.
      obj.command("ecSelWordRight") ' Whitespace bis zum ersten Wort markieren.
      sWSpaces = obj.selText()      ' Markierte Whitespace Zeichen speichern.

      If StrIsEmptyOrOnlyWhitespace(sWSpaces) = False Then sWSpaces = ""

      Do
        sLine = trim(obj.lineText())
        If right(sLine, 2) = " _" Then
          obj.command("ecDown")       ' Cursor jeweils eine Zeile tiefer setzen.
        Else
          Exit Do
        End If
      loop

      If obj.caretY() = obj.linesCount() Then obj.appendText(vbNewline)

      obj.command("ecDown")           ' Cursor eine Zeile tiefer setzen.
      obj.command("ecInsertLine")     ' Leere Zeile über der aktuellen einfügen.
      obj.caretX(1)                   ' Cursor ganz an den Zeilenanfang setzen.
      obj.selText(sWSpaces & sDebugLine)
      obj.caretX(1)                   ' Cursor ganz an den Zeilenanfang setzen.
      obj.command("ecDown")           ' Cursor eine Zeile tiefer setzen.
    End If
  End If
End Sub ' ==> DebugTo_General


' ==============================================================================
' DebugToConsole
'
' Fügt eine Codezeile ein, mit der Informationen zu der Variablen unter
' dem Cursor angezeigt werden können in der Console (Log-Fenster).
' ------------------------------------------------------------------------------
Sub DebugToConsole()
  DebugTo_General("Console")
End Sub ' ==> DebugToConsole


' ==============================================================================
' DebugToMsgbox
'
' Fügt eine Codezeile ein, mit der Informationen zu der Variablen unter
' dem Cursor angezeigt werden können in einer MsgBox.
' ------------------------------------------------------------------------------
Sub DebugToMsgbox()
  DebugTo_General("MsgBox")
End Sub ' ==> DebugToMsgbox


' ==============================================================================
' DownloadHelper_CallFromBaseModule
'
' Hilfs-Routine zum Aufrufen des DownloadHelper per AutoIt3-Menü-Unterpunkt.
'
' Die Routine "DownloadHelper_Show()" befindet sich im Modul "AutoIt3___Base.vbs".
' Um sie hier im Modul "AutoIt3__Main.vbs" aufzurufen, muss das Base-Modul im
' Aufruf vor die Routine gesetzt werden: "AutoIt3___Base.DownloadHelper_Show()".
' In Menü-Punkten darf aber KEINE Routine in anderen Modulen aufgerufen werden.
' Deshalb wird diese Hilfs-Routine benutzt, die im Menü-Punkt aufgerufen wird
' und ihrerseits "DownloadHelper_Show()" im Base-Modul aufruft.
' ------------------------------------------------------------------------------
Sub DownloadHelper_CallFromBaseModule()
  AutoIt3___Base.DownloadHelper_Show()
End Sub ' ==> DownloadHelper_CallFromBaseModule


' ==============================================================================
' FileTypesRegister_Run
'
' Routine zum Aufrufen des Scripts "FileTypesRegister.au3".
'
' Das Script zeigt eine GUI mit der der User die Datei-Typen ".au3" und ".ppr"
' zu Pau3 zuordnen kann, sodass sie z.B. per Doppelklick automatisch mit Pau3
' geöffnet werden.
'
' Es wird auch angezeigt, ob ".au3" und ".ppr" schon mit Pau3 verbunden sind.
'
' ------------------------------------------------------------------------------
' Sub FileTypeAssoc_Run()
Sub FileTypesRegister_Run()

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sFileTypesRegisterAu3_Path, sCommandLine, sErrMsgTxt

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sFileTypesRegisterAu3_Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\FileTypesRegister.au3"

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sFileTypesRegisterAu3_Path) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sFileTypesRegisterAu3_Path & """" & vbCRLF & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'FileTypesRegister_Run'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Sub
  End If

  sCommandLine = """" & sAutoIt3ExePath_Own & """" _
    & " " & """" & sFileTypesRegisterAu3_Path & """"

  Call oWshShell.Run(sCommandLine)

End Sub ' ==> FileTypesRegister_Run


' ==============================================================================
' Koda_RunFormDesigner
'
' Koda is a standalone application that was developed to help creating
' forms (GUIs) for AutoIt3 scripting language.
' ------------------------------------------------------------------------------
Sub Koda_RunFormDesigner()

  Dim oWshShell
  Dim sKodaExePath

  If LCase(getVarValue("%Ext%")) = LCase(".au3") Then

    Set oWshShell = CreateObject("WScript.Shell")

    sKodaExePath = getVarValue("%PSPath%") & "PSPad4Au3\3rdP\Koda\FD.exe"

    If AutoIt3___Base.FileExistsEx(sKodaExePath) = True Then

      Call AutoIt3___Base.Splasher_Run("Koda", "", 400) ' <= Splasher Start-Delay in ms.

      oWshShell.Run("""" & sKodaExePath & """")

      Call AutoIt3___Base.Splasher_Close()
    Else
      Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
        & "Function: 'Koda_RunFormDesigner'" & vbCRLF & vbCRLF _
        & "File not found: " & vbCRLF & vbCRLF & """" & sKodaExePath & """")
    End If
  Else
    Echo("Koda FormDesigner" & vbCRLF & vbCRLF & "Only available for .au3 files.")
  End If ' If LCase(getVarValue("%Ext%")) = LCase(".au3") ...

  Set oWshShell = Nothing
End Sub ' ==> Koda_RunFormDesigner


' ==============================================================================
' Pau3_OpenPau3Ini
'
' Derzeit gibt es noch keinen richtigen Settings-Dialog. Aber wenn der User will
' kann er die PSPad4AutoIt3.ini per Hand bearbeiten. In der Pau3-ini sind viele Infos.
' ------------------------------------------------------------------------------
Sub Pau3_OpenPau3Ini()
  Dim obj
  Dim sPathToTidyIni

  sPathToTidyIni = getVarValue("%PSPath%") & "PSPad4Au3\PSPad4AutoIt3.ini"

  If AutoIt3___Base.FileExistsEx(sPathToTidyIni) = True Then
    Set obj = NewEditor()
    obj.openFile(sPathToTidyIni)
  End If
End Sub ' ==> Pau3_OpenPau3Ini


' ==============================================================================
' ProjectPanelHiding
'
' Routine zum Verstecken des Projekt-Panels, wenn PSPad per Projekt-Datei
' (".ppr") gestartet wurde.
'
' Eigentlich merkt sich PSPad, ob das Projekt-Panel beim Schließen von PSPad
' sichtbar ist und stellt diesen Zustand nach einem PSPad-Neustart wieder her.
' Aber wenn man eine Projekt-Datei ".ppr" startet, wird das Projekt-Panel
' IMMER angezeigt, auch wenn es vorher ausgeblendet war. Dadurch muss man es
' jedesmal wieder von Hand ausblenden, wenn man es weg haben will!
'
' Diese Routine prüft zunächst, ob das Projekt-Panel beim letzten Schließen
' von PSPad versteckt war ("PSPad.ini") und ob es auch beim Starten von
' Projekt-Dateien versteckt werden soll ("PSPad4AutoIt3.ini"). Falls alle
' Bedingungen erfüllt sind, wird die Sichtbarkeit des Projekt-Panels bei
' jedem Start per ".ppr"-Datei wieder so hergestellt, wie sie vorher war.
'
' Dies ist ein Komfort-Feature, das nicht unbedingt benötigt wird. 2021-02-18.
' ------------------------------------------------------------------------------
Sub ProjectPanelHiding()

  Dim sPSPad4AutoIt3IniPath, sProjPanelHide, sShowProjPanel, sAu3SinlgeLine
  Dim iProjPnlVisible

  sPSPad4AutoIt3IniPath = getVarValue("%PSPath%") & "PSPad4Au3\PSPad4AutoIt3.ini"

  sProjPanelHide = AutoIt3___Base.INI_Pau3IniRead("Config", "ProjPanel_RememberVisibility", _
    module_name & ".vbs", "ProjectPanelHiding()")

  If (sProjPanelHide <> "0") And (sProjPanelHide <> "1") Then
    sProjPanelHide = "1"
    Call AutoIt3___Base.INI_Pau3IniWrite("Config", "ProjPanel_RememberVisibility", sProjPanelHide, _
      module_name & ".vbs", "ProjectPanelHiding()")

  End If

  sShowProjPanel = AutoIt3___Base.INI_IniRead(getVarValue("%PSPath%") _
    & "PSPad.ini", "Config", "ShowProject", "0") ' <= Wichtig: Der Key steht in der PSPad.ini, -NICHT- in der Pau3-Ini!

  If (sProjPanelHide <> "1") Or (sShowProjPanel <> "0") Then Exit Sub

  sAu3SinlgeLine = "Opt('WinTitleMatchMode', 2) & ControlCommand(WinGetHandle('" _
    & "[CLASS:TfPSPad]" & "'), '', 'TPageControl1', 'IsVisible', '')"

  iProjPnlVisible =  Right(AutoIt3___Base.ExecuteAu3Line(sAu3SinlgeLine, "/WaitForResult"), 1)

  If iProjPnlVisible = "1" Then runPSPadAction("aProjShow") ' Show/hide left Panel.

End Sub ' ==> ProjectPanelHiding


' ==============================================================================
' ReadMe_Show
'
' ReadMe als Mini-Hilfe anzeigen. Enthält derzeit AutoIt-relevante Shortcuts,
' einen Tipp zum Verknüpfen von au3-Dateien mit PSPad4Au3, und einen Link zum
' PSPad4Au3-Thread im DE AutoIt-Forum. 2020-02-12.
' ------------------------------------------------------------------------------
Sub ReadMe_Show()
  Dim obj
  Dim sPathToReadMeFile

  sPathToReadMeFile = getVarValue("%PSPath%") & "PSPad4Au3\Help\ReadMe.txt"

  If AutoIt3___Base.FileExistsEx(sPathToReadMeFile) = True Then
    Set obj = NewEditor()
    obj.openFile(sPathToReadMeFile)
  End If
End Sub ' ==> ReadMe_Show


' ==============================================================================
' RestoreImpKitFilesIfChangedByPSPadUpdate
'
' Prüfen der AutoIt Syntax und Definitions Dateien.
'
' Die Dateien "...\PSPad4AutoIt3\Context\AutoIt3.def" und
' "...\PSPad4AutoIt3\Syntax\AutoIt3.ini" dienen verschiedenen Zwecken, z. B.
' zum Highlighten oder für Code Clips usw. Bei PSPad-Updates werden diese
' beiden Dateien mitgeliefert und somit die für PSPad4Au3 angepassten Dateien
' überschrieben. Deshalb hier prüfen, ob die Dateien im PSPad-Ordner die
' richtigen sind, und falls nicht, die Dateien aus dem Backup wiederherstellen.
'
' Im Moment habe ich in der AutoIt3.ini den Kommentar in der ersten Zeile mit
' einer Erkennung ergänzt, die ich hier überprüfe. 2020-02-06.
'
' ------------------------------------------------------------------------------
Sub RestoreImpKitFilesIfChangedByPSPadUpdate()

  Dim fso, oFile
  Dim sLine1, sLine2, sErrMsg
  Dim sPSPad_Dir
  Dim sPathSynFile, sPathDefFile, sPathSynFile_Bak, sPathDefFile_Bak

  Const ForReading = 1, ForWriting = 2, ForAppending = 8

  sLine1 = "1"
  sLine2 = "2"
  sErrMsg = ""

  sPSPad_Dir = getVarValue("%PSPath%")

  sPathSynFile = sPSPad_Dir & "Syntax\AutoIt3.ini"
  sPathDefFile = sPSPad_Dir & "Context\AutoIt3.def"
  sPathSynFile_Bak = sPSPad_Dir & "PSPad4Au3\Bak\Syntax\AutoIt3.ini"
  sPathDefFile_Bak = sPSPad_Dir & "PSPad4Au3\Bak\Context\AutoIt3.def"

  If AutoIt3___Base.FileExistsEx(sPathSynFile_Bak) = False Then _
    sErrMsg = sErrMsg & vbNewline & vbNewline & """" & sPathSynFile_Bak & """"
  If AutoIt3___Base.FileExistsEx(sPathDefFile_Bak) = False Then _
    sErrMsg = sErrMsg & vbNewline & vbNewline & """" & sPathDefFile_Bak & """"

  If sErrMsg <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'RestoreImpKitFilesIfChangedByPSPadUpdate'" & vbCRLF & vbCRLF _
      & "Files not found:" & sErrMsg)
  Else ' Falls alle Backup-Dateien existieren.

    Set fso = CreateObject("Scripting.FileSystemObject")

    If AutoIt3___Base.FileExistsEx(sPathSynFile) = True Then
      Set oFile = fso.OpenTextFile(sPathSynFile, ForReading)
      sLine1 = oFile.ReadLine  ' Zeile aus derzeitiger Datei einlesen.
      oFile.Close

      Set oFile = fso.OpenTextFile(sPathSynFile_Bak, ForReading)
      sLine2 = oFile.ReadLine  ' Zeile aus Backup-Datei einlesen.
      oFile.Close
    End If

    If (AutoIt3___Base.FileExistsEx(sPathDefFile) = False) Or (sLine1 <> sLine2) Then

      fso.CopyFile sPathSynFile_Bak, sPSPad_Dir & "Syntax\"
      fso.CopyFile sPathDefFile_Bak, sPSPad_Dir & "Context\"

      reloadUserHighlighter("AutoIt3")
      reloadClips()
    End If

  End If ' If sErrMsg <> ""
End Sub ' ==> RestoreImpKitFilesIfChangedByPSPadUpdate


' ==============================================================================
' ShowScriptInExplorer
'
' Routine zum Öffnen des Windows Explorer mit dem Verzeichnis in dem sich das
' aktuelle Script befindet. Das aktuelle Script wird im Explorer markiert.
' ------------------------------------------------------------------------------
Sub ShowScriptInExplorer()

  Dim oWshShell, sCommandLine

  Set oWshShell = CreateObject("WScript.Shell")

  sCommandLine = """" & "explorer.exe" & """" & " /select, " _
    & """" & getVarValue("%FullFileName%") & """"

  oWshShell.Run(sCommandLine)

End Sub ' ==> ShowScriptInExplorer


' ==============================================================================
' Tidy_RunTidyForAu3
'
' ------------------------------------------------------------------------------
Sub Tidy_RunTidyForAu3()

  Dim obj, oWshShell, fso
  Dim sTitle_MB, sMsg, sErrMsg
  Dim sActiveAu3Path, sTidyINIPath, sTidyExePath, sTidyBak4UserScriptsDir, sCommandLine, sFileEncoding', sTidyParams
  Dim iRetVal, iButtons_MB, iRet_MB
  Dim bAuObjServerIsRunning

  sTidyINIPath = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\SciTE\Tidy\Tidy.ini"
  sTidyExePath = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\SciTE\Tidy\Tidy.exe"
  sTidyBak4UserScriptsDir = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\SciTE\Tidy\BakOfUserScripts\"
  sActiveAu3Path = getVarValue("%FullFileName%")
  sTitle_MB = "Tidy for .au3 files"

  If LCase(getVarValue("%Ext%")) <> LCase(".au3") Then          ' Nur für au3-Dateien ausführen.
    Echo("Tidy for Au3" & vbCRLF & vbCRLF & "Only available for .au3 files.")
    Exit Sub
  ElseIf AutoIt3___Base.FileExistsEx(sTidyExePath) = False Then ' Falls Tidy.exe nicht existiert.
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'Tidy_RunTidyForAu3'" & vbCRLF & vbCRLF _
      & "File not found: " & vbCRLF & vbCRLF & """" & sTidyExePath & """")
    Exit Sub
  End If

  Set oWshShell = CreateObject("WScript.Shell")
  Set fso = CreateObject("Scripting.FileSystemObject")

  If fso.FolderExists(sTidyBak4UserScriptsDir) = False Then
    fso.CreateFolder(sTidyBak4UserScriptsDir)
  End If

  If AutoIt3___Base.AutoItObjectServer_IsRunning("/Silent") = 1 Then
    bAuObjServerIsRunning = True
    sMsg = "Run Tidy for this file?" '& vbCRLF & vbCRLF & sActiveAu3Path
  Else
    bAuObjServerIsRunning = False
    sMsg = "Run Tidy for this file?" & vbCRLF & vbCRLF _
      & "(Due to limitations of PSPad existing ''Undo's'' are lost)." '_
  End If
  iRet_MB = MsgBox(sMsg, vbSystemModal + vbInformation + vbYesNoCancel, sTitle_MB)

  If iRet_MB = vbYes Then

    Set obj = NewEditor()
    obj.assignActiveEditor()

    If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = True Then

      If AutoIt3___Base.FileReadOnly_IsSet(sActiveAu3Path) = True Then

        sMsg = "'Read-only' is set for this file." & vbCRLF & vbCRLF _
          & "Should the 'read-only' be removed?"
        iRet_MB = MsgBox(sMsg, vbSystemModal + vbExclamation + vbYesNo, sTitle_MB)

        If iRet_MB = vbYes Then
          AutoIt3___Base.FileReadOnly_Remove(sActiveAu3Path)
        Else
          Exit Sub
        End IF
      End If

      obj.saveFile() ' Datei vor Tidy-Ausführung speichern.

    Else ' Falls die Datei (noch) NICHT existiert.

      runPSPadAction("aSave")

      sActiveAu3Path = getVarValue("%FullFileName%")

    End If ' If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = True

    If AutoIt3___Base.FileExistsEx(sActiveAu3Path) = False Then Exit Sub

    If AutoIt3___Base.INI_IniRead(sTidyINIPath, "ProgramSettings", "backupDir", "") <> sTidyBak4UserScriptsDir Then
      Call AutoIt3___Base.INI_IniWrite(sTidyINIPath, "ProgramSettings", "backupDir", sTidyBak4UserScriptsDir)
    End If

    sCommandLine = """" & sTidyExePath & """" & " " & """" & sActiveAu3Path & """" & " /portable"

    logClear()

    Call AutoIt3___Base.Splasher_Run("Tidy", "is running!", 800) ' <= Splasher Start-Delay in ms.

    If AutoIt3___Base.INI_IniRead(sTidyINIPath, "ProgramSettings", "ShowConsoleInfo", "0") = "0" Then
      iRetVal = oWshShell.Run(sCommandLine, 0, True) ' Tidy ausführen und warten, bis Tidy beendet ist.
    Else
      iRetVal = AutoIt3___Base.Run_OutputToLogWin(sCommandLine)       ' <= Bei Run() wird der Output erst nach Ende des Progs abgeholt.
    End If                                                            '    Bei Tidy ist das ok, da es nur sehr kurz läuft. 2021-03-14.

    Call AutoIt3___Base.Splasher_Close()

    If AutoIt3___Base.FileReadOnly_IsSet(sActiveAu3Path) = False Then obj.ReadOnly(False)

    sErrMsg = ""

    Select Case iRetVal

      Case -1 ' Datei war schon bereinigt.
        iButtons_MB = vbOKOnly
        sErrMsg = "file was already cleaned up"

      Case 0  ' Tidy erfolgreich ausgeführt.
        iButtons_MB = vbOKOnly
        sErrMsg = "successfully completed"

        If bAuObjServerIsRunning = True Then
          obj.activate() ' Falls der User den Tab gewecheslt hat, während Tidy läuft.
          AutoIt3___Base.ReloadOrReplaceActiveFile_AuObj("")
        Else
          obj.reloadFile()
        End If

      Case 1, 2, 3, 4 ' Mir unbekannte Fehler.
        iButtons_MB = vbYesNoCancel + vbQuestion
        sErrMsg = "Error " & iRetVal & vbCRLF & vbCRLF _
          & "Run syntax check?"

      Case 99 ' Falsche Zeichenkodierung oder Schreibschutz.
        iButtons_MB = vbOKOnly + vbExclamation

        sFileEncoding = AutoIt3___Base.GetFileEncoding(sActiveAu3Path)

        If sFileEncoding = "32" Then      ' "UTF-16LE"
          sErrMsg = "Input file is UTF16LE encoded." & vbCRLF & "Tidy does not support this and will be skipped."
        ElseIf sFileEncoding = "64" Then  ' "UTF-16BE"
          sErrMsg = "Input file is UTF16BE encoded." & vbCRLF & "Tidy does not support this and will be skipped."
        Else
          sErrMsg = "Error " & iRetVal & " - Tidy.exe  -  " &  "Is the file read-only?"
        End If

      Case Else
        iButtons_MB = vbYesNoCancel + vbQuestion
        sErrMsg = "Error " & iRetVal & vbCRLF & vbCRLF _
          & "Run syntax check?"
    End Select

    iRet_MB = MsgBox(sErrMsg, iButtons_MB, sTitle_MB)

    If iRet_MB = vbYes Then AutoIt3_CompRun_LogWin.Au3_Check()
  End If

  Set oWshShell = Nothing
End Sub ' ==> Tidy_RunTidyForAu3


' ==============================================================================
' Tidy_OpenTidyIni
'
' Derzeit gibt es noch keinen richtigen Settings-Dialog. Aber wenn der User will
' kann er die Tidy.ini per Hand bearbeiten. In der Tidy.ini sind viele Infos.
' ------------------------------------------------------------------------------
Sub Tidy_OpenTidyIni()
  Dim obj
  Dim sPathToTidyIni

  sPathToTidyIni = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\SciTE\Tidy\Tidy.ini"

  If AutoIt3___Base.FileExistsEx(sPathToTidyIni) = True Then
    Set obj = NewEditor()
    obj.openFile(sPathToTidyIni)
  End If
End Sub ' ==> Tidy_OpenTidyIni


' ==============================================================================
' Init
'
' name "Init" is required, its called automatically during initialization to create menu items
' ------------------------------------------------------------------------------
Sub Init()

  Dim sAu3Menu, sAu3Help, sAu3Settings

  CheckAutoItAndSciTE_OfTheUser(sAu3Help)

  CheckFileVersions_PSPad4AutoIt3("")

  RestoreImpKitFilesIfChangedByPSPadUpdate()

  CheckReadOnlyAndProvideACopyWithoutReadOnly()

  CheckVeryFirstStart()

  ProjectPanelHiding()

  sAu3Menu      = ""
  sAu3Help      = "        Help"
  sAu3Settings  = "        Settings (AutoIt3)"

  addMenuItem "                ====  AutoIt3  ====", ""       , ""                      ' <= Trenner zu Menüpunkt "Recompile internal VBScripts"

  addMenuItem "About PSPad4AutoIt3"           , sAu3Help      , "AboutDialog_Run"       ' Ohne Shortcut
  addMenuItem "Tips and shortcuts"            , sAu3Help      , "ReadMe_Show"           ' Ohne Shortcut
  addMenuItem "Help for script language"      , sAu3Help      , "AutoIt3Help_Helper"    , "F1"

  addMenuItem "Open PSPad4AutoIt3.ini"        , sAu3Settings  , "Pau3_OpenPau3Ini"      ' Ohne Shortcut
  addMenuItem "Open Tidy.ini  (Tidy for Au3)" , sAu3Settings  , "Tidy_OpenTidyIni"      ' Ohne Shortcut
  addMenuItem "Register file types"           , sAu3Settings  , "FileTypesRegister_Run" ' Ohne Shortcut

  addMenuItem "Debug to Console"              , sAu3Menu      , "DebugToConsole"        , "Ctrl+D"
  addMenuItem "Debug to MsgBox"               , sAu3Menu      , "DebugToMsgbox"         , "Shift+Ctrl+D"

  addMenuItem "Show script in explorer"       , sAu3Menu      , "ShowScriptInExplorer"  ' Ohne Shortcut

  addMenuItem "Koda FormDesigner"             , sAu3Menu      , "Koda_RunFormDesigner"  , "Alt+K"
  addMenuItem "Tidy for Au3"                  , sAu3Menu      , "Tidy_RunTidyForAu3"    , "Alt+T"

End Sub ' ==> Init
