Option Explicit

' = Info ===========================================================================================
' Title           : AutoIt3___Base.vbs
' --------------------------------------------------------------------------------------------------
'
Const module_name = "AutoIt3___Base"
Const module_ver  = "0.0.69"
Const module_desc = "This script is loaded first, before any others. It contains functions that can be used in other scripts."
' Date            : 2021-04-30
' Author(s)       : Professor Bernd
'
' Requirements    :
' Language        :
' ==================================================================================================


' Hint: Auf globale Variablen und Konstanten kann in anderen Modulen genauso
      ' zugegriffen werden, wie auf Funktionen, indem der Name des Moduls (das
      ' die Variable/Konstante/Funktion beinhaltet) und ein Punkt davor gesetzt
      ' werden. Beispiel: AutoIt3___Base.gc_sAuObjServerIdentifier

' global constants (gc_...)
' Const gc_sExample = "Hello world!"

Const gc_sKeyWordForDeleteINI_Key = "Delete_this_INI_key"

' Identifier with which the AutoItObject server is registered in "AutoItServer.au3"
' Kennung, mit der der AutoItObjekt-Server in der "AutoItServer.au3" registriert wird.
' Diese Kennung wird hier ("AutoIt3___Base.vbs") festgelegt und in der Funktion
' AutoItObjectServer_Run() per Start-Parameter an "AutoItServer.au3" gesendet.
Const gc_sAuObjServerIdentifier = "AutoIt.PSPad4AutoIt3_COM_Server"

' Window title as identifier with which Splasher can be closed.
Const gc_sSplasherWinTitle = "Splasher-PB"


' WICHTIG! Globale Variablen können NICHT im Init() initialisiert werden!
' =======  (Aber sie können hier im globalen Bereich initialisiert werden.
'           Der globale Bereich wird noch VOR der Init() Routine ausgeführt!)
'
' global variables
' Dim g_bExample


' ==============================================================================
' AutoItObjectServer_IsRunning
'
' Routine zum Prüfen, ob der AutoItObject-Server läuft.
'
' Parameter: s_Silent:
'           ""        - Bei Fehlern wird  eine Message angezeigt.
'           "/Silent" - Bei Fehlern wird KEINE Message angezeigt.
'
' Rückgabe: 1 = Server läuft.
'           0 = Server läuft NICHT.
'
' Autor: Professor Bernd, 2020-10-24.
' ------------------------------------------------------------------------------
Function AutoItObjectServer_IsRunning(s_Silent)
  Dim oAuObj
  Dim iIsRunning

  On Error Resume Next

  Set oAuObj = GetObject(gc_sAuObjServerIdentifier) ' the magic

  Call oAuObj.Call("FileExists", getVarValue("%PSPath%") _
    & "PSPad4Au3\Au3 scripts\misc\AutoItServer.au3")

  If Err.Number = 0 Then
    iIsRunning = 1
  Else
    iIsRunning = 0
    If LCase(s_Silent) <> LCase("/Silent") Then
      Echo("Message from " & "'" & module_name & ".vbs'" & vbCRLF _
        & "Function: 'AutoItObjectServer_IsRunning'" & vbCRLF & vbCRLF _
        & "AutoItObject-Server - Error" & vbCRLF & vbCRLF _
        & "Error no.:  " & Err.Number & vbCRLF _
        & "Source:  " & Err.Source & vbCRLF _
        & "Description:  " &  Err.Description)
    End If
    Err.Clear
  End If

  On Error GoTo 0

  AutoItObjectServer_IsRunning = iIsRunning
End Function ' ==> AutoItObjectServer_IsRunning


' ==============================================================================
' AutoItObjectServer_Run
'
' Routine zum Starten des AutoItObject-COM-Server.
'
' Der AutoItObject-COM-Server wird hier im "AutoIt3___Base.vbs" in "Init()"
' gestartet. Im "AutoItServer.au3" beendet sich der Server selbst, sobald
' kein PSPad-Fenster mehr existiert.
'
' Die Kennung für den AutoItObject-Server wird hier im "AutoIt3___Base.vbs"
' als Konstante "gc_sAuObjServerIdentifier" festgelegt und in dieser
' Funktion beim Starten des AutoItObject-Servers als Start-Parameter an
' "AutoItServer.au3" übergeben.
'
' Hint: Es dauert 4 - 5 Sekunden, bis der AuObjServer läuft!
'
' Autor: Professor Bernd, 2020-10-25.
' ------------------------------------------------------------------------------
Function AutoItObjectServer_Run()

  If AutoItObjectServer_IsRunning("/Silent") = 0 Then

    Dim oWshShell
    Dim sAutoIt3ExePath_Own, sAu3Path, sCommandLine, sErrMsgTxt
    Dim iRetVal

    Set oWshShell = WScript.CreateObject("WScript.Shell")

    sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

    sAu3Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\misc\AutoItServer.au3"

    If FileExistsEx(sAutoIt3ExePath_Own) = False Then _
      sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
    If FileExistsEx(sAu3Path) = False Then _
       sErrMsgTxt = sErrMsgTxt & """" & sAu3Path & """" & vbCRLF
    If sErrMsgTxt <> "" Then
      Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
        & "Function: 'AutoItObjectServer_Run'" & vbCRLF & vbCRLF _
        & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
      Exit Function
    End If

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sAu3Path & """" _
      & " " & gc_sAuObjServerIdentifier

    iRetVal = oWshShell.Run(sCommandLine)
  End If ' If AutoItObjectServer_IsRunning("/Silent") = 0

End Function ' ==> AutoItObjectServer_Run


' ==============================================================================
' DownloadHelper_Show
'
' Anzeigen der DownloadHelper.au3, eine GUI, die die Möglichkeit bietet,
' AutoIt3 aus dem Internet zu laden und zu installieren.
'
' Autor: Professor Bernd.
'
'   Rückgabe: Derzeit KEINE! 2021-01-27.
' ------------------------------------------------------------------------------
Function DownloadHelper_Show()

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sDownloadHelperAu3_Path, sCommandLine, sErrMsgTxt

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sDownloadHelperAu3_Path = getVarValue("%PSPath%") _
    & "PSPad4Au3\Au3 scripts\DownloadHelper.au3"

  If FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If FileExistsEx(sDownloadHelperAu3_Path) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sDownloadHelperAu3_Path & """" & vbCRLF & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'DownloadHelper_Show'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Function
  End If

  sCommandLine = """" & sAutoIt3ExePath_Own & """" _
    & " " & """" & sDownloadHelperAu3_Path & """"

  Call oWshShell.Run(sCommandLine)

End Function ' ==> DownloadHelper_Show


' ==============================================================================
' ExecuteAu3Line
'
' Funktion, mit der eine AutoIt SingleLine-Anweisung ausgeführt werden kann.
' Je nach Anweisung gibt "ExecuteAu3Line()" einen entsprechenden String zurück.
'
' Autor: Professor Bernd, 2020-11-01.
'
' Wichtig: "ExecuteAu3Line()" ist nur für "einfache" AutoIt Anweisungen gedacht!
'
'          Durch den Aufruf wird ein externes Programm (AutoIt3.exe) gestartet,
'          das dann eine SingleLine-Anweisung ausführt. Mit Tricks könnte man
'          das weit ausdehnen, aber das is NICHT empfehlenswert. Insbesondere
'          wenn der Parameter "/WaitForResult" benutzt wird, kann dadurch PSPad
'          total blockiert werden.
'
'          Geeignet ist es für AutoIt-Funktionen, die wenig "Drum-rum" benötigen
'          (z.B. Includes), oder deren Rückgabe nicht ausgewertet wird, oder
'          die einen Rückgabewert haben, der einfach weitergegeben werden kann
'          (schlecht wäre z.B. MouseGetPos(), Rückgabe: Ein Array mit X und Y).
'
' Parameter
'
'   s_Au3_SingleLine: String der die auszuführende AutoIt-Befehlszeile beinhaltet.
'
'           Für die SingleLine gemischte Anführungszeichen benutzen!
'
'           Äußere SingleLine-Bereiche mit doppelten Anführungszeichen
'           umschließen (VBScript kann nur doppelte) und innerhalb der
'           SingleLine nur einfache Anführungszeichen benutzen (AutoIt).
'
' Rückgabe
'
'   Wenn "/WaitForResult" gesetzt wurde: Einen String, der den Output (StdOut) enthält,
'   der von der aufgerufenen AutoIt-Funktion gesetzt wurde (per "ConsoleWrite").
'
'   Wenn "/WaitForResult" NICHT gesetzt wurde: -2.
'
'     (Dadurch bleiben weiter Rückgabe-Möglichkeiten offen, für die ich im
'      Moment noch keinen Wert festlegen kann. 2020-11-01.)
' ------------------------------------------------------------------------------
Function ExecuteAu3Line(s_Au3_SingleLine, s_WaitForResult)

  Dim oWshShell, oWshShellExec
  Dim sAutoIt3ExePath_Own', sAu3Path
  Dim sCommandLine, sErrMsgTxt, sOutput, sErrOutput

  Const iWshRunning = 0, iWshFinished = 1, iWshFailed = 2

  sOutput = "-2"

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  If FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'ExecuteAu3Line'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Function
  End If

  s_Au3_SingleLine = "ConsoleWrite(" & s_Au3_SingleLine & ")"

  sCommandLine = """" & sAutoIt3ExePath_Own & """" _
    & " /AutoIt3ExecuteLine " & """" & s_Au3_SingleLine & """"

  Set oWshShell = WScript.CreateObject("WScript.Shell")
  Set oWshShellExec = oWshShell.Exec(sCommandLine)

  If LCase(s_WaitForResult) = LCase("/WaitForResult") Then

    Do While oWshShellExec.Status = iWshRunning
      Sleep(10)
    Loop

        sOutput = oWshShellExec.StdOut.ReadAll()

  Else
  End If ' If LCase(s_WaitForResult) = LCase("/WaitForResult")

  Set oWshShell = Nothing
  Set oWshShellExec = Nothing

  ExecuteAu3Line = sOutput
End Function ' ==> ExecuteAu3Line


' ==============================================================================
' Execute_OutputToLogWin
'
' Routine zum Ausführen eines Programms, Auffangen des Outputs und zeilenweisen
' Anzeigen des Outputs im LogWin.
'
' Autor: Professor Bernd, 2021-03-13.
'
'   Rückgabe: Die Rückgabe des auszuführenden Programms.
' ------------------------------------------------------------------------------
Function Execute_OutputToLogWin(s_CommandLine)

  logClear() ' Mit leerem LogWin anfangen.

  Dim oWshShell, oExec
  Dim aOutput
  Dim sLine', sTmp, sRet
  Dim iCnt', iDblSpacesPos

  Const WshRunning = 0
  Const WshFinished = 1
  Const WshFailed = 2

  Set oWshShell = CreateObject("WScript.Shell")

  Set oExec = oWshShell.Exec(s_CommandLine)

  Do While oExec.Status = WshRunning

    aOutput = Split(oExec.StdOut.readall, vbLF) ' bzw. vbCRLF usw.

    For iCnt = 0 To UBound(aOutput) - 1

      sLine = aOutput(iCnt)

      logAddLine(sLine) ' oder "logAddLine(aOutput(iCnt))", falls Output NICHT bearbeitet wurde.
    Next

    Sleep(10)
  Loop

  Execute_OutputToLogWin = oExec.ExitCode
End Function ' ==> Execute_OutputToLogWin


' ==============================================================================
' ExtractFileExtension
'
' Extrahiert die Datei-Endung aus einem kopletten Datei-Pfad.
'
' Autor: Professor Bernd.
'
'   Rückgabe: Die Datei-Endung inkl. Punkt - oder "".
' ------------------------------------------------------------------------------
Function ExtractFileExtension(s_FullFilePath)
  Dim iPos
  iPos = InStrRev(s_FullFilePath, ".", -1)
  If iPos > 0 Then
    ExtractFileExtension = Mid(s_FullFilePath, iPos)     ' Mit Punkt
  Else
    ExtractFileExtension = ""
  End If

End Function ' ==> ExtractFileExtension


' ==============================================================================
' ExtractFileName
'
' Extrahiert den Datei-Namen aus einem kopletten Datei-Pfad.
'
' Autor: Professor Bernd.
'
'   Rückgabe: Der Datei-Name inkl. Endung - oder "".
' ------------------------------------------------------------------------------
Function ExtractFileName(s_FullFilePath)
  Dim iPos
  iPos = InStrRev(s_FullFilePath, "\", -1)
  If iPos > 0 Then
    ExtractFileName = Mid(s_FullFilePath, iPos + 1)
  Else
    ExtractFileName = ""
  End If
End Function ' ==> ExtractFileName


' ==============================================================================
' ExtractFilePath
'
' Extrahiert den Pfad-Anteil aus einem kopletten Datei-Pfad.
'
' Autor: Professor Bernd.
'
'   Rückgabe: Datei-Pfad inkl. LW-Buchstabe, ohne abschließendes "\" - oder "".
' ------------------------------------------------------------------------------
Function ExtractFilePath(s_FullFilePath)
  Dim iPos
  iPos = InStrRev(s_FullFilePath, "\", -1)
  If iPos > 0 Then
    ExtractFilePath = Left(s_FullFilePath, iPos - 1)
  Else
    ExtractFilePath = ""
  End If
End Function ' ==> ExtractFilePath


' ==============================================================================
' FileExistsEx
'
' Funktion zum Prüfen, ob eine Datei existiert. Erweitert um die Eigenschaft,
' dass der Pfad nicht nur aus dem Dateinamen + Endung bestehen darf.
'
' Hintergrund: Wird der original Funktion "FileExists()" lediglich ein Dateiname
'   + Endung übergeben, sucht die Funktion im aktuellen Verzeichnis und "baut"
'   sich einen Pfad zusammen. Dadurch wird das Ergebnis verfälscht, falls eine
'   Datei in PSPad neu erstellt wird (z. B. "New1.au3") und es schon eine Datei
'   mit selbem Namen + kompletten Pfad im aktuellen Verzeichnis gibt (z. B. Desktop).
'
'   Return: True or False.
' ------------------------------------------------------------------------------
Function FileExistsEx(s_FilePath) ': boolean
  Dim fso
  If InStr(1, s_FilePath, ":\", vbTextCompare) = 0 Then
    FileExistsEx = False
  Else
    Set fso = CreateObject("Scripting.FileSystemObject")
    FileExistsEx = fso.FileExists(s_FilePath)
    Set fso = Nothing
  End If
End Function ' ==> FileExistsEx


' ==============================================================================
' FileReadOnly_IsSet
'
' Read-only Attribut ermitteln.
'
'   Returns True if the attribute is set and False if it is not set.
'
'   Attributes: 1 = Read-only file. Attribute is read/write.
' ------------------------------------------------------------------------------
Function FileReadOnly_IsSet(s_FullFileName)

  Dim oFSO, oFile
  Dim bRetVal

  Set oFSO = CreateObject("Scripting.FileSystemObject")
  Set oFile = oFSO.GetFile(s_FullFileName)

  bRetVal = oFile.attributes And 1 ' Returns 1 if attribute is set and 0 if not.
  FileReadOnly_IsSet = CBool(bRetVal)
End Function


' ==============================================================================
' FileReadOnly_Remove
'
' Read-only Attribut entfernen.
'
'   Returns 1 if the attribute is set and 0 if it is not set.
'
'   Attributes: 1 = Read-only file. Attribute is read/write.
' ------------------------------------------------------------------------------
Function FileReadOnly_Remove(s_FullFileName)

  Dim oFSO, oFile

  Set oFSO = CreateObject("Scripting.FileSystemObject")
  Set oFile = oFSO.GetFile(s_FullFileName)

  If (oFile.attributes And 1) = 1 Then
    oFile.attributes = oFile.attributes - 1
  End If

  FileReadOnly_Remove = oFile.attributes And 1 ' Returns 1 if attribute is set and 0 if not.
End Function


' ==============================================================================
' FileReadOnly_Set
'
' Read-only Attribut setzen.
'
'   Returns 1 if the attribute is set and 0 if it is not set.
'
'   Attributes: 1 = Read-only file. Attribute is read/write.
' ------------------------------------------------------------------------------
Function FileReadOnly_Set(s_FullFileName)

  Dim oFSO, oFile

  Set oFSO = CreateObject("Scripting.FileSystemObject")
  Set oFile = oFSO.GetFile(s_FullFileName)

  If (oFile.attributes And 1) = 0 Then
    oFile.attributes = oFile.attributes + 1
  End If

  FileReadOnly_Set = oFile.attributes And 1 ' Returns 1 if attribute is set and 0 if not.
End Function


' ==============================================================================
' GetFileEncoding
'
' Dateikodierung einer Datei ermitteln, mithilfe von "ExecuteAu3Line()"
' (AutoIt3.exe und "/AutoIt3ExecuteLine"). Die Rückgabe wird von AutoIt
' per ConsoleWrite() in den StdOut Stream geschrieben und hier ausgelesen.
'
' Autor: Professor Bernd, 2020-10-20.
'
' Parameter:
'
'   s_Path_FileToExamine: Pfad zur Datei, deren Encoding ermittelt werden soll.
'
' Rückgabe: Weiterreichen der Rückgabe aus AutoIt's "FileGetEncoding()" als String(!).
'
' ------------------------------------------------------------------------
' Rückgabewerte (aus der AutoIt3 Hlfe zu "FileGetEncoding()")
' ------------------------------------------------------------------------
'
'   Erfolg: Die Dateikodierung unter Verwendung der gleichen Werte wie die FileOpen() Funktion.
'
'           $FO_UTF16_LE     (32) = UTF16 Little Endian.
'           $FO_UTF16_BE     (64) = UTF16 Big Endian.
'           $FO_UTF8        (128) = UTF8  (mit BOM).
'           $FO_UTF8_NOBOM  (256) = UTF8  (ohne BOM).
'           $FO_ANSI        (512) = ANSI  (enthält die Zeichen > 127 und < 255)
'
'   Fehler: -1.
'
'   Die Konstanten sind in FileConstants.au3 definiert.
' ------------------------------------------------------------------------------
Function GetFileEncoding(s_Path_FileToExamine)

  Dim sAu3SingleLine

  sAu3SingleLine = "FileGetEncoding('" & s_Path_FileToExamine & "')"

  GetFileEncoding = ExecuteAu3Line(sAu3SingleLine, "/WaitForResult")
End Function ' ==> GetFileEncoding


' ==============================================================================
' INI_DeleteINI_Key
'
' Deletes a Key(-Name) in the INI file.
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' ------------------------------------------------------------------------------
Function INI_DeleteINI_Key(s_FileName, s_Section, s_KeyName)
  Call INI_IniWrite(s_FileName, s_Section, s_KeyName, gc_sKeyWordForDeleteINI_Key)
End Function ' ==> INI_DeleteINI_Key


' ==============================================================================
' INI_FileContentsRead
'
' Read file.
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' ------------------------------------------------------------------------------
Function INI_FileContentsRead(ByVal s_FileName)
  Dim FS
  Set FS = CreateObject("Scripting.FileSystemObject")
  If (InStr(s_FileName, ":\") = 0) And (Left (s_FileName, 2) <> "\\") Then
    s_FileName = FS.GetSpecialFolder(0) & "\" & s_FileName
  End If
  On Error Resume Next

  INI_FileContentsRead = FS.OpenTextFile(s_FileName).ReadAll
End Function ' ==> INI_FileContentsRead


' ==============================================================================
' INI_FileContentsWrite
'
' Write file.
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' ------------------------------------------------------------------------------
Function INI_FileContentsWrite(ByVal s_FileName, ByVal s_Contents)
  Dim OutStream
  Dim FS

  Const ForReading = 1, ForWriting = 2, ForAppending = 8

  Set FS = CreateObject("Scripting.FileSystemObject")

  If (InStr(s_FileName, ":\") = 0) And (Left (s_FileName, 2) <> "\\") Then
    s_FileName = FS.GetSpecialFolder(0) & "\" & s_FileName
  End If

  Set OutStream = FS.OpenTextFile(s_FileName, ForWriting, True)
  OutStream.Write s_Contents
End Function ' ==> INI_FileContentsWrite


' ==============================================================================
' INI_IniRead
'
' Read from ini file.
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' ------------------------------------------------------------------------------
Function INI_IniRead(s_FileName, s_Section, s_KeyName, s_DefVal)
  Dim sINIContents, iPosSection, iPosEndSection, sContents, sValue, bFound

  sINIContents = INI_FileContentsRead(s_FileName)

  iPosSection = InStr(1, sINIContents, "[" & s_Section & "]", vbTextCompare)
  If iPosSection > 0 Then
    iPosEndSection = InStr(iPosSection, sINIContents, vbCrLf & "[")
    If iPosEndSection = 0 Then iPosEndSection = Len(sINIContents) + 1

    sContents = Mid(sINIContents, iPosSection, iPosEndSection - iPosSection)

    If InStr(1, sContents, vbCrLf & s_KeyName & "=", vbTextCompare) > 0 Then
      bFound = True
      sValue = INI_SeparateField(sContents, vbCrLf & s_KeyName & "=", vbCrLf)
    End If
  End If
  If IsEmpty(bFound) Then sValue = s_DefVal
  INI_IniRead = sValue
End Function ' ==> INI_IniRead


' ==============================================================================
' INI_IniWrite
'
' Write to ini file.
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
'
' Modified: Professort Bernd. (gc_sKeyWordForDeleteINI_Key)
' ------------------------------------------------------------------------------
Function INI_IniWrite(s_FileName, s_Section, s_KeyName, s_KeyVal)
  Dim sINIContents, iPosSection, iPosEndSection
  Dim sOldContents, sNewContents, sLine
  Dim aOldContents
  Dim bFound, bDelKey
  Dim iIdx, iCRLF_Cnt : iCRLF_Cnt = 0

  bDelKey = False

  sINIContents = INI_FileContentsRead(s_FileName)

  iPosSection = InStr(1, sINIContents, "[" & s_Section & "]", vbTextCompare)
  If iPosSection > 0 Then
    iPosEndSection = InStr(iPosSection, sINIContents, vbCrLf & "[")
    If iPosEndSection = 0 Then iPosEndSection = Len(sINIContents) + 1

    sOldContents = Mid(sINIContents, iPosSection, iPosEndSection - iPosSection)
    aOldContents = Split(sOldContents, vbCrLf)

    For Each sLine In aOldContents

      If LCase(Left(sLine, Len(s_KeyName & "="))) = LCase(s_KeyName & "=") Then
        If s_KeyVal = gc_sKeyWordForDeleteINI_Key Then
          sLine = ""
          bDelKey = True
        Else ' ---- Key plus Wert eintragen.
          sLine = s_KeyName & "=" & s_KeyVal
        End If
        bFound = True
      End If

      If bDelKey = True Then                                             ' <=
        bDelKey = False                                                  ' <=
      Else                                                               ' <=
        sNewContents = sNewContents & sLine & vbCrLf                     ' <=
      End If                                                             ' <=
    Next

    If IsEmpty(bFound) And (s_KeyVal <> gc_sKeyWordForDeleteINI_Key) Then

      Do While Right(sNewContents, 2) = vbCRLF                                  ' <=
        iCRLF_Cnt = iCRLF_Cnt + 1                                               ' <=
        sNewContents = Left(sNewContents, Len(sNewContents) - 2)                ' <=
      Loop                                                                      ' <=
      sNewContents = sNewContents & vbCRLF                                      ' <=
      sNewContents = sNewContents & s_KeyName & "=" & s_KeyVal                  '
      If iCRLF_Cnt > 1 Then                                                     ' <=
        For iIdx = 2 To iCRLF_Cnt                                               ' <=
          sNewContents = sNewContents & vbCRLF                                  ' <=
        Next                                                                    ' <=
      End If                                                                    ' <=
    Else
      sNewContents = Left(sNewContents, Len(sNewContents) - 2)
    End If

    sINIContents = Left(sINIContents, iPosSection-1) & sNewContents & Mid(sINIContents, iPosEndSection)
  Else ' If iPosSection > 0
    If (Right(sINIContents, 2) <> vbCrLf) And (Len(sINIContents) > 0) Then
      sINIContents = sINIContents & vbCrLf
    End If
    sINIContents = sINIContents & "[" & s_Section & "]" & vbCrLf & s_KeyName & "=" & s_KeyVal
  End If ' If iPosSection > 0
  INI_FileContentsWrite s_FileName, sINIContents
End Function ' ==> INI_IniWrite


' ==============================================================================
' INI_Pau3IniRead
'
' Daten aus der Pau3-Ini "PSPad4AutoIt3.ini" lesen.
'
' Autor: Professor Bernd.
'
'   Rückgabe: Wert des Keys - oder "".
' ------------------------------------------------------------------------------
Function INI_Pau3IniRead(s_Section, s_Key, s_FileNameForErrMsg, s_FuncNameForErrMsg) ': string

  Dim fso
  Dim sErrMsgTxt
  Dim sRetVal
  Dim sPSPad4AutoIt3_IniDir
  Dim sPSPad4AutoIt3_IniPath

  sPSPad4AutoIt3_IniDir = getVarValue("%PSPath%") & "PSPad4Au3\"
  sPSPad4AutoIt3_IniPath = sPSPad4AutoIt3_IniDir & "PSPad4AutoIt3.ini"

  sRetVal = ""
  sErrMsgTxt = ""
  Set fso = CreateObject("Scripting.FileSystemObject")

  If fso.FolderExists(sPSPad4AutoIt3_IniDir) = True Then
    If FileExistsEx(sPSPad4AutoIt3_IniPath) = True Then
      sRetVal = INI_IniRead(sPSPad4AutoIt3_IniPath, s_Section, s_Key, "")
    Else ' File does not exist.
      sErrMsgTxt = "File not found: " & vbCRLF & vbCRLF & """" & sPSPad4AutoIt3_IniPath & """"
    End If
  Else ' Folder does not exist.
    sErrMsgTxt = "Folder does not exist: " & vbCRLF & vbCRLF & """" & sPSPad4AutoIt3_IniDir & """"
  End If

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & s_FileNameForErrMsg & "'" & vbCRLF _
      & "Function: '" & s_FuncNameForErrMsg & "'" & vbCRLF & vbCRLF & sErrMsgTxt)
  End If

  INI_Pau3IniRead = sRetVal
End Function ' ==> INI_Pau3IniRead


' ==============================================================================
' INI_Pau3IniWrite
'
' Daten in die Pau3-Ini "PSPad4AutoIt3.ini" schreiben.
'
' Autor: Professor Bernd.
' ------------------------------------------------------------------------------
Function INI_Pau3IniWrite(s_Section, s_Key, s_KeyVal, s_FileNameForErrMsg, s_FuncNameForErrMsg)

  Dim fso
  Dim sErrMsgTxt
  Dim sPSPad4AutoIt3_IniDir
  Dim sPSPad4AutoIt3_IniPath

  sPSPad4AutoIt3_IniDir = getVarValue("%PSPath%") & "PSPad4Au3\"
  sPSPad4AutoIt3_IniPath = sPSPad4AutoIt3_IniDir & "PSPad4AutoIt3.ini"

  sErrMsgTxt = ""
  Set fso = CreateObject("Scripting.FileSystemObject")

  If fso.FolderExists(sPSPad4AutoIt3_IniDir) = True Then

    Call INI_IniWrite(sPSPad4AutoIt3_IniPath, s_Section, s_Key, s_KeyVal)

    If FileExistsEx(sPSPad4AutoIt3_IniPath) = True Then
      If (INI_IniRead(sPSPad4AutoIt3_IniPath, s_Section, s_Key, "") = s_KeyVal) Then
      Else ' Error writing the file.
        sErrMsgTxt = "Error writing the file: " & vbCRLF & vbCRLF & """" & sPSPad4AutoIt3_IniPath & """"
      End If
    Else ' File does not exist.
      sErrMsgTxt = "File could not be created: " & vbCRLF & vbCRLF & """" & sPSPad4AutoIt3_IniPath & """"
    End If
  Else ' Folder does not exist.
    sErrMsgTxt = "Folder does not exist: " & vbCRLF & vbCRLF & """" & sPSPad4AutoIt3_IniDir & """"
  End If

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & s_FileNameForErrMsg & "'" & vbCRLF _
      & "Function: '" & s_FuncNameForErrMsg & "'" & vbCRLF & vbCRLF & sErrMsgTxt)
  End If
End Function ' ==> INI_Pau3IniWrite


' ==============================================================================
' INI_SeparateField
'
' Separates one field between s_Start and s_End
'
' Autor: ' 2003 Antonin Foller, PSTRUH Software, http://www.motobit.com
' ------------------------------------------------------------------------------
Function INI_SeparateField(ByVal s_From, ByVal s_Start, ByVal s_End)
  Dim iPosB
  Dim iPosE
  iPosB = InStr(1, s_From, s_Start, 1)
  If iPosB > 0 Then
    iPosB = iPosB + Len(s_Start)
    iPosE = InStr(iPosB, s_From, s_End, 1)
    If iPosE = 0 Then iPosE = InStr(iPosB, s_From, vbCrLf, 1)
    If iPosE = 0 Then iPosE = Len(s_From) + 1
    INI_SeparateField = Mid(s_From, iPosB, iPosE - iPosB)
  End If
End Function ' ==> INI_SeparateField


' ==============================================================================
' ReadFile_AuObj
'
' Routine zum Einlesen einer Datei mithilfe eines COM-Objekts, das von einem
' AutoItObject-Server zur Verfügung gestellt wird.
'
' Autor: Professor Bernd, 2020-10-24.
' ------------------------------------------------------------------------------
Function ReadFile_AuObj(s_FileFullPath)

  Dim oAuObj
  Dim hInput
  Dim sFileText : sFileText = ""

  If AutoItObjectServer_IsRunning("") = 1 Then

    Set oAuObj = GetObject(gc_sAuObjServerIdentifier) ' the magic

    hInput = oAuObj.Call("FileOpen", s_FileFullPath, 0)
    sFileText = oAuObj.Call("FileRead", hInput)
    Call oAuObj.Call("FileClose", hInput)
  End If

  ReadFile_AuObj = sFileText
End Function ' ==> ReadFile_AuObj


' ==============================================================================
' ReloadOrReplaceActiveFile_AuObj
'
' Routine zum Neuladen oder Ersetzen der Datei des aktiven Editors, wobei
' vorhandene Undo's erhalten bleiben.
'
' Mit Undo's sind die Änderungen im Text gemeint, die der User mit Ctrl+Z
' rückgängig machen kann. Mit dieser Funktion bleiben nicht nur die
' vorhandenen Undo's erhalten, sondern auch das Neuladen der Datei kann
' rückgängig gemacht werden.
'
' Parameter:
'
'   s_PathToReplacementFile: Pfad zur Datei, mit deren Text der Text des
'                            aktuellen Editors überschrieben werden soll.
'                            Wird ein leerer Sting ("") übergeben wird mit dem
'                            (evtl. extern geänderten) Text aus der aktuellen
'                            Datei überschrieben.
'
' Autor: Professor Bernd, 2020-10-30.
' ------------------------------------------------------------------------------
Function ReloadOrReplaceActiveFile_AuObj(s_PathToReplacementFile)

  Dim obj
  Dim sAu3Path, sFileText
  Dim iCarteX_Old, iCarteY_Old, iTopLineIdx_Old, iRetVal

  Set obj = NewEditor()
  obj.assignActiveEditor()

  iCarteX_Old = AutoIt3_CallTip.CaretX_TTS(obj) ' Virtuelles CaretX berücksichtigt Tabs im Text.
  iCarteY_Old = obj.caretY()
  iTopLineIdx_Old = obj.topLineIndex() ' Index der ersten sichtbaren Zeile im Editor.

  If s_PathToReplacementFile <> "" Then
    sAu3Path = s_PathToReplacementFile
  Else
    sAu3Path = getVarValue("%FullFileName%")
  End If

  sFileText = ReadFile_AuObj(sAu3Path)

  If sFileText <> "" Then
    obj.Text(sFileText)

    obj.saveFile() ' Speichert OHNE Nachfrage, auch wenn die Datei ReadOnly ist!

    Call obj.setCaretPos(iCarteX_Old, iCarteY_Old)
    obj.topLineIndex(iTopLineIdx_Old)

    iRetVal = 1
  Else

    iRetVal = 0
  End If

  ReloadOrReplaceActiveFile_AuObj = iRetVal
End Function ' ==> ReloadOrReplaceActiveFile_AuObj


' ==============================================================================
' Run_OutputToLogWin
'
' Routine zum Ausführen eines Programms, Auffangen des Outputs und  Anzeigen
' des Outputs im LogWin.
'
' Autor: Professor Bernd, 2021-03-13.
'
'   Rückgabe: Die Rückgabe des auszuführenden Programms.
' ------------------------------------------------------------------------------
Function Run_OutputToLogWin(s_CommandLine)

  Dim oWshShell, fso, oFile
  Dim sOutput, sTmpFilePath
  Dim aOutput
  Dim iRet, iCnt

  logClear() ' Mit leerem LogWin anfangen.

  Set oWshShell = CreateObject("WScript.Shell")

  sTmpFilePath = oWshShell.ExpandEnvironmentStrings("%Temp%") & "\TmpFileForPau3_" _
    & Replace(Date, ".", "-") & "_" & Replace(Time, ":", "-") & ".txt"

  iRet = oWshShell.Run("cmd.exe /C " & """" & s_CommandLine & " > " & """" & sTmpFilePath & """" & """", 0, true)

  Set fso  = CreateObject("Scripting.FileSystemObject")
  Set oFile = fso.OpenTextFile(sTmpFilePath, 1) ' 1 = ForReading
  sOutput = oFile.ReadAll
  oFile.Close
  fso.DeleteFile sTmpFilePath

  If InStr(sOutput, vbCRLF) > 0 Then
    aOutput = Split(sOutput, vbCRLF)
  Else
    aOutput = Split(sOutput, vbLF)
  End If

  For iCnt = 0 To UBound(aOutput) - 1
    logAddLine(aOutput(iCnt))
  Next

  Run_OutputToLogWin = iRet
End Function ' ==> Run_OutputToLogWin


' ==============================================================================
' Splasher_Close
'
' Beendet Splasher mithilfe des Fenster-Titels.
'
' Rückgabe:         Ergebnis von "WinClose" als String.
'
'   Erfolg:         1.
'   Fehler:         0, wenn das Fenster nicht geschlossen werden konnte.
'
' Autor: Professor Bernd, 2020-10-30.
' ------------------------------------------------------------------------------
Function Splasher_Close()

  Dim sAu3SingleLine
  Dim iCnt, iMilliSeks

  iMilliSeks = 4000

  For iCnt = 1 To Int(iMilliSeks / 100)

    If (ExecuteAu3Line("WinExists('" & gc_sSplasherWinTitle & "')", "/WaitForResult") = 1) Then
      Exit For
    End If
    Sleep 100
  Next

  sAu3SingleLine = "WinClose('" & gc_sSplasherWinTitle & "')"

  Call ExecuteAu3Line(sAu3SingleLine, "")
End Function ' ==> Splasher_Close


' ==============================================================================
' Splasher_Run
'
' Routine zum Starten von Splasher und Übergeben von Start-Parametern.
'
' Parameter:
'
'   s_Line_1:       Erste Zeile im Splash-Fenster.
'   s_Line_2:       Zweite Zeile im Splash-Fenster.
'   i_StartDelay:   Start-Verzögerung in ms.
'
' Aufruf-Beispiel:  Call Splasher_Run("Tidy", "is running!", 800)
'
' Autor: Professor Bernd, 2020-10-30.
'
'
' Beschreibung Splasher
'
'   Splasher zeigt einen kleinen Splash-Screen mit 2 Zeilen Text an, mit dem
'   ein User z.B. erkennen kann, dass ein Vorgang läuft. Der Text kann per
'   Start-Parameter übergeben werden.
'
'   Splasher kann z.B. von einem anderen Programm gestartet werden, das dann
'   Aufgaben erfüllt, auf die gewartet wird. Wenn die Aufgaben beendet sind,
'   kann Splasher mithilfe des Fenstertiteles beendet werden.
'
'   Start-Parameter, die von Splasher ausgewertet werden:
'
'     Param 1: Text für Zeile 1
'     Param 2: Text für Zeile 2
'     Param 3: Fenstertitel mit dem das aufrufende Programm das Splasher-Fenster
'              identifizieren kann, um es zu schließen (WinGetHandle, WinClose).
'     Param 4: Start-Verzögerung in ms, bevor Splasher angezeigt wird.
' ------------------------------------------------------------------------------
Function Splasher_Run(s_Line_1, s_Line_2, i_StartDelay) ' i_StartDelay in ms.

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sSplasherAu3Path
  Dim sCommandLine, sErrMsgTxt

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sSplasherAu3Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\misc\Splasher.au3"

  If FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If FileExistsEx(sSplasherAu3Path) = False Then _
     sErrMsgTxt = sErrMsgTxt & """" & sSplasherAu3Path & """" & vbCRLF
  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'Splasher_Run'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
    Exit Function
  End If

  If (ExecuteAu3Line("WinExists('" & gc_sSplasherWinTitle & "')", "/WaitForResult") = 0) Then ' Wichtig: "If Not..." funktioniert NICHT!

    Set oWshShell = WScript.CreateObject("WScript.Shell")

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sSplasherAu3Path & """" _
      & " " & """" & s_Line_1 & """" _
      & " " & """" & s_Line_2 & """" _
      & " " & """" & gc_sSplasherWinTitle & """" _
      & " " & i_StartDelay

    Call oWshShell.Run(sCommandLine)
  End If

End Function ' ==> Splasher_Run


' ==============================================================================
' StringBetween
'
' Usage: sText = StringBetween("123 get this 321", "123", "321")
'
' Author: stum
' Quelle: http://www.vbforums.com/showthread.php?739829-Find-text-between-two-characters&p=4536811&viewfull=1#post4536811
' ------------------------------------------------------------------------------
' Function StringBetween(ByRef sSearch, ByRef sStart, ByRef sStop)
Function StringBetween(sSearch, sStart, sStop) ': string
  Dim lSearch
  Dim lTemp
  lSearch = 1
  lSearch = InStr(lSearch, sSearch, sStart)
  If lSearch > 0 Then
    lSearch = lSearch + Len(sStart)
    lTemp = InStr(lSearch, sSearch, sStop)
    If lTemp > lSearch Then
      StringBetween = Mid(sSearch, lSearch, lTemp - lSearch)
    End If
  Else
    StringBetween = ""
  End If
End Function ' ==> StringBetween


' ==============================================================================
' WaitForMilliSecs
'
' Wait-Routine, die (hoffentlich) die Auführung bei längerem Warten nicht
' einfriert, so wie das z.B. bei "Sleep 2000" der Fall ist. 2020-03-13.
'
' ------------------------------------------------------------------------------
Sub WaitForMilliSecs(i_MilliSeks)

  Call AutoIt3___Base.ExecuteAu3Line("Sleep(" & i_MilliSeks & ")", "/WaitForResult")

End Sub ' ==> WaitForMilliSecs


' ==============================================================================
' Init
'
' name "Init" is required, its called automatically during initialization to create menu items
'
' ----------------------------------------------------------------------------
' Wichtig: Derzeit wird dieses Modul "AutoIt3___Base.vbs" als erstes, also
'          vor allen anderen Modulen (VBScripts) gestartet. Nur die JScripts
'          werden noch vor "AutoIt3___Base.vbs" gestartet. 2020-11-11.
'
'          "AutoIt3___Base.vbs" wird vorrangig dazu benutzt, um hier alle
'          Funktionen zu platzieren, die von mehreren VBScripts benutzt werden.
'          Da dieses Modul als erstes gestartet wird, stehen die Funktionen
'          dieses Moduls den anderen Modulen schon beim Laden zur Verfügung.
'
'          Grundsätzlich sollten hier im Init so wenig wie möglich Funktionen
'          gestartet werden. Außnahmen sind Funktionen, bei denen es wichtig
'          ist, dass sie möglichst früh gestartet werden.
' ------------------------------------------------------------------------------
Sub Init()

  AutoItObjectServer_Run()

End Sub ' ==> Init
