Option Explicit

' = Info ===========================================================================================
' Title           : zzzzAutoIt3_Cleaner.vbs
' --------------------------------------------------------------------------------------------------
'
Const module_name = "zzzzAutoIt3_Cleaner"
Const module_ver  = "0.0.15"
Const module_desc = "Routines that are executed at the end of the VBScript chain (to clean up)."
' Date            : 2021-04-30
' Author(s)       : Professor Bernd
'
' Requirements    :
' Language        :
'
' ==================================================================================================


' global constants (gc_...)
' Const gc_sExample = "Hello world!"


' WICHTIG! Globale Variablen können NICHT im Init() initialisiert werden!
' =======  (Aber sie können hier im globalen Bereich initialisiert werden.
'           Der globale Bereich wird noch VOR der Init() Routine ausgeführt!)
'
' global variables
' Dim g_bExample


' ==============================================================================
' RemoveJScriptVBScript_StandardScripts
'
' Entfernen der JScript- und VBScript-Dateien, die von PSPad als Standard
' mitgeliefert werden. Diese Dateien können z.B. nach einem Update durch
' den User wieder auftauchen und Menü-Punkte in das "AutoIt3" Hauptmenü
' eintragen, die nicht AutoIt-relevant sind.
'
' Durch die Routine unten "ReplaceMnuItmsInAllLangFiles_ToAutoIt3()" wird der
' Hauptmenü-Punkt "Scripts" zu "AutoIt3" umbenannt. Im "AutoIt3" Menü sollen
' möglichst nur noch AutoIt-relevante Menü-Punkte gezeigt werden. 2020-11-10.
'
' Wenn die Dateien gelöscht wurden, werden die Änderungen erst sichtbar nach
' einem PSPad-Neustart oder nach der Ausführung der ReloadPSPadScripts-Routine
' per AutoIt-SingleLine.
'
' Rückgabewert  0: Falls KEINE Dateien gelöscht wurden.
'               1: Falls Dateien gelöscht wurden.
' ------------------------------------------------------------------------------
Function RemoveJScriptVBScript_StandardScripts()

  Dim fso
  Dim iIdx
  Dim sAu3SingleLine  ' Befehlszeile für AutoIt-SingleLine.
  Dim sScriptDir      ' "Script"-Ordner im PSPad-Ordner. (Darin sind "JScript\" u "VBScript\").
  Dim aFilePaths(9)   ' Array mit 10 Elementen deklarieren.
  Dim iRetVal : iRetVal = 0

  sScriptDir = getVarValue("%PSPath%") & "Script"

  aFilePaths(0) = sScriptDir & "\JScript\Align_Columns.js"
  aFilePaths(1) = sScriptDir & "\JScript\any_tag_close.js"
  aFilePaths(2) = sScriptDir & "\JScript\FilterLines.js"
  aFilePaths(3) = sScriptDir & "\JScript\ToDoList.js"
  aFilePaths(4) = "" ' Reserve
  aFilePaths(5) = sScriptDir & "\VBScript\ListTransform.vbs"
  aFilePaths(6) = sScriptDir & "\VBScript\PrinAll.vbs"
  aFilePaths(7) = sScriptDir & "\VBScript\VBIndent.vbs"
  aFilePaths(8) = "" ' Reserve
  aFilePaths(9) = "" ' Reserve

  Set fso = CreateObject("Scripting.FileSystemObject")

  For iIdx = 0 To UBound(aFilePaths)
    If AutoIt3___Base.FileExistsEx(aFilePaths(iIdx)) Then
      fso.DeleteFile(aFilePaths(iIdx))
      iRetVal = 1
    End If
  Next

  If iRetVal = 1 Then

    addMenuItem "Reload scripts", "", "ReloadPSPadScripts", "Shift+F4"

    sAu3SingleLine = "ControlSend('[ACTIVE]', '', 0, '+{F4}')"

    Call AutoIt3___Base.ExecuteAu3Line(sAu3SingleLine, "")
  End If

  RemoveJScriptVBScript_StandardScripts = iRetVal ' Returns 1 if files were deleted, 0 otherwise.
End Function ' ==> RemoveJScriptVBScript_StandardScripts


' ==============================================================================
' ReplaceMnuItmsInAllLangFiles_ToAutoIt3
'
' Routine zum Ersetzen des Textes im Hauptmenü-Punkt "Scripts" durch "AutoIt3"
' und des Menü-Punkts "Recompile scripts" durch "[Recompile internal VBScripts]".
'
' Dazu werden in allen Sprach-Dateien ("...\PSPad4AutoIt3\Language\*.ini") für
' die PSPad-Oberfläche bestimmte Original-Werte durch gewünschte Werte ersetzt.
'
' Die originalen und die gewünschten Schlüssel-Werte:
'
'   [Main Menu]
'   Original:  mScripts_Caption=Sc&ripts
'   AutoIt3:   mScripts_Caption=AutoIt3
'
'   [Action List]
'   Original:  aScriptsRecompile_Caption=Recompile scripts
'   AutoIt3:   aScriptsRecompile_Caption=[Recompile internal VBScripts]
'
' Hint: Die gewünschten Werte, die in die Sprach-Dateien geschrieben werden,
'       sind für alle GUI-Sprachen in englisch!
'
' Änderungen im Hauptmenü werden erst sichtbar nach einem PSPad-Neustart, da
' es keinen VBScript-Befehl zum Aktualisieren des Hauptmenüs gibt. Deshalb
' werde ich vorerst als Workaround dem User eine PSPad-Neustart-Mitteilung
' machen und PSPad per Code beenden.
'
' Das Ändern ALLER PSPad-Sprach-Dateien dauert ein paar Sekunden. Damit der User
' die Mitteilung zum PSPad-Neustart nicht erst erhält, wenn er vielleicht schon
' angefangen hat zu arbeiten, wird das Ersetzen in 2 Schritten durchgeführt. Im
' ersten Schritt wird nur in der AKTUELLEN Sprach-Datei ersetzt, ein Eintrag in
' die Pau3-Ini geschrieben, die Neustart-Mitteilung gezeigt und PSPad
' beendet. Beim nächsten Start von PSPad wird der Eintrag aus der Pau3-Ini
' gelesen und damit der zweite Schritt ausgelöst, in dem in ALLEN Sprach-Dateien
' ersetzt wird (silent).
'
' Rückgabewert  0:  PSPad-Close wurde NICHT veranlasst. (Die gewünschten Werte
'                     in der aktuellen PSPad-Sprach-Datei waren vorhanden.)
'               1:  PSPad-Close wurde veranlasst. (Die gewünschten Werte in der
'                     aktuellen PSPad-Sprach-Datei wurden neu geschrieben.)
' ------------------------------------------------------------------------------
Function ReplaceMnuItmsInAllLangFiles_ToAutoIt3()

  Dim fso, oFolder, oFile, col_oFiles
  Dim sFolderPath, sPSPadLanguage, sLangIniPath
  Dim sMenuCaption, sMenuRecompileScriptsText, sCaption_AutoIt, sRecompileScriptsText_AutoIt
  Dim sTitle, sMsg, sAu3SingleLine
  Dim iRetVal : iRetVal = 0

  sFolderPath = getVarValue("%PSPath%") & "Language\"

  sCaption_AutoIt = "AutoIt&3"                                    ' Hauptmenü-Punkt.
  sRecompileScriptsText_AutoIt = "[Recompile internal VBScripts]" ' Menü-Punkt.

  sPSPadLanguage = AutoIt3___Base.INI_IniRead(getVarValue("%PSPath%") _
    & "PSPad.ini", "Config", "Localization", "ENGLISH")

  sLangIniPath = getVarValue("%PSPath%") & "Language\" & sPSPadLanguage & ".ini"

  sMenuCaption = AutoIt3___Base.INI_IniRead(sLangIniPath, _
    "Main Menu", "mScripts_Caption", "")

  sMenuRecompileScriptsText = AutoIt3___Base.INI_IniRead(sLangIniPath, _
    "Action List", "aScriptsRecompile_Caption", "")

  If (sMenuCaption <> sCaption_AutoIt) _
    Or (sMenuRecompileScriptsText <> sRecompileScriptsText_AutoIt) Then

    Call AutoIt3___Base.INI_IniWrite(sLangIniPath, _
      "Main Menu", "mScripts_Caption", sCaption_AutoIt)

    Call AutoIt3___Base.INI_IniWrite(sLangIniPath, _
      "Action List", "aScriptsRecompile_Caption", sRecompileScriptsText_AutoIt)

    Call AutoIt3___Base.INI_Pau3IniWrite("General", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3", "1", _
      module_name & ".vbs", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3()")

    sTitle = "'PSPad script files updated'"
    sMsg = "'Internal script files have been updated.' & @CRLF & @CRLF" _
      & " & 'PSPad has been closed. Please restart PSPad.'"

    sAu3SingleLine = "WinClose('[TITLE:' & WinGetTitle('') & '; CLASS:TfPSPad]')" _
      & " & MsgBox(4096 + 64, " & sTitle & ", " & sMsg & ")" ' Aktives PSPad-Fenster schließen und MsgBox anzeigen.

    Call AutoIt3___Base.ExecuteAu3Line(sAu3SingleLine, "")

    iRetVal = 1 ' PSPad-Close wurde veranlasst.
  End If

  If (iRetVal = 0) _
      And AutoIt3___Base.INI_Pau3IniRead("General", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3", _
      module_name & ".vbs", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3()") = "1" Then
    Call AutoIt3___Base.INI_Pau3IniWrite("General", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3", "0", _
      module_name & ".vbs", "ReplaceMnuItmsInAllLangFiles_ToAutoIt3()")                        '-- Beachte: string!

    Set fso = CreateObject("Scripting.FileSystemObject")
    Set oFolder = fso.GetFolder(sFolderPath)
    Set col_oFiles = oFolder.Files

    For Each oFile in col_oFiles

      If LCase(AutoIt3___Base.ExtractFileExtension(oFile.Name)) = ".ini" Then

        Call AutoIt3___Base.INI_IniWrite(oFile.Path, _
          "Main Menu", "mScripts_Caption", sCaption_AutoIt)

        Call AutoIt3___Base.INI_IniWrite(oFile.Path, _
          "Action List", "aScriptsRecompile_Caption", sRecompileScriptsText_AutoIt)

      End If
    Next

  End If ' If (iRetVal = 0) And AutoIt3___Base.INI_Pau3IniRead(...

  ReplaceMnuItmsInAllLangFiles_ToAutoIt3 = iRetVal
End Function ' ==> ReplaceMnuItmsInAllLangFiles_ToAutoIt3


' ==============================================================================
' IniRefresher_Run
'
' Routine zum Auffrischen einer User-Ini mithilfe einer Global-Ini. Zur Zeit
' werden die Pau3-Ini und die eigene Tidy-Ini im Pau3-Ordner refreshed. 2021-04-30.
'
' Beispiel: Global-Ini: "<PSPad-Ordner>\PSPad4Au3\Bak\Global\Pau3Global.ini"
'           User-Ini:   "<PSPad-Ordner>\PSPad4Au3\PSPad4AutoIt3.ini"
'
' Auffrischen:
'
' - Falls die User-Ini existiert, wird der Inhalt der Global-Ini in die User-Ini
'   geschrieben, wobei die Werte der User-Ini erhalten bleiben.
'
' - Falls die User-Ini NICHT existiert, wird sie erstellt und der Inhalt der
'   schreibgeschützen Global-Ini in die nicht-schreibgeschützte User-Ini kopiert.
' ------------------------------------------------------------------------------
Sub IniRefresher_Run()

  Dim oWshShell
  Dim sAutoIt3ExePath_Own, sAu3Path
  Dim sCommandLine, sErrMsgTxt
  Dim iRetVal

  Set oWshShell = WScript.CreateObject("WScript.Shell")

  sAutoIt3ExePath_Own = getVarValue("%PSPath%") & "PSPad4Au3\AutoIt3\AutoIt3.exe"

  sAu3Path = getVarValue("%PSPath%") & "PSPad4Au3\Au3 scripts\IniRefresher.au3"

  If AutoIt3___Base.FileExistsEx(sAutoIt3ExePath_Own) = False Then _
    sErrMsgTxt = """" & sAutoIt3ExePath_Own & """" & vbCRLF & vbCRLF
  If AutoIt3___Base.FileExistsEx(sAu3Path) = False Then _
    sErrMsgTxt = sErrMsgTxt & """" & sAu3Path & """" & vbCRLF

  If sErrMsgTxt <> "" Then
    Echo("Message from " & "'" & module_name & ".vbs" & "'" & vbCRLF _
      & "Function: 'IniRefresher_Run'" & vbCRLF & vbCRLF _
      & "Files not found: " & vbCRLF & vbCRLF & sErrMsgTxt)
  Else ' Dateien wurden gefunden.

    sCommandLine = """" & sAutoIt3ExePath_Own & """" _
      & " " & """" & sAu3Path & """"

    Call oWshShell.Run(sCommandLine) ' IniRefresher ausführen OHNE warten bis beendet.
  End If
End Sub ' ==> IniRefresher_Run


' ==============================================================================
' ReloadPSPadScripts
'
' Routine zum Rekompilieren der internen PSPad-Scripts.
'
' Für diese Routine wird temporär einen Menü-Punkt erstellt, damit ein Shortcut
' dafür zugewiesen werden kann. Mit dem Shortcut kann ein externes Programm
' (AutoIt-SingleLine) diese Routine asynchron aufrufen, während hier die
' Init-Routine komplett durchläuft.
' ------------------------------------------------------------------------------
Sub ReloadPSPadScripts()
  runPSPadAction("aScriptsRecompile") ' <== ACHTUNG! Endlos-Schleife, wenn OHNE Bedingung in Init()!
End Sub ' ==> ReloadPSPadScripts


' ==============================================================================
' Init
'
' name "Init" is required, its called automatically during initialization to create menu items
' ------------------------------------------------------------------------------
Sub Init()

  Dim iRetVal

  iRetVal = ReplaceMnuItmsInAllLangFiles_ToAutoIt3()

  IniRefresher_Run()

  iRetVal = RemoveJScriptVBScript_StandardScripts()
End Sub ' ==> Init
